@@ -1,12 +1,11 @@
-#!/usr/bin/env python2
-# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
+#!/usr/bin/env python
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
 import os, time
-from base64 import b64decode
 from datetime import date
 
 from calibre import prints, guess_type, isbytestring, fsync
@@ -16,15 +15,18 @@ from calibre.constants import DEBUG, preferred_encoding
 from calibre.ebooks.chardet import xml_to_unicode
 from calibre.ebooks.metadata import authors_to_string, title_sort, \
                                     authors_to_sort_string
+from polyglot.binary import from_base64_bytes
 
 '''
-cahceExt.xml
+cacheExt.xml
 
 Periodical identifier sample from a PRS-650:
 
 <?xml version="1.0" encoding="UTF-8"?>
 <cacheExt xmlns="http://www.sony.com/xmlns/product/prs/device/1">
-    <text conformsTo="http://xmlns.sony.net/e-book/prs/periodicals/1.0/newspaper/1.0" periodicalName="The Atlantic" description="Current affairs and politics focussed on the US" publicationDate="Tue, 19 Oct 2010 00:00:00 GMT" path="database/media/books/calibre/Atlantic [Mon, 18 Oct 2010], The - calibre_1701.epub">
+    <text conformsTo="http://xmlns.sony.net/e-book/prs/periodicals/1.0/newspaper/1.0" periodicalName="The Atlantic"
+    description="Current affairs and politics focused on the US" publicationDate="Tue, 19 Oct 2010 00:00:00 GMT"
+    path="database/media/books/calibre/Atlantic [Mon, 18 Oct 2010], The - calibre_1701.epub">
         <thumbnail width="167" height="217">main_thumbnail.jpg</thumbnail>
     </text>
 </cacheExt>
@@ -32,13 +34,13 @@ Periodical identifier sample from a PRS-650:
 '''
 
 # Utility functions {{{
-EMPTY_CARD_CACHE = '''\
+EMPTY_CARD_CACHE = b'''\
 <?xml version="1.0" encoding="UTF-8"?>
 <cache xmlns="http://www.kinoma.com/FskCache/1">
 </cache>
 '''
 
-EMPTY_EXT_CACHE = '''\
+EMPTY_EXT_CACHE = b'''\
 <?xml version="1.0" encoding="UTF-8"?>
 <cacheExt xmlns="http://www.sony.com/xmlns/product/prs/device/1">
 </cacheExt>
@@ -58,6 +60,7 @@ MONTH_MAP = dict(Jan=1, Feb=2, Mar=3, Apr=4, May=5, Jun=6, Jul=7, Aug=8, Sep=9,
 INVERSE_DAY_MAP = dict(zip(DAY_MAP.values(), DAY_MAP.keys()))
 INVERSE_MONTH_MAP = dict(zip(MONTH_MAP.values(), MONTH_MAP.keys()))
 
+
 def strptime(src):
     src = src.strip()
     src = src.split()
@@ -65,6 +68,7 @@ def strptime(src):
     src[2] = str(MONTH_MAP[src[2]])
     return time.strptime(' '.join(src), '%w, %d %m %Y %H:%M:%S %Z')
 
+
 def strftime(epoch, zone=time.localtime):
     try:
         src = time.strftime("%w, %d %m %Y %H:%M:%S GMT", zone(epoch)).split()
@@ -75,16 +79,18 @@ def strftime(epoch, zone=time.localtime):
     src[2] = INVERSE_MONTH_MAP[int(src[2])]
     return ' '.join(src)
 
+
 def uuid():
     from uuid import uuid4
     return str(uuid4()).replace('-', '', 1).upper()
 
 # }}}
 
-class XMLCache(object):
+
+class XMLCache:
 
     def __init__(self, paths, ext_paths, prefixes, use_author_sort):
-        from lxml import etree
+        from calibre.utils.xml_parse import safe_xml_fromstring
 
         if DEBUG:
             debug_print('Building XMLCache...', paths)
@@ -93,25 +99,23 @@ class XMLCache(object):
         self.use_author_sort = use_author_sort
 
         # Parse XML files {{{
-        parser = etree.XMLParser(recover=True)
         self.roots = {}
         for source_id, path in paths.items():
             if source_id == 0:
                 if not os.path.exists(path):
                     raise DeviceError(('The SONY XML cache %r does not exist. Try'
                         ' disconnecting and reconnecting your reader.')%repr(path))
-                with open(path, 'rb') as f:
+                with lopen(path, 'rb') as f:
                     raw = f.read()
             else:
                 raw = EMPTY_CARD_CACHE
                 if os.access(path, os.R_OK):
-                    with open(path, 'rb') as f:
+                    with lopen(path, 'rb') as f:
                         raw = f.read()
 
-            self.roots[source_id] = etree.fromstring(xml_to_unicode(
-                        raw, strip_encoding_pats=True, assume_utf8=True,
-                        verbose=DEBUG)[0],
-                        parser=parser)
+            self.roots[source_id] = safe_xml_fromstring(
+                xml_to_unicode(raw, strip_encoding_pats=True, assume_utf8=True, verbose=DEBUG)[0]
+            )
             if self.roots[source_id] is None:
                 raise Exception(('The SONY database at %r is corrupted. Try '
                         ' disconnecting and reconnecting your reader.')%path)
@@ -120,18 +124,17 @@ class XMLCache(object):
         for source_id, path in ext_paths.items():
             if not os.path.exists(path):
                 try:
-                    with open(path, 'wb') as f:
+                    with lopen(path, 'wb') as f:
                         f.write(EMPTY_EXT_CACHE)
                         fsync(f)
                 except:
                     pass
             if os.access(path, os.W_OK):
                 try:
-                    with open(path, 'rb') as f:
-                        self.ext_roots[source_id] = etree.fromstring(
-                                xml_to_unicode(f.read(),
-                                    strip_encoding_pats=True, assume_utf8=True,
-                                    verbose=DEBUG)[0], parser=parser)
+                    with lopen(path, 'rb') as f:
+                        self.ext_roots[source_id] = safe_xml_fromstring(
+                            xml_to_unicode(f.read(), strip_encoding_pats=True, assume_utf8=True, verbose=DEBUG)[0]
+                        )
                         self.ext_paths[source_id] = path
                 except:
                     pass
@@ -150,12 +153,11 @@ class XMLCache(object):
         self.detect_namespaces()
         debug_print('Done building XMLCache...')
 
-
     # Playlist management {{{
     def purge_broken_playlist_items(self, root):
         id_map = self.build_id_map(root)
         for pl in root.xpath('//*[local-name()="playlist"]'):
-            seen = set([])
+            seen = set()
             for item in list(pl):
                 id_ = item.get('id', None)
                 if id_ is None or id_ in seen or id_map.get(id_, None) is None:
@@ -185,7 +187,7 @@ class XMLCache(object):
 
     def ensure_unique_playlist_titles(self):
         for i, root in self.record_roots.items():
-            seen = set([])
+            seen = set()
             for playlist in root.xpath('//*[local-name()="playlist"]'):
                 title = playlist.get('title', None)
                 if title is None:
@@ -275,7 +277,7 @@ class XMLCache(object):
         return ans
     # }}}
 
-    def fix_ids(self): # {{{
+    def fix_ids(self):  # {{{
         debug_print('Running fix_ids()')
 
         def ensure_numeric_ids(root):
@@ -316,7 +318,7 @@ class XMLCache(object):
             # Only rebase ids of nodes that are immediate children of the
             # record root (that way playlist/itemnodes are unaffected
             items = root.xpath('child::*[@id]')
-            items.sort(cmp=lambda x,y:cmp(int(x.get('id')), int(y.get('id'))))
+            items.sort(key=lambda x: int(x.get('id')))
             idmap = {}
             for i, item in enumerate(items):
                 old = int(item.get('id'))
@@ -375,8 +377,8 @@ class XMLCache(object):
                             'descendant::*[local-name()="png"]'):
                         if img.text:
                             try:
-                                raw = b64decode(img.text.strip())
-                            except:
+                                raw = from_base64_bytes(img.text.strip())
+                            except Exception:
                                 continue
                             book.thumbnail = raw
                             break
@@ -485,7 +487,6 @@ class XMLCache(object):
         except:
             pass
 
-
     def rebuild_collections(self, booklist, bl_index):
         if bl_index not in self.record_roots:
             return
@@ -574,7 +575,7 @@ class XMLCache(object):
 
     def create_ext_text_record(self, root, bl_id, lpath, thumbnail):
         namespace = root.nsmap[None]
-        attrib = { 'path': lpath }
+        attrib = {'path': lpath}
         ans = root.makeelement('{%s}text'%namespace, attrib=attrib,
                 nsmap=root.nsmap)
         ans.tail = '\n'
@@ -593,7 +594,6 @@ class XMLCache(object):
             t.tail = '\n\t'
         return ans
 
-
     def update_text_record(self, record, book, path, bl_index,
                            gtz_count, ltz_count, use_tz_var):
         '''
@@ -625,7 +625,7 @@ class XMLCache(object):
         def clean(x):
             if isbytestring(x):
                 x = x.decode(preferred_encoding, 'replace')
-            x.replace(u'\0', '')
+            x.replace('\0', '')
             return x
 
         def record_set(k, v):
@@ -635,14 +635,14 @@ class XMLCache(object):
                 # v is not suitable for XML, ignore
                 pass
 
-        if not getattr(book, '_new_book', False): # book is not new
+        if not getattr(book, '_new_book', False):  # book is not new
             if record.get('tz', None) is not None:
                 use_tz_var = True
             if strftime(timestamp, zone=time.gmtime) == rec_date:
                 gtz_count += 1
             elif strftime(timestamp, zone=time.localtime) == rec_date:
                 ltz_count += 1
-        else: # book is new. Set the time using the current votes
+        else:  # book is new. Set the time using the current votes
             if use_tz_var:
                 tz = time.localtime
                 record.set('tz', '0')
@@ -702,8 +702,8 @@ class XMLCache(object):
                     child.text = '\n'+'\t'*(level+1)
                     for gc in child:
                         gc.tail = '\n'+'\t'*(level+1)
-                    child.iterchildren(reversed=True).next().tail = '\n'+'\t'*level
-            root.iterchildren(reversed=True).next().tail = '\n'+'\t'*(level-1)
+                    next(child.iterchildren(reversed=True)).tail = '\n'+'\t'*level
+            next(root.iterchildren(reversed=True)).tail = '\n'+'\t'*(level-1)
 
     def move_playlists_to_bottom(self):
         for root in self.record_roots.values():
@@ -714,7 +714,6 @@ class XMLCache(object):
             for pl in seen:
                 root.append(pl)
 
-
     def write(self):
         from lxml import etree
 
@@ -723,9 +722,9 @@ class XMLCache(object):
             self.cleanup_whitespace(i)
             raw = etree.tostring(self.roots[i], encoding='UTF-8',
                     xml_declaration=True)
-            raw = raw.replace("<?xml version='1.0' encoding='UTF-8'?>",
-                    '<?xml version="1.0" encoding="UTF-8"?>')
-            with open(path, 'wb') as f:
+            raw = raw.replace(b"<?xml version='1.0' encoding='UTF-8'?>",
+                    b'<?xml version="1.0" encoding="UTF-8"?>')
+            with lopen(path, 'wb') as f:
                 f.write(raw)
                 fsync(f)
 
@@ -735,9 +734,9 @@ class XMLCache(object):
                     xml_declaration=True)
             except:
                 continue
-            raw = raw.replace("<?xml version='1.0' encoding='UTF-8'?>",
-                    '<?xml version="1.0" encoding="UTF-8"?>')
-            with open(path, 'wb') as f:
+            raw = raw.replace(b"<?xml version='1.0' encoding='UTF-8'?>",
+                    b'<?xml version="1.0" encoding="UTF-8"?>')
+            with lopen(path, 'wb') as f:
                 f.write(raw)
                 fsync(f)
 
@@ -758,7 +757,7 @@ class XMLCache(object):
         return m
 
     def book_by_lpath(self, lpath, root):
-        matches = root.xpath(u'//*[local-name()="text" and @path="%s"]'%lpath)
+        matches = root.xpath('//*[local-name()="text" and @path="%s"]'%lpath)
         if matches:
             return matches[0]
 
@@ -797,4 +796,3 @@ class XMLCache(object):
                 self.namespaces[i] = ns
 
     # }}}
-
