@@ -1,11 +1,8 @@
-#!/usr/bin/env python2
-# vim:fileencoding=utf-8
+#!/usr/bin/env python
 # License: GPLv3 Copyright: 2016, Kovid Goyal <kovid at kovidgoyal.net>
 
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
 
-import os, string, _winreg as winreg, re, sys
+import os, string, re, sys, errno
 from collections import namedtuple, defaultdict
 from operator import itemgetter
 from ctypes import (
@@ -15,14 +12,21 @@ from ctypes import (
 )
 from ctypes.wintypes import DWORD, WORD, ULONG, LPCWSTR, HWND, BOOL, LPWSTR, UINT, BYTE, HANDLE, USHORT
 from pprint import pprint, pformat
-from future_builtins import map
+from polyglot.builtins import iteritems, itervalues
 
 from calibre import prints, as_unicode
 
 is64bit = sys.maxsize > (1 << 32)
 
+try:
+    import winreg
+except ImportError:
+    import _winreg as winreg
+
+
 # Data and function type definitions {{{
 
+
 class GUID(Structure):
     _fields_ = [
         ("data1", DWORD),
@@ -44,7 +48,7 @@ class GUID(Structure):
         self.data4[7] = b8
 
     def __str__(self):
-        return "{%08x-%04x-%04x-%s-%s}" % (
+        return "{{{:08x}-{:04x}-{:04x}-{}-{}}}".format(
             self.data1,
             self.data2,
             self.data3,
@@ -52,6 +56,7 @@ class GUID(Structure):
             ''.join(["%02x" % d for d in self.data4[2:]]),
         )
 
+
 CONFIGRET = DWORD
 DEVINST = DWORD
 LPDWORD = POINTER(DWORD)
@@ -60,11 +65,17 @@ REG_QWORD = 11
 IOCTL_STORAGE_MEDIA_REMOVAL = 0x2D4804
 IOCTL_STORAGE_EJECT_MEDIA = 0x2D4808
 IOCTL_STORAGE_GET_DEVICE_NUMBER = 0x2D1080
+
+
 def CTL_CODE(DeviceType, Function, Method, Access):
     return (DeviceType << 16) | (Access << 14) | (Function << 2) | Method
+
+
 def USB_CTL(id):
     # CTL_CODE(FILE_DEVICE_USB, (id), METHOD_BUFFERED, FILE_ANY_ACCESS)
     return CTL_CODE(0x22, id, 0, 0)
+
+
 IOCTL_USB_GET_ROOT_HUB_NAME = USB_CTL(258)
 IOCTL_USB_GET_NODE_INFORMATION = USB_CTL(258)
 IOCTL_USB_GET_NODE_CONNECTION_INFORMATION = USB_CTL(259)
@@ -80,6 +91,7 @@ MAXIMUM_USB_STRING_LENGTH = 255
 
 StorageDeviceNumber = namedtuple('StorageDeviceNumber', 'type number partition_number')
 
+
 class STORAGE_DEVICE_NUMBER(Structure):
     _fields_ = [
         ('DeviceType', DWORD),
@@ -90,6 +102,7 @@ class STORAGE_DEVICE_NUMBER(Structure):
     def as_tuple(self):
         return StorageDeviceNumber(self.DeviceType, self.DeviceNumber, self.PartitionNumber)
 
+
 class SP_DEVINFO_DATA(Structure):
     _fields_ = [
         ('cbSize', DWORD),
@@ -97,11 +110,14 @@ class SP_DEVINFO_DATA(Structure):
         ('DevInst', DEVINST),
         ('Reserved', POINTER(ULONG)),
     ]
+
     def __str__(self):
-        return "ClassGuid:%s DevInst:%s" % (self.ClassGuid, self.DevInst)
+        return f"ClassGuid:{self.ClassGuid} DevInst:{self.DevInst}"
+
 
 PSP_DEVINFO_DATA = POINTER(SP_DEVINFO_DATA)
 
+
 class SP_DEVICE_INTERFACE_DATA(Structure):
     _fields_ = [
         ('cbSize', DWORD),
@@ -109,19 +125,24 @@ class SP_DEVICE_INTERFACE_DATA(Structure):
         ('Flags', DWORD),
         ('Reserved', POINTER(ULONG)),
     ]
+
     def __str__(self):
-        return "InterfaceClassGuid:%s Flags:%s" % (self.InterfaceClassGuid, self.Flags)
+        return f"InterfaceClassGuid:{self.InterfaceClassGuid} Flags:{self.Flags}"
+
 
 ANYSIZE_ARRAY = 1
 
+
 class SP_DEVICE_INTERFACE_DETAIL_DATA(Structure):
     _fields_ = [
         ("cbSize", DWORD),
         ("DevicePath", c_wchar*ANYSIZE_ARRAY)
     ]
 
+
 UCHAR = c_ubyte
 
+
 class USB_DEVICE_DESCRIPTOR(Structure):
     _fields_ = (
         ('bLength', UCHAR),
@@ -146,6 +167,7 @@ class USB_DEVICE_DESCRIPTOR(Structure):
             self.idVendor, self.idProduct, self.bcdDevice, self.iManufacturer,
             self.iProduct, self.iSerialNumber)
 
+
 class USB_ENDPOINT_DESCRIPTOR(Structure):
     _fields_ = (
         ('bLength', UCHAR),
@@ -156,6 +178,7 @@ class USB_ENDPOINT_DESCRIPTOR(Structure):
         ('bInterval', UCHAR)
     )
 
+
 class USB_PIPE_INFO(Structure):
     _fields_ = (
         ('EndpointDescriptor', USB_ENDPOINT_DESCRIPTOR),
@@ -176,6 +199,7 @@ class USB_NODE_CONNECTION_INFORMATION_EX(Structure):
         ('PipeList', USB_PIPE_INFO*ANYSIZE_ARRAY),
     )
 
+
 class USB_STRING_DESCRIPTOR(Structure):
     _fields_ = (
         ('bLength', UCHAR),
@@ -183,6 +207,7 @@ class USB_STRING_DESCRIPTOR(Structure):
         ('String', UCHAR * ANYSIZE_ARRAY),
     )
 
+
 class USB_DESCRIPTOR_REQUEST(Structure):
 
     class SetupPacket(Structure):
@@ -200,12 +225,13 @@ class USB_DESCRIPTOR_REQUEST(Structure):
         ('Data', USB_STRING_DESCRIPTOR),
     )
 
+
 PUSB_DESCRIPTOR_REQUEST = POINTER(USB_DESCRIPTOR_REQUEST)
 PSP_DEVICE_INTERFACE_DETAIL_DATA = POINTER(SP_DEVICE_INTERFACE_DETAIL_DATA)
 PSP_DEVICE_INTERFACE_DATA = POINTER(SP_DEVICE_INTERFACE_DATA)
 INVALID_HANDLE_VALUE = c_void_p(-1).value
-GENERIC_READ = 0x80000000L
-GENERIC_WRITE = 0x40000000L
+GENERIC_READ = 0x80000000
+GENERIC_WRITE = 0x40000000
 FILE_SHARE_READ = 0x1
 FILE_SHARE_WRITE = 0x2
 OPEN_EXISTING = 0x3
@@ -343,6 +369,7 @@ setupapi = windll.setupapi
 cfgmgr = windll.CfgMgr32
 kernel32 = windll.Kernel32
 
+
 def cwrap(name, restype, *argtypes, **kw):
     errcheck = kw.pop('errcheck', None)
     use_last_error = bool(kw.pop('use_last_error', True))
@@ -355,21 +382,25 @@ def cwrap(name, restype, *argtypes, **kw):
         func.errcheck = errcheck
     return func
 
+
 def handle_err_check(result, func, args):
     if result == INVALID_HANDLE_VALUE:
         raise WinError(get_last_error())
     return result
 
+
 def bool_err_check(result, func, args):
     if not result:
         raise WinError(get_last_error())
     return result
 
+
 def config_err_check(result, func, args):
     if result != CR_CODES['CR_SUCCESS']:
-        raise WindowsError(result, 'The cfgmgr32 function failed with err: %s' % CR_CODE_NAMES.get(result, result))
+        raise WinError(result, 'The cfgmgr32 function failed with err: %s' % CR_CODE_NAMES.get(result, result))
     return args
 
+
 GetLogicalDrives = cwrap('GetLogicalDrives', DWORD, errcheck=bool_err_check, lib=kernel32)
 GetDriveType = cwrap('GetDriveTypeW', UINT, LPCWSTR, lib=kernel32)
 GetVolumeNameForVolumeMountPoint = cwrap('GetVolumeNameForVolumeMountPointW', BOOL, LPCWSTR, LPWSTR, DWORD, errcheck=bool_err_check, lib=kernel32)
@@ -402,6 +433,8 @@ CM_Get_Device_ID = cwrap('CM_Get_Device_IDW', CONFIGRET, DEVINST, LPWSTR, ULONG,
 # Utility functions {{{
 
 _devid_pat = None
+
+
 def devid_pat():
     global _devid_pat
     if _devid_pat is None:
@@ -409,7 +442,7 @@ def devid_pat():
     return _devid_pat
 
 
-class DeviceSet(object):
+class DeviceSet:
 
     def __init__(self, guid=GUID_DEVINTERFACE_VOLUME, enumerator=None, flags=DIGCF_PRESENT | DIGCF_DEVICEINTERFACE):
         self.guid_ref, self.enumerator, self.flags = (None if guid is None else byref(guid)), enumerator, flags
@@ -430,7 +463,7 @@ class DeviceSet(object):
                 break
             try:
                 buf, devinfo, devpath = get_device_interface_detail_data(self.dev_list, byref(interface_data), buf)
-            except WindowsError:
+            except OSError:
                 if ignore_errors:
                     continue
                 raise
@@ -455,7 +488,7 @@ def iterchildren(parent_devinst):
     NO_MORE = CR_CODES['CR_NO_SUCH_DEVINST']
     try:
         CM_Get_Child(byref(child), parent_devinst, 0)
-    except WindowsError as err:
+    except OSError as err:
         if err.winerror == NO_MORE:
             return
         raise
@@ -463,17 +496,18 @@ def iterchildren(parent_devinst):
     while True:
         try:
             CM_Get_Sibling(byref(child), child, 0)
-        except WindowsError as err:
+        except OSError as err:
             if err.winerror == NO_MORE:
                 break
             raise
         yield child.value
 
+
 def iterdescendants(parent_devinst):
     for child in iterchildren(parent_devinst):
         yield child
-        for gc in iterdescendants(child):
-            yield gc
+        yield from iterdescendants(child)
+
 
 def iterancestors(devinst):
     NO_MORE = CR_CODES['CR_NO_SUCH_DEVINST']
@@ -481,25 +515,27 @@ def iterancestors(devinst):
     while True:
         try:
             CM_Get_Parent(byref(parent), parent, 0)
-        except WindowsError as err:
+        except OSError as err:
             if err.winerror == NO_MORE:
                 break
             raise
         yield parent.value
 
+
 def device_io_control(handle, which, inbuf, outbuf, initbuf):
     bytes_returned = DWORD(0)
     while True:
         initbuf(inbuf)
         try:
             DeviceIoControl(handle, which, inbuf, len(inbuf), outbuf, len(outbuf), byref(bytes_returned), None)
-        except WindowsError as err:
+        except OSError as err:
             if err.winerror not in (ERROR_INSUFFICIENT_BUFFER, ERROR_MORE_DATA):
                 raise
             outbuf = create_string_buffer(2*len(outbuf))
         else:
             return outbuf, bytes_returned
 
+
 def get_storage_number(devpath):
     sdn = STORAGE_DEVICE_NUMBER()
     handle = CreateFile(devpath, 0, FILE_SHARE_READ | FILE_SHARE_WRITE, None, OPEN_EXISTING, 0, None)
@@ -510,6 +546,7 @@ def get_storage_number(devpath):
         CloseHandle(handle)
     return sdn.as_tuple()
 
+
 def get_device_id(devinst, buf=None):
     if buf is None:
         buf = create_unicode_buffer(512)
@@ -521,10 +558,11 @@ def get_device_id(devinst, buf=None):
             buf = create_unicode_buffer(devid_size.value)
             continue
         if ret != CR_CODES['CR_SUCCESS']:
-            raise WindowsError((ret, 'The cfgmgr32 function failed with err: %s' % CR_CODE_NAMES.get(ret, ret)))
+            raise WinError(ret, 'The cfgmgr32 function failed with err: %s' % CR_CODE_NAMES.get(ret, ret))
         break
     return wstring_at(buf), buf
 
+
 def expand_environment_strings(src):
     sz = ExpandEnvironmentStrings(src, None, 0)
     while True:
@@ -534,6 +572,7 @@ def expand_environment_strings(src):
             return buf.value
         sz = nsz
 
+
 def convert_registry_data(raw, size, dtype):
     if dtype == winreg.REG_NONE:
         return None
@@ -556,6 +595,7 @@ def convert_registry_data(raw, size, dtype):
         return cast(raw, POINTER(QWORD)).contents.value
     raise ValueError('Unsupported data type: %r' % dtype)
 
+
 def get_device_registry_property(dev_list, p_devinfo, property_type=SPDRP_HARDWAREID, buf=None):
     if buf is None:
         buf = create_string_buffer(1024)
@@ -566,7 +606,7 @@ def get_device_registry_property(dev_list, p_devinfo, property_type=SPDRP_HARDWA
         if not SetupDiGetDeviceRegistryProperty(dev_list, p_devinfo, property_type, byref(data_type), cast(buf, POINTER(BYTE)), len(buf), byref(required_size)):
             err = get_last_error()
             if err == ERROR_INSUFFICIENT_BUFFER:
-                buf = create_string_buffer(required_size)
+                buf = create_string_buffer(required_size.value)
                 continue
             if err == ERROR_INVALID_DATA:
                 break
@@ -575,6 +615,7 @@ def get_device_registry_property(dev_list, p_devinfo, property_type=SPDRP_HARDWA
         break
     return buf, ans
 
+
 def get_device_interface_detail_data(dev_list, p_interface_data, buf=None):
     if buf is None:
         buf = create_string_buffer(512)
@@ -589,7 +630,7 @@ def get_device_interface_detail_data(dev_list, p_interface_data, buf=None):
         if not SetupDiGetDeviceInterfaceDetail(dev_list, p_interface_data, detail, len(buf), byref(required_size), byref(devinfo)):
             err = get_last_error()
             if err == ERROR_INSUFFICIENT_BUFFER:
-                buf = create_string_buffer(required_size + 50)
+                buf = create_string_buffer(required_size.value + 50)
                 detail = cast(buf, PSP_DEVICE_INTERFACE_DETAIL_DATA)
                 detail.contents.cbSize = 8 if is64bit else 6
                 continue
@@ -597,6 +638,7 @@ def get_device_interface_detail_data(dev_list, p_interface_data, buf=None):
         break
     return buf, devinfo, wstring_at(addressof(buf) + sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA._fields_[0][1]))
 
+
 def get_volume_information(drive_letter):
     if not drive_letter.endswith('\\'):
         drive_letter += ':\\'
@@ -612,16 +654,17 @@ def get_volume_information(drive_letter):
         'max_component_length': max_component_length.value,
     }
 
-    for name, num in {'FILE_CASE_PRESERVED_NAMES':0x00000002, 'FILE_CASE_SENSITIVE_SEARCH':0x00000001, 'FILE_FILE_COMPRESSION':0x00000010,
+    for name, num in iteritems({'FILE_CASE_PRESERVED_NAMES':0x00000002, 'FILE_CASE_SENSITIVE_SEARCH':0x00000001, 'FILE_FILE_COMPRESSION':0x00000010,
               'FILE_NAMED_STREAMS':0x00040000, 'FILE_PERSISTENT_ACLS':0x00000008, 'FILE_READ_ONLY_VOLUME':0x00080000,
               'FILE_SEQUENTIAL_WRITE_ONCE':0x00100000, 'FILE_SUPPORTS_ENCRYPTION':0x00020000, 'FILE_SUPPORTS_EXTENDED_ATTRIBUTES':0x00800000,
               'FILE_SUPPORTS_HARD_LINKS':0x00400000, 'FILE_SUPPORTS_OBJECT_IDS':0x00010000, 'FILE_SUPPORTS_OPEN_BY_FILE_ID':0x01000000,
               'FILE_SUPPORTS_REPARSE_POINTS':0x00000080, 'FILE_SUPPORTS_SPARSE_FILES':0x00000040, 'FILE_SUPPORTS_TRANSACTIONS':0x00200000,
               'FILE_SUPPORTS_USN_JOURNAL':0x02000000, 'FILE_UNICODE_ON_DISK':0x00000004, 'FILE_VOLUME_IS_COMPRESSED':0x00008000,
-              'FILE_VOLUME_QUOTAS':0x00000020}.iteritems():
+              'FILE_VOLUME_QUOTAS':0x00000020}):
         ans[name] = bool(num & flags)
     return ans
 
+
 def get_volume_pathnames(volume_id, buf=None):
     if buf is None:
         buf = create_unicode_buffer(512)
@@ -630,20 +673,22 @@ def get_volume_pathnames(volume_id, buf=None):
         try:
             GetVolumePathNamesForVolumeName(volume_id, buf, len(buf), byref(bufsize))
             break
-        except WindowsError as err:
+        except OSError as err:
             if err.winerror == ERROR_MORE_DATA:
                 buf = create_unicode_buffer(bufsize.value + 10)
                 continue
             raise
     ans = wstring_at(buf, bufsize.value)
-    return buf, filter(None, ans.split('\0'))
+    return buf, list(filter(None, ans.split('\0')))
 
 # }}}
 
 # def scan_usb_devices(): {{{
 
+
 _USBDevice = namedtuple('USBDevice', 'vendor_id product_id bcd devid devinst')
 
+
 class USBDevice(_USBDevice):
 
     def __repr__(self):
@@ -651,12 +696,14 @@ class USBDevice(_USBDevice):
             if x is None:
                 return 'None'
             return '0x%x' % x
-        return 'USBDevice(vendor_id=%s product_id=%s bcd=%s devid=%s devinst=%s)' % (
+        return 'USBDevice(vendor_id={} product_id={} bcd={} devid={} devinst={})'.format(
             r(self.vendor_id), r(self.product_id), r(self.bcd), self.devid, self.devinst)
 
+
 def parse_hex(x):
     return int(x.replace(':', 'a'), 16)
 
+
 def iterusbdevices():
     buf = None
     pat = devid_pat()
@@ -675,11 +722,13 @@ def iterusbdevices():
                 else:
                     yield USBDevice(vid, pid, bcd, devid, devinfo.DevInst)
 
+
 def scan_usb_devices():
     return tuple(iterusbdevices())
 
 # }}}
 
+
 def get_drive_letters_for_device(usbdev, storage_number_map=None, debug=False):  # {{{
     '''
     Get the drive letters for a connected device. The drive letters are sorted
@@ -717,6 +766,7 @@ def get_drive_letters_for_device(usbdev, storage_number_map=None, debug=False):
     else:
         return get_drive_letters_for_device_single(usbdev, sn_map, debug=debug)
 
+
 def get_drive_letters_for_device_single(usbdev, storage_number_map, debug=False):
     ans = {'pnp_id_map': {}, 'drive_letters':[], 'readonly_drives':set(), 'sort_map':{}}
     descendants = frozenset(iterdescendants(usbdev.devinst))
@@ -725,16 +775,16 @@ def get_drive_letters_for_device_single(usbdev, storage_number_map, debug=False)
             if debug:
                 try:
                     devid = get_device_id(devinfo.DevInst)[0]
-                except Exception as err:
+                except Exception:
                     devid = 'Unknown'
             try:
                 storage_number = get_storage_number(devpath)
-            except WindowsError as err:
+            except OSError as err:
                 if debug:
-                    prints('Failed to get storage number for: %s with error: %s' % (devid, as_unicode(err)))
+                    prints(f'Failed to get storage number for: {devid} with error: {as_unicode(err)}')
                 continue
             if debug:
-                prints('Storage number for %s: %s' % (devid, storage_number))
+                prints(f'Storage number for {devid}: {storage_number}')
             if storage_number:
                 partitions = storage_number_map.get(storage_number[:2])
                 drive_letters = []
@@ -750,30 +800,32 @@ def get_drive_letters_for_device_single(usbdev, storage_number_map, debug=False)
         try:
             if is_readonly(dl):
                 ans['readonly_drives'].add(dl)
-        except WindowsError as err:
+        except OSError as err:
             if debug:
-                prints('Failed to get readonly status for drive: %s with error: %s' % (dl, as_unicode(err)))
+                prints(f'Failed to get readonly status for drive: {dl} with error: {as_unicode(err)}')
 
     return ans
 
+
 def get_storage_number_map(drive_types=(DRIVE_REMOVABLE, DRIVE_FIXED), debug=False):
     ' Get a mapping of drive letters to storage numbers for all drives on system (of the specified types) '
     mask = GetLogicalDrives()
     type_map = {letter:GetDriveType(letter + ':' + os.sep) for i, letter in enumerate(string.ascii_uppercase) if mask & (1 << i)}
-    drives = (letter for letter, dt in type_map.iteritems() if dt in drive_types)
+    drives = (letter for letter, dt in iteritems(type_map) if dt in drive_types)
     ans = defaultdict(list)
     for letter in drives:
         try:
             sn = get_storage_number('\\\\.\\' + letter + ':')
             ans[sn[:2]].append((sn[2], letter))
-        except WindowsError as err:
+        except OSError as err:
             if debug:
-                prints('Failed to get storage number for drive: %s with error: %s' % (letter, as_unicode(err)))
+                prints(f'Failed to get storage number for drive: {letter} with error: {as_unicode(err)}')
             continue
-    for val in ans.itervalues():
+    for val in itervalues(ans):
         val.sort(key=itemgetter(0))
     return dict(ans)
 
+
 def get_storage_number_map_alt(debug=False):
     ' Alternate implementation that works without needing to call GetDriveType() (which causes floppy drives to seek) '
     wbuf = create_unicode_buffer(512)
@@ -783,16 +835,16 @@ def get_storage_number_map_alt(debug=False):
             devpath += os.sep
         try:
             GetVolumeNameForVolumeMountPoint(devpath, wbuf, len(wbuf))
-        except WindowsError as err:
+        except OSError as err:
             if debug:
-                prints('Failed to get volume id for drive: %s with error: %s' % (devpath, as_unicode(err)))
+                prints(f'Failed to get volume id for drive: {devpath} with error: {as_unicode(err)}')
             continue
         vname = wbuf.value
         try:
             wbuf, names = get_volume_pathnames(vname, buf=wbuf)
-        except WindowsError as err:
+        except OSError as err:
             if debug:
-                prints('Failed to get mountpoints for volume %s with error: %s' % (devpath, as_unicode(err)))
+                prints(f'Failed to get mountpoints for volume {devpath} with error: {as_unicode(err)}')
             continue
         for name in names:
             name = name.upper()
@@ -800,21 +852,22 @@ def get_storage_number_map_alt(debug=False):
                 break
         else:
             if debug:
-                prints('Ignoring volume %s as it has no assigned drive letter. Mountpoints: %s' % (devpath, names))
+                prints(f'Ignoring volume {devpath} as it has no assigned drive letter. Mountpoints: {names}')
             continue
         try:
             sn = get_storage_number('\\\\.\\' + name[0] + ':')
             ans[sn[:2]].append((sn[2], name[0]))
-        except WindowsError as err:
+        except OSError as err:
             if debug:
-                prints('Failed to get storage number for drive: %s with error: %s' % (name[0], as_unicode(err)))
+                prints(f'Failed to get storage number for drive: {name[0]} with error: {as_unicode(err)}')
             continue
-    for val in ans.itervalues():
+    for val in itervalues(ans):
         val.sort(key=itemgetter(0))
     return dict(ans)
 
 # }}}
 
+
 def is_usb_device_connected(vendor_id, product_id):  # {{{
     for usbdev in iterusbdevices():
         if usbdev.vendor_id == vendor_id and usbdev.product_id == product_id:
@@ -822,6 +875,7 @@ def is_usb_device_connected(vendor_id, product_id):  # {{{
     return False
 # }}}
 
+
 def get_usb_info(usbdev, debug=False):  # {{{
     '''
     The USB info (manufacturer/product names and serial number) Requires communication with the hub the device is connected to.
@@ -859,7 +913,7 @@ def get_usb_info(usbdev, debug=False):  # {{{
                 if index:
                     try:
                         buf, ans[name] = get_device_string(handle, device_port, index, buf=buf)
-                    except WindowsError as err:
+                    except OSError as err:
                         if debug:
                             # Note that I have observed that this fails
                             # randomly after some time of my Kindle being
@@ -870,6 +924,7 @@ def get_usb_info(usbdev, debug=False):  # {{{
         CloseHandle(handle)
     return ans
 
+
 def alloc_descriptor_buf(buf):
     if buf is None:
         buf = create_string_buffer(sizeof(USB_DESCRIPTOR_REQUEST) + 700)
@@ -877,6 +932,7 @@ def alloc_descriptor_buf(buf):
         memset(buf, 0, len(buf))
     return buf
 
+
 def get_device_descriptor(hub_handle, device_port, buf=None):
     buf = alloc_descriptor_buf(buf)
 
@@ -886,6 +942,7 @@ def get_device_descriptor(hub_handle, device_port, buf=None):
     buf, bytes_returned = device_io_control(hub_handle, IOCTL_USB_GET_NODE_CONNECTION_INFORMATION_EX, buf, buf, initbuf)
     return buf, USB_DEVICE_DESCRIPTOR.from_buffer_copy(cast(buf, POINTER(USB_NODE_CONNECTION_INFORMATION_EX)).contents.DeviceDescriptor)
 
+
 def get_device_string(hub_handle, device_port, index, buf=None, lang=0x409):
     buf = alloc_descriptor_buf(buf)
 
@@ -902,9 +959,10 @@ def get_device_string(hub_handle, device_port, index, buf=None, lang=0x409):
     data = cast(buf, PUSB_DESCRIPTOR_REQUEST).contents.Data
     sz, dtype = data.bLength, data.bType
     if dtype != 0x03:
-        raise WindowsError('Invalid datatype for string descriptor: 0x%x' % dtype)
+        raise OSError(errno.EINVAL, 'Invalid datatype for string descriptor: 0x%x' % dtype)
     return buf, wstring_at(addressof(data.String), sz // 2).rstrip('\0')
 
+
 def get_device_languages(hub_handle, device_port, buf=None):
     ' Get the languages supported by the device for strings '
     buf = alloc_descriptor_buf(buf)
@@ -921,24 +979,25 @@ def get_device_languages(hub_handle, device_port, buf=None):
     data = cast(buf, PUSB_DESCRIPTOR_REQUEST).contents.Data
     sz, dtype = data.bLength, data.bType
     if dtype != 0x03:
-        raise WindowsError('Invalid datatype for string descriptor: 0x%x' % dtype)
+        raise OSError(errno.EINVAL, 'Invalid datatype for string descriptor: 0x%x' % dtype)
     data = cast(data.String, POINTER(USHORT*(sz//2)))
-    return buf, filter(None, data.contents)
+    return buf, list(filter(None, data.contents))
 
 # }}}
 
+
 def is_readonly(drive_letter):  # {{{
     return get_volume_information(drive_letter)['FILE_READ_ONLY_VOLUME']
 # }}}
 
+
 def develop():  # {{{
     from calibre.customize.ui import device_plugins
     usb_devices = scan_usb_devices()
     drive_letters = set()
     pprint(usb_devices)
     print()
-    devplugins = list(sorted(device_plugins(), cmp=lambda
-            x,y:cmp(x.__class__.__name__, y.__class__.__name__)))
+    devplugins = list(sorted(device_plugins(), key=lambda x: x.__class__.__name__))
     for dev in devplugins:
         dev.startup()
     for dev in devplugins:
@@ -947,7 +1006,7 @@ def develop():  # {{{
         connected, usbdev = dev.is_usb_connected(usb_devices, debug=True)
         if connected:
             print('\n')
-            print('Potentially connected device: %s at %s' % (dev.get_gui_name(), usbdev))
+            print(f'Potentially connected device: {dev.get_gui_name()} at {usbdev}')
             print()
             print('Drives for this device:')
             data = get_drive_letters_for_device(usbdev, debug=True)
@@ -958,15 +1017,17 @@ def develop():  # {{{
             print()
             print('Device USB data:', get_usb_info(usbdev, debug=True))
 
+
 def drives_for(vendor_id, product_id=None):
     usb_devices = scan_usb_devices()
     pprint(usb_devices)
     for usbdev in usb_devices:
         if usbdev.vendor_id == vendor_id and (product_id is None or usbdev.product_id == product_id):
-            print('Drives for: {}'.format(usbdev))
+            print(f'Drives for: {usbdev}')
             pprint(get_drive_letters_for_device(usbdev, debug=True))
             print('USB info:', get_usb_info(usbdev, debug=True))
 
+
 if __name__ == '__main__':
     develop()
 # }}}
