@@ -1,25 +1,19 @@
-from __future__ import with_statement
 ''' CHM File decoding support '''
 __license__ = 'GPL v3'
 __copyright__  = '2008, Kovid Goyal <kovid at kovidgoyal.net>,' \
                  ' and Alex Bramley <a.bramley at gmail.com>.'
 
-import os, re, codecs
+import codecs
+import os
+import re
 
 from calibre import guess_type as guess_mimetype
+from calibre.constants import filesystem_encoding, iswindows
 from calibre.ebooks.BeautifulSoup import BeautifulSoup, NavigableString
-from calibre.constants import iswindows, filesystem_encoding
-from calibre.utils.chm.chm import CHMFile
-
-from calibre.constants import plugins
-from calibre.ebooks.metadata.toc import TOC
 from calibre.ebooks.chardet import xml_to_unicode
-from polyglot.builtins import unicode_type
-
-
-chmlib, chmlib_err = plugins['chmlib']
-if chmlib_err:
-    raise RuntimeError('Failed to load chmlib: ' + chmlib_err)
+from calibre.ebooks.metadata.toc import TOC
+from chm.chm import CHMFile, chmlib
+from polyglot.builtins import as_unicode
 
 
 def match_string(s1, s2_already_lowered):
@@ -49,8 +43,15 @@ class CHMReader(CHMFile):
 
     def __init__(self, input, log, input_encoding=None):
         CHMFile.__init__(self)
-        if isinstance(input, unicode_type):
-            input = input.encode(filesystem_encoding)
+        if isinstance(input, str):
+            enc = 'mbcs' if iswindows else filesystem_encoding
+            try:
+                input = input.encode(enc)
+            except UnicodeEncodeError:
+                from calibre.ptempfile import PersistentTemporaryFile
+                with PersistentTemporaryFile(suffix='.chm') as t:
+                    t.write(open(input, 'rb').read())
+                input = t.name
         if not self.LoadCHM(input):
             raise CHMError("Unable to open CHM file '%s'"%(input,))
         self.log = log
@@ -61,16 +62,58 @@ class CHMReader(CHMFile):
         self._metadata = False
         self._extracted = False
         self.re_encoded_files = set()
+        self.get_encodings()
+        if self.home:
+            self.home = self.decode_hhp_filename(self.home)
+        if self.topics:
+            self.topics = self.decode_hhp_filename(self.topics)
 
         # location of '.hhc' file, which is the CHM TOC.
-        if self.topics is None:
-            self.root, ext = os.path.splitext(self.home.lstrip('/'))
-            self.hhc_path = self.root + ".hhc"
-        else:
-            self.root, ext = os.path.splitext(self.topics.lstrip('/'))
-            self.hhc_path = self.root + ".hhc"
+        base = self.topics or self.home
+        self.root = os.path.splitext(base.lstrip('/'))[0]
+        self.hhc_path = self.root + ".hhc"
 
-    def _parse_toc(self, ul, basedir=os.getcwdu()):
+    def decode_hhp_filename(self, path):
+        if isinstance(path, str):
+            return path
+        for enc in (self.encoding_from_system_file, self.encoding_from_lcid, 'cp1252', 'cp1251', 'latin1', 'utf-8'):
+            if enc:
+                try:
+                    q = path.decode(enc)
+                except UnicodeDecodeError:
+                    continue
+                res, ui = self.ResolveObject(q)
+                if res == chmlib.CHM_RESOLVE_SUCCESS:
+                    return q
+
+    def get_encodings(self):
+        self.encoding_from_system_file = self.encoding_from_lcid = None
+        q = self.GetEncoding()
+        if q:
+            try:
+                if isinstance(q, bytes):
+                    q = q.decode('ascii')
+                    codecs.lookup(q)
+                    self.encoding_from_system_file = q
+            except Exception:
+                pass
+
+        lcid = self.GetLCID()
+        if lcid is not None:
+            q = lcid[0]
+            if q:
+                try:
+                    if isinstance(q, bytes):
+                        q = q.decode('ascii')
+                        codecs.lookup(q)
+                        self.encoding_from_lcid = q
+                except Exception:
+                    pass
+
+    def get_encoding(self):
+        return self.encoding_from_system_file or self.encoding_from_lcid or 'cp1252'
+
+    def _parse_toc(self, ul, basedir=os.getcwd()):
         toc = TOC(play_order=self._playorder, base_path=basedir, text='')
         self._playorder += 1
         for li in ul('li', recursive=False):
@@ -90,6 +133,13 @@ class CHMReader(CHMFile):
         # print toc
         return toc
 
+    def ResolveObject(self, path):
+        # filenames are utf-8 encoded in the chm index as far as I can
+        # determine, see https://tika.apache.org/1.11/api/org/apache/tika/parser/chm/accessor/ChmPmgiHeader.html
+        if not isinstance(path, bytes):
+            path = path.encode('utf-8')
+        return CHMFile.ResolveObject(self, path)
+
     def GetFile(self, path):
         # have to have abs paths for ResolveObject, but Contents() deliberately
         # makes them relative. So we don't have to worry, re-add the leading /.
@@ -98,24 +148,19 @@ class CHMReader(CHMFile):
             path = '/' + path
         res, ui = self.ResolveObject(path)
         if res != chmlib.CHM_RESOLVE_SUCCESS:
-            raise CHMError("Unable to locate '%s' within CHM file '%s'"%(path, self.filename))
+            raise CHMError(f"Unable to locate {path!r} within CHM file {self.filename!r}")
         size, data = self.RetrieveObject(ui)
         if size == 0:
-            raise CHMError("'%s' is zero bytes in length!"%(path,))
+            raise CHMError(f"{path!r} is zero bytes in length!")
         return data
 
-    def ExtractFiles(self, output_dir=os.getcwdu(), debug_dump=False):
-        html_files = set([])
-        try:
-            x = self.get_encoding()
-            codecs.lookup(x)
-            enc = x
-        except:
-            enc = 'cp1252'
+    def get_home(self):
+        return self.GetFile(self.home)
+
+    def ExtractFiles(self, output_dir=os.getcwd(), debug_dump=False):
+        html_files = set()
         for path in self.Contents():
             fpath = path
-            if not isinstance(path, unicode_type):
-                fpath = path.decode(enc)
             lpath = os.path.join(output_dir, fpath)
             self._ensure_dir(lpath)
             try:
@@ -147,7 +192,7 @@ class CHMReader(CHMFile):
             with lopen(lpath, 'r+b') as f:
                 data = f.read()
                 data = self._reformat(data, lpath)
-                if isinstance(data, unicode_type):
+                if isinstance(data, str):
                     data = data.encode('utf-8')
                 f.seek(0)
                 f.truncate()
@@ -274,11 +319,14 @@ class CHMReader(CHMFile):
         paths = []
 
         def get_paths(chm, ui, ctx):
+            # these are supposed to be UTF-8 in CHM as best as I can determine
+            # see https://tika.apache.org/1.11/api/org/apache/tika/parser/chm/accessor/ChmPmgiHeader.html
+            path = as_unicode(ui.path, 'utf-8')
             # skip directories
             # note this path refers to the internal CHM structure
-            if ui.path[-1] != '/':
+            if path[-1] != '/':
                 # and make paths relative
-                paths.append(ui.path.lstrip('/'))
+                paths.append(path.lstrip('/'))
         chmlib.chm_enumerate(self.file, chmlib.CHM_ENUMERATE_NORMAL, get_paths, None)
         self._contents = paths
         return self._contents
@@ -288,5 +336,5 @@ class CHMReader(CHMFile):
         if not os.path.isdir(dir):
             os.makedirs(dir)
 
-    def extract_content(self, output_dir=os.getcwdu(), debug_dump=False):
+    def extract_content(self, output_dir=os.getcwd(), debug_dump=False):
         self.ExtractFiles(output_dir=output_dir, debug_dump=debug_dump)
