@@ -1,5 +1,4 @@
 #!/usr/bin/env python
-# vim:fileencoding=utf-8
 
 
 __license__   = 'GPL v3'
@@ -14,7 +13,7 @@ from contextlib import suppress
 
 from calibre import relpath, guess_type, prints, force_unicode
 from calibre.utils.config_base import tweaks
-from polyglot.builtins import codepoint_to_chr, unicode_type, range, map, zip, getcwd, iteritems, as_unicode
+from polyglot.builtins import codepoint_to_chr, iteritems, as_unicode
 from polyglot.urllib import quote, unquote, urlparse
 
 
@@ -161,15 +160,18 @@ def get_title_sort_pat(lang=None):
     except AttributeError:
         ans = None  # invalid tweak value
     try:
-        ans = frozenset(ans) if ans else frozenset(data['eng'])
-    except:
+        ans = frozenset(ans) if ans is not None else frozenset(data['eng'])
+    except Exception:
         ans = frozenset((r'A\s+', r'The\s+', r'An\s+'))
-    ans = '|'.join(ans)
-    ans = '^(%s)'%ans
-    try:
-        ans = re.compile(ans, re.IGNORECASE)
-    except:
-        ans = re.compile(r'^(A|The|An)\s+', re.IGNORECASE)
+    if ans:
+        ans = '|'.join(ans)
+        ans = '^(%s)'%ans
+        try:
+            ans = re.compile(ans, re.IGNORECASE)
+        except:
+            ans = re.compile(r'^(A|The|An)\s+', re.IGNORECASE)
+    else:
+        ans = re.compile('^$')  # matches only the empty string
     _title_pats[lang] = ans
     return ans
 
@@ -193,9 +195,10 @@ def title_sort(title, order=None, lang=None):
         except IndexError:
             pass
         else:
-            title = title[len(prep):] + ', ' + prep
-            if title[0] in _ignore_starts:
-                title = title[1:]
+            if prep:
+                title = title[len(prep):] + ', ' + prep
+                if title[0] in _ignore_starts:
+                    title = title[1:]
     return title.strip()
 
 
@@ -207,7 +210,7 @@ coding = list(zip(
 
 def roman(num):
     if num <= 0 or num >= 4000 or int(num) != num:
-        return unicode_type(num)
+        return str(num)
     result = []
     for d, r in coding:
         while num >= d:
@@ -222,13 +225,16 @@ def fmt_sidx(i, fmt='%.2f', use_roman=False):
     try:
         i = float(i)
     except Exception:
-        return unicode_type(i)
+        return str(i)
     if int(i) == float(i):
         return roman(int(i)) if use_roman else '%d'%int(i)
-    return fmt%i
+    ans = fmt%i
+    if '.' in ans:
+        ans = ans.rstrip('0')
+    return ans
 
 
-class Resource(object):
+class Resource:
 
     '''
     Represents a resource (usually a file on the filesystem or a URL pointing
@@ -242,7 +248,7 @@ class Resource(object):
 
     '''
 
-    def __init__(self, href_or_path, basedir=getcwd(), is_path=True):
+    def __init__(self, href_or_path, basedir=os.getcwd(), is_path=True):
         self._href = None
         self._basedir = basedir
         self.path = None
@@ -266,7 +272,7 @@ class Resource(object):
                 self._href = href_or_path
             else:
                 pc = url[2]
-                if isinstance(pc, unicode_type):
+                if isinstance(pc, str):
                     pc = pc.encode('utf-8')
                 pc = unquote(pc).decode('utf-8')
                 self.path = os.path.abspath(os.path.join(basedir, pc.replace('/', os.sep)))
@@ -284,10 +290,10 @@ class Resource(object):
             if self._basedir:
                 basedir = self._basedir
             else:
-                basedir = getcwd()
+                basedir = os.getcwd()
         if self.path is None:
             return self._href
-        f = self.fragment.encode('utf-8') if isinstance(self.fragment, unicode_type) else self.fragment
+        f = self.fragment.encode('utf-8') if isinstance(self.fragment, str) else self.fragment
         frag = '#'+as_unicode(quote(f)) if self.fragment else ''
         if self.path == basedir:
             return ''+frag
@@ -295,7 +301,7 @@ class Resource(object):
             rpath = relpath(self.path, basedir)
         except OSError:  # On windows path and basedir could be on different drives
             rpath = self.path
-        if isinstance(rpath, unicode_type):
+        if isinstance(rpath, str):
             rpath = rpath.encode('utf-8')
         return as_unicode(quote(rpath.replace(os.sep, '/')))+frag
 
@@ -309,14 +315,13 @@ class Resource(object):
         return 'Resource(%s, %s)'%(repr(self.path), repr(self.href()))
 
 
-class ResourceCollection(object):
+class ResourceCollection:
 
     def __init__(self):
         self._resources = []
 
     def __iter__(self):
-        for r in self._resources:
-            yield r
+        yield from self._resources
 
     def __len__(self):
         return len(self._resources)
@@ -332,7 +337,7 @@ class ResourceCollection(object):
         return '[%s]'%', '.join(resources)
 
     def __repr__(self):
-        return unicode_type(self)
+        return str(self)
 
     def append(self, resource):
         if not isinstance(resource, Resource):
