@@ -4,6 +4,7 @@
 from __future__ import absolute_import, division, print_function, unicode_literals
 
 import re
+import string
 import socket
 import time
 from functools import partial
@@ -26,6 +27,23 @@ from calibre.utils.random_ua import accept_header_for_ua
 from calibre.ebooks.oeb.base import urlquote
 
 
+def sort_matches_preferring_kindle_editions(matches):
+    upos_map = {url:i for i, url in enumerate(matches)}
+
+    def skey(url):
+        opos = upos_map[url]
+        parts = url.split('/')
+        try:
+            idx = parts.index('dp')
+        except Exception:
+            idx = -1
+        if idx < 0 or idx + 1 >= len(parts) or not parts[idx+1].startswith('B'):
+            return 1, opos
+        return 0, opos
+    matches.sort(key=skey)
+    return matches
+
+
 def iri_quote_plus(url):
     ans = urlquote(url)
     if isinstance(ans, bytes):
@@ -97,7 +115,7 @@ def parse_details_page(url, log, timeout, browser, domain):
         return
     if domain == 'jp':
         for a in root.xpath('//a[@href]'):
-            if 'black-curtain-redirect.html' in a.get('href'):
+            if ('black-curtain-redirect.html' in a.get('href')) or ('/black-curtain/save-eligibility/black-curtain' in a.get('href')):
                 url = a.get('href')
                 if url:
                     if url.startswith('/'):
@@ -281,7 +299,7 @@ class Worker(Thread):  # Get details {{{
             descendant::*[starts-with(text(), "Publication Date:") or \
                     starts-with(text(), "Audible.com Release Date:")]
         '''
-        self.publisher_names = {'Publisher', 'Uitgever', 'Verlag', 'Utgivare',
+        self.publisher_names = {'Publisher', 'Uitgever', 'Verlag', 'Utgivare', 'Herausgeber',
                                 'Editore', 'Editeur', 'Editor', 'Editora', '出版社'}
 
         self.language_xpath =    '''
@@ -511,16 +529,19 @@ class Worker(Thread):  # Get details {{{
     def totext(self, elem, only_printable=False):
         res = self.tostring(elem, encoding='unicode', method='text')
         if only_printable:
-            filtered_characters = list(s for s in res if s.isprintable())
+            try:
+                filtered_characters = list(s for s in res if s.isprintable())
+            except AttributeError:
+                filtered_characters = list(s for s in res if s in string.printable)
             res = ''.join(filtered_characters).strip()
         return res
 
     def parse_title(self, root):
 
         def sanitize_title(title):
-            ans = re.sub(r'[(\[].*[)\]]', '', title).strip()
-            if not ans:
-                ans = title.rpartition('[')[0].strip()
+            ans = title.strip()
+            if not ans.startswith('['):
+                ans = re.sub(r'[(\[].*[)\]]', '', title).strip()
             return ans
 
         h1 = root.xpath('//h1[@id="title"]')
@@ -733,7 +754,10 @@ class Worker(Thread):  # Get details {{{
                 a = series[0].xpath('descendant::a')
                 if a:
                     raw = self.tostring(a[0], encoding='unicode', method='text', with_tail=False)
-                    m = re.search(r'(?:Book|Libro)\s+(?P<index>[0-9.]+)\s+(?:of|de)\s+([0-9.]+)\s*:\s*(?P<series>.+)', raw.strip())
+                    if self.domain == 'jp':
+                        m = re.search(r'(?P<index>[0-9.]+)\s*(?:巻|冊)\s*\(全\s*([0-9.]+)\s*(?:巻|冊)\):\s*(?P<series>.+)', raw.strip())
+                    else:
+                        m = re.search(r'(?:Book|Libro|Buch)\s+(?P<index>[0-9.]+)\s+(?:of|de|von)\s+([0-9.]+)\s*:\s*(?P<series>.+)', raw.strip())
                     if m is not None:
                         ans = (m.group('series').strip(), float(m.group('index')))
 
@@ -885,7 +909,7 @@ class Worker(Thread):  # Get details {{{
 
     def parse_detail_cells(self, mi, c1, c2):
         name = self.totext(c1, only_printable=True).strip().strip(':').strip()
-        val = self.totext(c2)
+        val = self.totext(c2).strip()
         if not val:
             return
         if name in self.language_names:
@@ -968,7 +992,7 @@ class Worker(Thread):  # Get details {{{
 class Amazon(Source):
 
     name = 'Amazon.com'
-    version = (1, 2, 21)
+    version = (1, 2, 28)
     minimum_calibre_version = (2, 82, 0)
     description = _('Downloads metadata and covers from Amazon')
 
@@ -990,6 +1014,7 @@ class Amazon(Source):
         'jp': _('Japan'),
         'es': _('Spain'),
         'br': _('Brazil'),
+        'in': _('India'),
         'nl': _('Netherlands'),
         'cn': _('China'),
         'ca': _('Canada'),
@@ -1023,12 +1048,34 @@ class Amazon(Source):
                    ' MOBI file came from a different Amazon country store, you could get'
                    ' incorrect results.'
                )),
+        Option('prefer_kindle_edition', 'bool', False, _('Prefer the Kindle edition, when available'),
+               _(
+                   'When searching for a book and the search engine returns both paper and Kindle editions,'
+                   ' always prefer the Kindle edition, instead of whatever the search engine returns at the'
+                   ' top.')
+        ),
     )
 
     def __init__(self, *args, **kwargs):
         Source.__init__(self, *args, **kwargs)
         self.set_amazon_id_touched_fields()
 
+    def id_from_url(self, url):
+        from polyglot.urllib import urlparse
+        purl = urlparse(url)
+        if purl.netloc and purl.path and '/dp/' in purl.path:
+            host_parts = tuple(x.lower() for x in purl.netloc.split('.'))
+            if 'amazon' in host_parts:
+                domain = host_parts[-1]
+            parts = purl.path.split('/')
+            idx = parts.index('dp')
+            try:
+                val = parts[idx+1]
+            except IndexError:
+                return
+            aid = 'amazon' if domain == 'com' else ('amazon_' + domain)
+            return aid, val
+
     def test_fields(self, mi):
         '''
         Return the first field from self.touched_fields that is null on the
@@ -1106,7 +1153,7 @@ class Amazon(Source):
 
     def _get_book_url(self, identifiers):  # {{{
         domain, asin = self.get_domain_and_asin(
-            identifiers, extra_domains=('in', 'au', 'ca'))
+            identifiers, extra_domains=('au', 'ca'))
         if domain and asin:
             url = None
             r = self.referrer_for_domain(domain)
@@ -1504,6 +1551,9 @@ class Amazon(Source):
             log.error('No matches found with query: %r' % query)
             return
 
+        if self.prefs['prefer_kindle_edition']:
+            matches = sort_matches_preferring_kindle_editions(matches)
+
         workers = [Worker(
             url, result_queue, br, log, i, domain, self, testing=testing, timeout=timeout,
             cover_url_processor=cover_url_processor, filter_result=partial(
@@ -1656,15 +1706,23 @@ def manual_tests(domain, **kw):  # {{{
     # }}}
 
     all_tests['de'] = [  # {{{
+        # series
+        (
+            {'identifiers': {'isbn': '3499275120'}},
+            [title_test('Vespasian: Das Schwert des Tribuns: Historischer Roman',
+                        exact=False), authors_test(['Robert Fabbri']), series_test('Die Vespasian-Reihe', 1)
+             ]
+
+        ),
+
         (  # umlaut in title/authors
             {'title': 'Flüsternde Wälder',
              'authors': ['Nicola Förg']},
             [title_test('Flüsternde Wälder'),
-             authors_test(['Nicola Förg'])
+             authors_test(['Nicola Förg'], subset=True)
              ]
         ),
 
-
         (
             {'identifiers': {'isbn': '9783453314979'}},
             [title_test('Die letzten Wächter: Roman',
@@ -1801,6 +1859,13 @@ def manual_tests(domain, **kw):  # {{{
         ),
     ]  # }}}
 
+    all_tests['in'] = [  # {{{
+        (   # Paperback with series
+            {'identifiers': {'amazon_in': '1423146786'}},
+            [title_test('The Heroes of Olympus, Book Five The Blood of Olympus', exact=True)]
+        ),
+    ]  # }}}
+
     def do_test(domain, start=0, stop=None, server='auto'):
         tests = all_tests[domain]
         if stop is None:
