@@ -1,17 +1,13 @@
-#!/usr/bin/env python2
-# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:fdm=marker:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+#!/usr/bin/env python
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
 import re
-from urlparse import urlparse
 from collections import Counter, OrderedDict
 from functools import partial
-from future_builtins import map
 from operator import itemgetter
 
 from lxml import etree
@@ -19,20 +15,22 @@ from lxml.builder import ElementMaker
 
 from calibre import __version__
 from calibre.ebooks.oeb.base import (
-    XPath, uuid_id, xml2text, NCX, NCX_NS, XML, XHTML, XHTML_NS, serialize, EPUB_NS)
+    XPath, uuid_id, xml2text, NCX, NCX_NS, XML, XHTML, XHTML_NS, serialize, EPUB_NS, XML_NS, OEB_DOCS)
 from calibre.ebooks.oeb.polish.errors import MalformedMarkup
 from calibre.ebooks.oeb.polish.utils import guess_type, extract
 from calibre.ebooks.oeb.polish.opf import set_guide_item, get_book_language
-from calibre.ebooks.oeb.polish.pretty import pretty_html_tree
+from calibre.ebooks.oeb.polish.pretty import pretty_html_tree, pretty_xml_tree
 from calibre.translations.dynamic import translate
 from calibre.utils.localization import get_lang, canonicalize_lang, lang_as_iso639_1
+from polyglot.builtins import iteritems
+from polyglot.urllib import urlparse
 
 ns = etree.FunctionNamespace('calibre_xpath_extensions')
 ns.prefix = 'calibre'
 ns['lower-case'] = lambda c, x: x.lower() if hasattr(x, 'lower') else x
 
 
-class TOC(object):
+class TOC:
 
     toc_title = None
 
@@ -43,6 +41,7 @@ class TOC(object):
             self.title = self.title.strip()
         self.parent = None
         self.children = []
+        self.page_list = []
 
     def add(self, title, dest, frag=None):
         c = TOC(title, dest, frag)
@@ -65,17 +64,32 @@ class TOC(object):
         self.parent = None
 
     def __iter__(self):
-        for c in self.children:
-            yield c
+        yield from self.children
 
     def __len__(self):
         return len(self.children)
 
-    def iterdescendants(self):
+    def iterdescendants(self, level=None):
+        gc_level = None if level is None else level + 1
         for child in self:
-            yield child
-            for gc in child.iterdescendants():
-                yield gc
+            if level is None:
+                yield child
+            else:
+                yield level, child
+            yield from child.iterdescendants(level=gc_level)
+
+    def remove_duplicates(self, only_text=True):
+        seen = set()
+        remove = []
+        for child in self:
+            key = child.title if only_text else (child.title, child.dest, (child.frag or None))
+            if key in seen:
+                remove.append(child)
+            else:
+                seen.add(key)
+                child.remove_duplicates()
+        for child in remove:
+            self.remove(child)
 
     @property
     def depth(self):
@@ -91,26 +105,32 @@ class TOC(object):
 
     def get_lines(self, lvl=0):
         frag = ('#'+self.frag) if self.frag else ''
-        ans = [(u'\t'*lvl) + u'TOC: %s --> %s%s'%(self.title, self.dest, frag)]
+        ans = [('\t'*lvl) + 'TOC: %s --> %s%s'%(self.title, self.dest, frag)]
         for child in self:
             ans.extend(child.get_lines(lvl+1))
         return ans
 
     def __str__(self):
-        return b'\n'.join([x.encode('utf-8') for x in self.get_lines()])
+        return '\n'.join(self.get_lines())
 
-    @property
-    def as_dict(self):
+    def to_dict(self, node_counter=None):
         ans = {
             'title':self.title, 'dest':self.dest, 'frag':self.frag,
-            'children':[c.as_dict for c in self.children]
+            'children':[c.to_dict(node_counter) for c in self.children]
         }
         if self.dest_exists is not None:
             ans['dest_exists'] = self.dest_exists
         if self.dest_error is not None:
             ans['dest_error'] = self.dest_error
+        if node_counter is not None:
+            ans['id'] = next(node_counter)
         return ans
 
+    @property
+    def as_dict(self):
+        return self.to_dict()
+
+
 def child_xpath(tag, name):
     return tag.xpath('./*[calibre:lower-case(local-name()) = "%s"]'%name)
 
@@ -123,7 +143,7 @@ def add_from_navpoint(container, navpoint, parent, ncx_name):
         text = ''
         for txt in child_xpath(nl, 'text'):
             text += etree.tostring(txt, method='text',
-                    encoding=unicode, with_tail=False)
+                    encoding='unicode', with_tail=False)
     content = child_xpath(navpoint, 'content')
     if content:
         content = content[0]
@@ -148,20 +168,30 @@ def parse_ncx(container, ncx_name):
     if navmaps:
         process_ncx_node(container, navmaps[0], toc_root, ncx_name)
     toc_root.lang = toc_root.uid = None
-    for attr, val in root.attrib.iteritems():
+    for attr, val in iteritems(root.attrib):
         if attr.endswith('lang'):
-            toc_root.lang = unicode(val)
+            toc_root.lang = str(val)
             break
     for uid in root.xpath('//*[calibre:lower-case(local-name()) = "meta" and @name="dtb:uid"]/@content'):
         if uid:
-            toc_root.uid = unicode(uid)
+            toc_root.uid = str(uid)
             break
+    for pl in root.xpath('//*[calibre:lower-case(local-name()) = "pagelist"]'):
+        for pt in pl.xpath('descendant::*[calibre:lower-case(local-name()) = "pagetarget"]'):
+            pagenum = pt.get('value')
+            if pagenum:
+                href = pt.xpath('descendant::*[calibre:lower-case(local-name()) = "content"]/@src')
+                if href:
+                    dest = container.href_to_name(href[0], base=ncx_name)
+                    frag = urlparse(href[0]).fragment or None
+                    toc_root.page_list.append({'dest': dest, 'pagenum': pagenum, 'frag': frag})
     return toc_root
 
+
 def add_from_li(container, li, parent, nav_name):
     dest = frag = text = None
     for x in li.iterchildren(XHTML('a'), XHTML('span')):
-        text = etree.tostring(x, method='text', encoding=unicode, with_tail=False).strip() or ' '.join(x.xpath('descendant-or-self::*/@title')).strip()
+        text = etree.tostring(x, method='text', encoding='unicode', with_tail=False).strip() or ' '.join(x.xpath('descendant-or-self::*/@title')).strip()
         href = x.get('href')
         if href:
             dest = nav_name if href.startswith('#') else container.href_to_name(href, base=nav_name)
@@ -169,12 +199,14 @@ def add_from_li(container, li, parent, nav_name):
         break
     return parent.add(text or None, dest or None, frag or None)
 
+
 def first_child(parent, tagname):
     try:
         return next(parent.iterchildren(tagname))
     except StopIteration:
         return None
 
+
 def process_nav_node(container, node, toc_parent, nav_name):
     for li in node.iterchildren(XHTML('li')):
         child = add_from_li(container, li, toc_parent, nav_name)
@@ -182,6 +214,7 @@ def process_nav_node(container, node, toc_parent, nav_name):
         if child is not None and ol is not None:
             process_nav_node(container, ol, child, nav_name)
 
+
 def parse_nav(container, nav_name):
     root = container.parsed(nav_name)
     toc_root = TOC()
@@ -193,13 +226,14 @@ def parse_nav(container, nav_name):
             if ol is not None:
                 process_nav_node(container, ol, toc_root, nav_name)
                 for h in nav.iterchildren(*map(XHTML, 'h1 h2 h3 h4 h5 h6'.split())):
-                    text = etree.tostring(h, method='text', encoding=unicode, with_tail=False) or h.get('title')
+                    text = etree.tostring(h, method='text', encoding='unicode', with_tail=False) or h.get('title')
                     if text:
                         toc_root.toc_title = text
                         break
                 break
     return toc_root
 
+
 def verify_toc_destinations(container, toc):
     anchor_map = {}
     anchor_xpath = XPath('//*/@id|//h:a/@name')
@@ -230,6 +264,7 @@ def verify_toc_destinations(container, toc):
                 'The anchor %(a)s does not exist in file %(f)s')%dict(
                 a=item.frag, f=name)
 
+
 def find_existing_ncx_toc(container):
     toc = container.opf_xpath('//opf:spine/@toc')
     if toc:
@@ -239,10 +274,12 @@ def find_existing_ncx_toc(container):
         toc = container.manifest_type_map.get(ncx, [None])[0]
     return toc or None
 
+
 def find_existing_nav_toc(container):
     for name in container.manifest_items_with_property('nav'):
         return name
 
+
 def get_x_toc(container, find_toc, parse_toc, verify_destinations=True):
     def empty_toc():
         ans = TOC()
@@ -255,6 +292,7 @@ def get_x_toc(container, find_toc, parse_toc, verify_destinations=True):
         verify_toc_destinations(container, ans)
     return ans
 
+
 def get_toc(container, verify_destinations=True):
     ver = container.opf_version_parsed
     if ver.major < 3:
@@ -265,19 +303,69 @@ def get_toc(container, verify_destinations=True):
             ans = get_x_toc(container, find_existing_ncx_toc, parse_ncx, verify_destinations=verify_destinations)
         return ans
 
-def ensure_id(elem):
+
+def get_guide_landmarks(container):
+    for ref in container.opf_xpath('./opf:guide/opf:reference'):
+        href, title, rtype = ref.get('href'), ref.get('title'), ref.get('type')
+        href, frag = href.partition('#')[::2]
+        name = container.href_to_name(href, container.opf_name)
+        if container.has_name(name):
+            yield {'dest':name, 'frag':frag, 'title':title or '', 'type':rtype or ''}
+
+
+def get_nav_landmarks(container):
+    nav = find_existing_nav_toc(container)
+    if nav and container.has_name(nav):
+        root = container.parsed(nav)
+        et = '{%s}type' % EPUB_NS
+        for elem in root.iterdescendants(XHTML('nav')):
+            if elem.get(et) == 'landmarks':
+                for li in elem.iterdescendants(XHTML('li')):
+                    for a in li.iterdescendants(XHTML('a')):
+                        href, rtype = a.get('href'), a.get(et)
+                        if href:
+                            title = etree.tostring(a, method='text', encoding='unicode', with_tail=False).strip()
+                            href, frag = href.partition('#')[::2]
+                            name = container.href_to_name(href, nav)
+                            if container.has_name(name):
+                                yield {'dest':name, 'frag':frag, 'title':title or '', 'type':rtype or ''}
+                            break
+
+
+def get_landmarks(container):
+    ver = container.opf_version_parsed
+    if ver.major < 3:
+        return list(get_guide_landmarks(container))
+    ans = list(get_nav_landmarks(container))
+    if len(ans) == 0:
+        ans = list(get_guide_landmarks(container))
+    return ans
+
+
+def ensure_id(elem, all_ids):
+    elem_id = elem.get('id')
+    if elem_id:
+        return False, elem_id
     if elem.tag == XHTML('a'):
         anchor = elem.get('name', None)
         if anchor:
+            elem.set('id', anchor)
             return False, anchor
-    elem_id = elem.get('id', None)
-    if elem_id:
-        return False, elem_id
-    elem.set('id', uuid_id())
+    c = 0
+    while True:
+        c += 1
+        q = f'toc_{c}'
+        if q not in all_ids:
+            elem.set('id', q)
+            all_ids.add(q)
+            break
     return True, elem.get('id')
 
-def elem_to_toc_text(elem):
+
+def elem_to_toc_text(elem, prefer_title=False):
     text = xml2text(elem).strip()
+    if prefer_title:
+        text = elem.get('title', '').strip() or text
     if not text:
         text = elem.get('title', '')
     if not text:
@@ -288,6 +376,7 @@ def elem_to_toc_text(elem):
         text = _('(Untitled)')
     return text
 
+
 def item_at_top(elem):
     try:
         body = XPath('//h:body')(elem.getroottree().getroot())[0]
@@ -310,12 +399,13 @@ def item_at_top(elem):
                 return False
     return True
 
-def from_xpaths(container, xpaths):
+
+def from_xpaths(container, xpaths, prefer_title=False):
     '''
     Generate a Table of Contents from a list of XPath expressions. Each
     expression in the list corresponds to a level of the generate ToC. For
     example: :code:`['//h:h1', '//h:h2', '//h:h3']` will generate a three level
-    table of contents from the ``<h1>``, ``<h2>`` and ``<h3>`` tags.
+    Table of Contents from the ``<h1>``, ``<h2>`` and ``<h3>`` tags.
     '''
     tocroot = TOC()
     xpaths = [XPath(xp) for xp in xpaths]
@@ -327,14 +417,14 @@ def from_xpaths(container, xpaths):
         name = container.abspath_to_name(spinepath)
         root = container.parsed(name)
         level_item_map = maps[name] = {i+1:frozenset(xp(root)) for i, xp in enumerate(xpaths)}
-        for lvl, elems in level_item_map.iteritems():
+        for lvl, elems in iteritems(level_item_map):
             if elems:
                 empty_levels.discard(lvl)
     # Remove empty levels from all level_maps
     if empty_levels:
-        for name, lmap in tuple(maps.iteritems()):
-            lmap = {lvl:items for lvl, items in lmap.iteritems() if lvl not in empty_levels}
-            lmap = sorted(lmap.iteritems(), key=itemgetter(0))
+        for name, lmap in tuple(iteritems(maps)):
+            lmap = {lvl:items for lvl, items in iteritems(lmap) if lvl not in empty_levels}
+            lmap = sorted(iteritems(lmap), key=itemgetter(0))
             lmap = {i+1:items for i, (l, items) in enumerate(lmap)}
             maps[name] = lmap
 
@@ -352,21 +442,22 @@ def from_xpaths(container, xpaths):
 
         return process_node(tocroot)
 
-    for name, level_item_map in maps.iteritems():
+    for name, level_item_map in iteritems(maps):
         root = container.parsed(name)
-        item_level_map = {e:i for i, elems in level_item_map.iteritems() for e in elems}
+        item_level_map = {e:i for i, elems in iteritems(level_item_map) for e in elems}
         item_dirtied = False
+        all_ids = set(root.xpath('//*/@id'))
 
         for item in root.iterdescendants(etree.Element):
             lvl = item_level_map.get(item, None)
             if lvl is None:
                 continue
-            text = elem_to_toc_text(item)
+            text = elem_to_toc_text(item, prefer_title)
             parent = parent_for_level(lvl)
             if item_at_top(item):
                 dirtied, elem_id = False, None
             else:
-                dirtied, elem_id = ensure_id(item)
+                dirtied, elem_id = ensure_id(item, all_ids)
             item_dirtied = dirtied or item_dirtied
             toc = parent.add(text, name, elem_id)
             node_level_map[toc] = lvl
@@ -377,6 +468,7 @@ def from_xpaths(container, xpaths):
 
     return tocroot
 
+
 def from_links(container):
     '''
     Generate a Table of Contents from links in the book.
@@ -384,15 +476,20 @@ def from_links(container):
     toc = TOC()
     link_path = XPath('//h:a[@href]')
     seen_titles, seen_dests = set(), set()
-    for spinepath in container.spine_items:
-        name = container.abspath_to_name(spinepath)
+    for name, is_linear in container.spine_names:
         root = container.parsed(name)
         for a in link_path(root):
             href = a.get('href')
             if not href or not href.strip():
                 continue
-            dest = container.href_to_name(href, base=name)
-            frag = href.rpartition('#')[-1] or None
+            frag = None
+            if href.startswith('#'):
+                dest = name
+                frag = href[1:]
+            else:
+                href, _, frag = href.partition('#')
+                dest = container.href_to_name(href, base=name)
+            frag = frag or None
             if (dest, frag) in seen_dests:
                 continue
             seen_dests.add((dest, frag))
@@ -407,6 +504,7 @@ def from_links(container):
             toc.remove(child)
     return toc
 
+
 def find_text(node):
     LIMIT = 200
     pat = re.compile(r'\s+')
@@ -423,6 +521,7 @@ def find_text(node):
             else:
                 return text
 
+
 def from_files(container):
     '''
     Generate a Table of Contents from files in the book.
@@ -442,6 +541,7 @@ def from_files(container):
         toc.add(text, name)
     return toc
 
+
 def node_from_loc(root, locs, totals=None):
     node = root.xpath('//*[local-name()="body"]')[0]
     for i, loc in enumerate(locs):
@@ -451,6 +551,7 @@ def node_from_loc(root, locs, totals=None):
         node = children[loc]
     return node
 
+
 def add_id(container, name, loc, totals=None):
     root = container.parsed(name)
     try:
@@ -469,10 +570,12 @@ def add_id(container, name, loc, totals=None):
                                     ' before editing.') % name)
         container.replace(name, root)
 
-    node.set('id', node.get('id', uuid_id()))
+    if not node.get('id'):
+        ensure_id(node, set(root.xpath('//*/@id')))
     container.commit_item(name, keep_parsed=True)
     return node.get('id')
 
+
 def create_ncx(toc, to_href, btitle, lang, uid):
     lang = lang.replace('_', '-')
     ncx = etree.Element(NCX('ncx'),
@@ -480,7 +583,7 @@ def create_ncx(toc, to_href, btitle, lang, uid):
         nsmap={None: NCX_NS})
     head = etree.SubElement(ncx, NCX('head'))
     etree.SubElement(head, NCX('meta'),
-        name='dtb:uid', content=unicode(uid))
+        name='dtb:uid', content=str(uid))
     etree.SubElement(head, NCX('meta'),
         name='dtb:depth', content=str(toc.depth))
     generator = ''.join(['calibre (', __version__, ')'])
@@ -552,19 +655,10 @@ def commit_ncx_toc(container, toc, lang=None, uid=None):
     container.replace(tocname, root)
     container.pretty_print.add(tocname)
 
-def commit_nav_toc(container, toc, lang=None):
-    from calibre.ebooks.oeb.polish.pretty import pretty_xml_tree
-    tocname = find_existing_nav_toc(container)
-    if tocname is None:
-        item = container.generate_item('nav.xhtml', id_prefix='nav')
-        item.set('properties', 'nav')
-        tocname = container.href_to_name(item.get('href'), base=container.opf_name)
-    try:
-        root = container.parsed(tocname)
-    except KeyError:
-        root = container.parse_xhtml(P('templates/new_nav.html', data=True).decode('utf-8'))
+
+def ensure_single_nav_of_type(root, ntype='toc'):
     et = '{%s}type' % EPUB_NS
-    navs = [n for n in root.iterdescendants(XHTML('nav')) if n.get(et) == 'toc']
+    navs = [n for n in root.iterdescendants(XHTML('nav')) if n.get(et) == ntype]
     for x in navs[1:]:
         extract(x)
     if navs:
@@ -577,7 +671,71 @@ def commit_nav_toc(container, toc, lang=None):
     else:
         nav = root.makeelement(XHTML('nav'))
         first_child(root, XHTML('body')).append(nav)
-    nav.set('{%s}type' % EPUB_NS, 'toc')
+    nav.set('{%s}type' % EPUB_NS, ntype)
+    return nav
+
+
+def ensure_container_has_nav(container, lang=None, previous_nav=None):
+    tocname = find_existing_nav_toc(container)
+    if previous_nav is not None:
+        nav_name = container.href_to_name(previous_nav[0])
+        if nav_name and container.exists(nav_name):
+            tocname = nav_name
+            container.apply_unique_properties(tocname, 'nav')
+    if tocname is None:
+        item = container.generate_item('nav.xhtml', id_prefix='nav')
+        item.set('properties', 'nav')
+        tocname = container.href_to_name(item.get('href'), base=container.opf_name)
+        if previous_nav is not None:
+            root = previous_nav[1]
+        else:
+            root = container.parse_xhtml(P('templates/new_nav.html', data=True).decode('utf-8'))
+        container.replace(tocname, root)
+    else:
+        root = container.parsed(tocname)
+    if lang:
+        lang = lang_as_iso639_1(lang) or lang
+        root.set('lang', lang)
+        root.set('{%s}lang' % XML_NS, lang)
+    return tocname, root
+
+
+def collapse_li(parent):
+    for li in parent.iterdescendants(XHTML('li')):
+        if len(li) == 1:
+            li.text = None
+            li[0].tail = None
+
+
+def create_nav_li(container, ol, entry, tocname):
+    li = ol.makeelement(XHTML('li'))
+    ol.append(li)
+    a = li.makeelement(XHTML('a'))
+    li.append(a)
+    href = container.name_to_href(entry['dest'], tocname)
+    if entry['frag']:
+        href += '#' + entry['frag']
+    a.set('href', href)
+    return a
+
+
+def set_landmarks(container, root, tocname, landmarks):
+    nav = ensure_single_nav_of_type(root, 'landmarks')
+    nav.set('hidden', '')
+    ol = nav.makeelement(XHTML('ol'))
+    nav.append(ol)
+    for entry in landmarks:
+        if entry['type'] and container.has_name(entry['dest']) and container.mime_map[entry['dest']] in OEB_DOCS:
+            a = create_nav_li(container, ol, entry, tocname)
+            a.set('{%s}type' % EPUB_NS, entry['type'])
+            a.text = entry['title'] or None
+    pretty_xml_tree(nav)
+    collapse_li(nav)
+
+
+def commit_nav_toc(container, toc, lang=None, landmarks=None, previous_nav=None):
+    tocname, root = ensure_container_has_nav(container, lang=lang, previous_nav=previous_nav)
+    nav = ensure_single_nav_of_type(root, 'toc')
     if toc.toc_title:
         nav.append(nav.makeelement(XHTML('h1')))
         nav[-1].text = toc.toc_title
@@ -606,18 +764,31 @@ def commit_nav_toc(container, toc, lang=None):
                 li.append(ol)
                 process_node(ol, child)
     process_node(rnode, toc)
-    pretty_xml_tree(rnode)
-    for li in rnode.iterdescendants(XHTML('li')):
-        if len(li) == 1:
-            li.text = None
-            li[0].tail = None
+    pretty_xml_tree(nav)
+
+    collapse_li(nav)
+    nav.tail = '\n'
+
+    if toc.page_list:
+        nav = ensure_single_nav_of_type(root, 'page-list')
+        nav.set('hidden', '')
+        ol = nav.makeelement(XHTML('ol'))
+        nav.append(ol)
+        for entry in toc.page_list:
+            if container.has_name(entry['dest']) and container.mime_map[entry['dest']] in OEB_DOCS:
+                a = create_nav_li(container, ol, entry, tocname)
+                a.text = str(entry['pagenum'])
+        pretty_xml_tree(nav)
+        collapse_li(nav)
     container.replace(tocname, root)
 
+
 def commit_toc(container, toc, lang=None, uid=None):
     commit_ncx_toc(container, toc, lang=lang, uid=uid)
     if container.opf_version_parsed.major > 2:
         commit_nav_toc(container, toc, lang=lang)
 
+
 def remove_names_from_toc(container, names):
     changed = []
     names = frozenset(names)
@@ -638,11 +809,13 @@ def remove_names_from_toc(container, names):
                 changed.append(find_toc(container))
     return changed
 
+
 def find_inline_toc(container):
     for name, linear in container.spine_names:
         if container.parsed(name).xpath('//*[local-name()="body" and @id="calibre_generated_inline_toc"]'):
             return name
 
+
 def toc_to_html(toc, container, toc_name, title, lang=None):
 
     def process_node(html_parent, toc, level=1, indent='  ', style_level=2):
@@ -673,7 +846,7 @@ def toc_to_html(toc, container, toc_name, title, lang=None):
     html = E.html(
         E.head(
             E.title(title),
-            E.style(P('templates/inline_toc_styles.css', data=True), type='text/css'),
+            E.style(P('templates/inline_toc_styles.css', data=True).decode('utf-8'), type='text/css'),
         ),
         E.body(
             E.h2(title),
@@ -691,9 +864,10 @@ def toc_to_html(toc, container, toc_name, title, lang=None):
     pretty_html_tree(container, html)
     return html
 
+
 def create_inline_toc(container, title=None):
     '''
-    Create an inline (HTML) Table of Contents from an existing NCX table of contents.
+    Create an inline (HTML) Table of Contents from an existing NCX Table of Contents.
 
     :param title: The title for this table of contents.
     '''
@@ -722,4 +896,3 @@ def create_inline_toc(container, title=None):
             f.write(raw)
     set_guide_item(container, 'toc', title, name, frag='calibre_generated_inline_toc')
     return name
-
