@@ -1,5 +1,3 @@
-
-
 #########################################################################
 #                                                                       #
 #                                                                       #
@@ -163,7 +161,7 @@ class PreambleDiv:
 
     def __make_default_font_table(self):
         """
-        If not font table is fount, need to write one out.
+        If not font table is found, need to write one out.
         """
         self.__font_table_final = 'mi<tg<open______<font-table\n'
         self.__font_table_final += 'mi<mk<fonttb-beg\n'
@@ -423,7 +421,7 @@ cw<ci<font-style<nu<0
         self.__write_obj.write('mi<tg<empty-att_<page-definition')
         for key in self.__page.keys():
             self.__write_obj.write(
-            '<%s>%s' % (key, self.__page[key])
+            f'<{key}>{self.__page[key]}'
             )
         self.__write_obj.write('\n')
 # mi<tg<open-att__<footn
