@@ -1,35 +1,38 @@
-#!/usr/bin/env python2
-# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
+#!/usr/bin/env python
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
 import os
-from functools import partial
 from collections import defaultdict
+from functools import partial
+from qt.core import QApplication, QDialog, QPixmap, QTimer
 
-from PyQt5.Qt import QPixmap, QTimer
-
-from calibre import as_unicode
-from calibre.gui2 import (error_dialog, choose_files, choose_dir,
-        warning_dialog, info_dialog, gprefs)
+from calibre import as_unicode, guess_type, prepare_string_for_xml
+from calibre.constants import iswindows
+from calibre.ebooks import BOOK_EXTENSIONS
+from calibre.ebooks.metadata import MetaInformation, normalize_isbn
+from calibre.gui2 import (
+    choose_dir, choose_files, choose_files_and_remember_all_files, error_dialog,
+    gprefs, info_dialog, question_dialog, warning_dialog
+)
+from calibre.gui2.actions import InterfaceAction
 from calibre.gui2.dialogs.add_empty_book import AddEmptyBookDialog
 from calibre.gui2.dialogs.confirm_delete import confirm
 from calibre.gui2.dialogs.progress import ProgressDialog
-from calibre.ebooks import BOOK_EXTENSIONS
-from calibre.utils.config_base import tweaks
-from calibre.utils.filenames import ascii_filename
-from calibre.utils.icu import sort_key
-from calibre.gui2.actions import InterfaceAction
-from calibre.gui2 import question_dialog
-from calibre.ebooks.metadata import MetaInformation
 from calibre.ptempfile import PersistentTemporaryFile
+from calibre.utils.config_base import tweaks
+from calibre.utils.filenames import ascii_filename, make_long_path_useable
+from calibre.utils.icu import sort_key
+from polyglot.builtins import iteritems, string_or_bytes
 
 
 def get_filters():
+    archives = ['zip', 'rar']
     return [
-            (_('Books'), BOOK_EXTENSIONS),
+            (_('Books'), [x for x in BOOK_EXTENSIONS if x not in archives]),
             (_('EPUB books'), ['epub', 'kepub']),
             (_('Kindle books'), ['mobi', 'prc', 'azw', 'azw3', 'kfx', 'tpz', 'azw1', 'azw4']),
             (_('PDF books'), ['pdf', 'azw4']),
@@ -37,7 +40,7 @@ def get_filters():
             (_('LIT books'), ['lit']),
             (_('Text books'), ['txt', 'text', 'rtf', 'md', 'markdown', 'textile', 'txtz']),
             (_('Comics'), ['cbz', 'cbr', 'cbc']),
-            (_('Archives'), ['zip', 'rar']),
+            (_('Archives'), archives),
             (_('Wordprocessor files'), ['odt', 'doc', 'docx']),
     ]
 
@@ -50,42 +53,28 @@ class AddAction(InterfaceAction):
             , _('A'))
     action_type = 'current'
     action_add_menu = True
-    action_menu_clone_qaction = _('Add books from a single directory')
+    action_menu_clone_qaction = _('Add books from a single folder')
 
     def genesis(self):
         self._add_filesystem_book = self.Dispatcher(self.__add_filesystem_book)
         self.add_menu = self.qaction.menu()
         ma = partial(self.create_menu_action, self.add_menu)
-        ma('recursive-single', _('Add books from directories, including '
-            'sub-directories (One book per directory, assumes every e-book '
-            'file is the same book in a different format)')).triggered.connect(
-            self.add_recursive_single)
-        ma('recursive-multiple', _('Add books from directories, including '
-            'sub-directories (Multiple books per directory, assumes every '
-            'e-book file is a different book)')).triggered.connect(
-                    self.add_recursive_multiple)
-        arm = self.add_archive_menu = self.add_menu.addMenu(_('Add multiple books from archive (ZIP/RAR)'))
-        self.create_menu_action(arm, 'recursive-single-archive', _(
-            'One book per directory in the archive')).triggered.connect(partial(self.add_archive, True))
-        self.create_menu_action(arm, 'recursive-multiple-archive', _(
-            'Multiple books per directory in the archive')).triggered.connect(partial(self.add_archive, False))
+        ma('recursive-add', _('Add from folders and sub-folders'), icon='mimetypes/dir.png').triggered.connect(self.add_recursive_question)
+        ma('archive-add-book', _('Add multiple books from archive (ZIP/RAR)'), icon='mimetypes/zip.png').triggered.connect(self.add_from_archive)
         self.add_menu.addSeparator()
         ma('add-empty', _('Add empty book (Book entry with no formats)'),
                 shortcut='Shift+Ctrl+E').triggered.connect(self.add_empty)
-        ma('add-isbn', _('Add from ISBN')).triggered.connect(self.add_from_isbn)
+        ma('add-isbn', _('Add from ISBN'), icon='identifiers.png').triggered.connect(self.add_from_isbn)
         self.add_menu.addSeparator()
         ma('add-formats', _('Add files to selected book records'),
                 triggered=self.add_formats, shortcut='Shift+A')
-        arm = self.add_archive_menu = self.add_menu.addMenu(_('Add an empty file to selected book records'))
-        from calibre.ebooks.oeb.polish.create import valid_empty_formats
-        for fmt in sorted(valid_empty_formats):
-            self.create_menu_action(arm, 'add-empty-' + fmt,
-                                    _('Add empty {}').format(fmt.upper())).triggered.connect(
-                                         partial(self.add_empty_format, fmt))
+        ma('add-formats-clipboard', _('Add files to selected book records from clipboard'),
+                triggered=self.add_formats_from_clipboard, shortcut='Shift+Alt+A', icon='edit-paste.png')
+        ma('add-empty-format-to-books', _(
+            'Add an empty file to selected book records')).triggered.connect(self.add_empty_format_choose)
         self.add_menu.addSeparator()
-        ma('add-config', _('Control the adding of books'),
+        ma('add-config', _('Control the adding of books'), icon='config.png',
                 triggered=self.add_config)
-
         self.qaction.triggered.connect(self.add_books)
 
     def location_selected(self, loc):
@@ -98,16 +87,64 @@ class AddAction(InterfaceAction):
             initial_plugin=('Import/Export', 'Adding'),
             close_after_initial=True)
 
-    def add_formats(self, *args):
-        if self.gui.stack.currentIndex() != 0:
-            return
+    def _check_add_formats_ok(self):
+        if self.gui.current_view() is not self.gui.library_view:
+            return []
         view = self.gui.library_view
         rows = view.selectionModel().selectedRows()
         if not rows:
-            return error_dialog(self.gui, _('No books selected'),
+            error_dialog(self.gui, _('No books selected'),
                     _('Cannot add files as no books are selected'), show=True)
         ids = [view.model().id(r) for r in rows]
+        return ids
 
+    def add_formats_from_clipboard(self):
+        ids = self._check_add_formats_ok()
+        if not ids:
+            return
+        md = QApplication.instance().clipboard().mimeData()
+        files_to_add = []
+        images = []
+        if md.hasUrls():
+            for url in md.urls():
+                if url.isLocalFile():
+                    path = url.toLocalFile()
+                    if os.access(path, os.R_OK):
+                        mt = guess_type(path)[0]
+                        if mt and mt.startswith('image/'):
+                            images.append(path)
+                        else:
+                            files_to_add.append(path)
+        if not files_to_add and not images:
+            return error_dialog(self.gui, _('No files in clipboard'),
+                    _('No files have been copied to the clipboard'), show=True)
+        if files_to_add:
+            self._add_formats(files_to_add, ids)
+        if images:
+            if len(ids) > 1 and not question_dialog(
+                    self.gui,
+                    _('Are you sure?'),
+                    _('Are you sure you want to set the same'
+                    ' cover for all %d books?')%len(ids)):
+                return
+            with lopen(images[0], 'rb') as f:
+                cdata = f.read()
+            self.gui.current_db.new_api.set_cover({book_id: cdata for book_id in ids})
+            self.gui.refresh_cover_browser()
+            m = self.gui.library_view.model()
+            current = self.gui.library_view.currentIndex()
+            m.current_changed(current, current)
+
+    def add_formats(self, *args):
+        ids = self._check_add_formats_ok()
+        if not ids:
+            return
+        books = choose_files_and_remember_all_files(self.gui, 'add formats dialog dir',
+                _('Select book files'), filters=get_filters())
+        if books:
+            self._add_formats(books, ids)
+
+    def _add_formats(self, paths, ids):
         if len(ids) > 1 and not question_dialog(
                 self.gui,
                 _('Are you sure?'),
@@ -115,18 +152,14 @@ class AddAction(InterfaceAction):
                   ' files to all %d books? If the format'
                   ' already exists for a book, it will be replaced.')%len(ids)):
             return
+        paths = list(map(make_long_path_useable, paths))
 
-        books = choose_files(self.gui, 'add formats dialog dir',
-                _('Select book files'), filters=get_filters())
-        if not books:
-            return
-
-        db = view.model().db
+        db = self.gui.current_db
         if len(ids) == 1:
             formats = db.formats(ids[0], index_is_id=True)
             if formats:
                 formats = {x.upper() for x in formats.split(',')}
-                nformats = {f.rpartition('.')[-1].upper() for f in books}
+                nformats = {f.rpartition('.')[-1].upper() for f in paths}
                 override = formats.intersection(nformats)
                 if override:
                     title = db.title(ids[0], index_is_id=True)
@@ -137,18 +170,18 @@ class AddAction(InterfaceAction):
                     if not confirm(msg, 'confirm_format_override_on_add', title=_('Are you sure?'), parent=self.gui):
                         return
 
-        fmt_map = {os.path.splitext(fpath)[1][1:].upper():fpath for fpath in books}
+        fmt_map = {os.path.splitext(fpath)[1][1:].upper():fpath for fpath in paths}
 
         for id_ in ids:
-            for fmt, fpath in fmt_map.iteritems():
+            for fmt, fpath in iteritems(fmt_map):
                 if fmt:
                     db.add_format_with_hooks(id_, fmt, fpath, index_is_id=True,
                         notify=True)
         current_idx = self.gui.library_view.currentIndex()
         if current_idx.isValid():
-            view.model().current_changed(current_idx, current_idx)
+            self.gui.library_view.model().current_changed(current_idx, current_idx)
 
-    def add_empty_format(self, format_):
+    def is_ok_to_add_empty_formats(self):
         if self.gui.stack.currentIndex() != 0:
             return
         view = self.gui.library_view
@@ -166,7 +199,27 @@ class AddAction(InterfaceAction):
                   ' empty file to all %d books? If the format'
                   ' already exists for a book, it will be replaced.')%len(ids)):
             return
+        return True
 
+    def add_empty_format_choose(self):
+        if not self.is_ok_to_add_empty_formats():
+            return
+        from calibre.ebooks.oeb.polish.create import valid_empty_formats
+        from calibre.gui2.dialogs.choose_format import ChooseFormatDialog
+        d = ChooseFormatDialog(self.gui, _('Choose format of empty file'), sorted(valid_empty_formats))
+        if d.exec() != QDialog.DialogCode.Accepted or not d.format():
+            return
+        self._add_empty_format(d.format())
+
+    def add_empty_format(self, format_):
+        if not self.is_ok_to_add_empty_formats():
+            return
+        self._add_empty_format(format_)
+
+    def _add_empty_format(self, format_):
+        view = self.gui.library_view
+        rows = view.selectionModel().selectedRows()
+        ids = [view.model().id(r) for r in rows]
         db = self.gui.library_view.model().db
         if len(ids) == 1:
             formats = db.formats(ids[0], index_is_id=True)
@@ -174,28 +227,32 @@ class AddAction(InterfaceAction):
                 formats = {x.lower() for x in formats.split(',')}
                 if format_ in formats:
                     title = db.title(ids[0], index_is_id=True)
-                    msg = _('The {0} format will be replaced in the book {1}. Are you sure?').format(
+                    msg = _('The {0} format will be replaced in the book: {1}. Are you sure?').format(
                         format_, title)
                     if not confirm(msg, 'confirm_format_override_on_add', title=_('Are you sure?'),
                                    parent=self.gui):
                         return
 
         for id_ in ids:
-            from calibre.ebooks.oeb.polish.create import create_book
-            pt = PersistentTemporaryFile(suffix='.' + format_)
-            pt.close()
-            try:
-                mi = db.new_api.get_metadata(id_, get_cover=False,
-                                    get_user_categories=False, cover_as_data=False)
-                create_book(mi, pt.name, fmt=format_)
-                db.add_format_with_hooks(id_, format_, pt.name, index_is_id=True, notify=True)
-            finally:
-                os.remove(pt.name)
+            self.add_empty_format_to_book(id_, format_)
 
         current_idx = self.gui.library_view.currentIndex()
         if current_idx.isValid():
             view.model().current_changed(current_idx, current_idx)
 
+    def add_empty_format_to_book(self, book_id, fmt):
+        from calibre.ebooks.oeb.polish.create import create_book
+        db = self.gui.current_db
+        pt = PersistentTemporaryFile(suffix='.' + fmt.lower())
+        pt.close()
+        try:
+            mi = db.new_api.get_metadata(book_id, get_cover=False,
+                                get_user_categories=False, cover_as_data=False)
+            create_book(mi, pt.name, fmt=fmt.lower())
+            db.add_format_with_hooks(book_id, fmt, pt.name, index_is_id=True, notify=True)
+        finally:
+            os.remove(pt.name)
+
     def add_archive(self, single):
         paths = choose_files(
             self.gui, 'recursive-archive-add', _('Choose archive file'),
@@ -203,6 +260,15 @@ class AddAction(InterfaceAction):
         if paths:
             self.do_add_recursive(paths, single, list_of_archives=True)
 
+    def add_from_archive(self):
+        single = question_dialog(self.gui, _('Type of archive'), _(
+            'Will the archive have a single book per internal folder?'))
+        paths = choose_files(
+            self.gui, 'recursive-archive-add', _('Choose archive file'),
+            filters=[(_('Archives'), ('zip', 'rar'))], all_files=False, select_only_single_file=False)
+        if paths:
+            self.do_add_recursive(paths, single, list_of_archives=True)
+
     def add_recursive(self, single):
         root = choose_dir(self.gui, 'recursive book import root dir dialog',
                           _('Select root folder'))
@@ -233,6 +299,11 @@ class AddAction(InterfaceAction):
         '''
         self.add_recursive(False)
 
+    def add_recursive_question(self):
+        single =  question_dialog(self.gui, _('Multi-file books?'), _(
+            'Assume all e-book files in a single folder are multiple formats of the same book?'))
+        self.add_recursive(single)
+
     def add_empty(self, *args):
         '''
         Add an empty book item to the library. This does not import any formats
@@ -250,7 +321,7 @@ class AddAction(InterfaceAction):
             title = index.model().db.title(index.row())
         dlg = AddEmptyBookDialog(self.gui, self.gui.library_view.model().db,
                                  author, series, dup_title=title)
-        if dlg.exec_() == dlg.Accepted:
+        if dlg.exec() == QDialog.DialogCode.Accepted:
             temp_files = []
             num = dlg.qty_to_add
             series = dlg.selected_series
@@ -263,7 +334,7 @@ class AddAction(InterfaceAction):
                     book_id = db.id(index.row())
                     orig_fmts = tuple(db.new_api.format(book_id, fmt, as_path=True) for fmt in db.new_api.formats(book_id))
 
-            for x in xrange(num):
+            for x in range(num):
                 if dlg.duplicate_current_book:
                     mi = origmi
                 else:
@@ -283,25 +354,51 @@ class AddAction(InterfaceAction):
                     create_book(mi, pt.name, fmt=empty_format)
                     fmts = [pt.name]
                 ids.append(db.import_book(mi, fmts))
-            tuple(map(os.remove, orig_fmts))
-            self.gui.library_view.model().books_added(num)
-            self.gui.refresh_cover_browser()
-            self.gui.tags_view.recount()
+            for path in orig_fmts:
+                os.remove(path)
+            self.refresh_gui(num)
             if ids:
                 ids.reverse()
                 self.gui.library_view.select_rows(ids)
             for path in temp_files:
                 os.remove(path)
 
-    def add_isbns(self, books, add_tags=[]):
-        self.isbn_books = list(books)
+    def check_for_existing_isbns(self, books):
+        db = self.gui.current_db.new_api
+        book_id_identifiers = db.all_field_for('identifiers', db.all_book_ids(tuple))
+        existing_isbns = {normalize_isbn(ids.get('isbn', '')): book_id for book_id, ids in book_id_identifiers.items()}
+        existing_isbns.pop('', None)
+        ok = []
+        duplicates = []
+        for book in books:
+            q = normalize_isbn(book['isbn'])
+            if q and q in existing_isbns:
+                duplicates.append((book, existing_isbns[q]))
+            else:
+                ok.append(book)
+        if duplicates:
+            det_msg = '\n'.join(f'{book["isbn"]}: {db.field_for("title", book_id)}' for book, book_id in duplicates)
+            if question_dialog(self.gui, _('Duplicates found'), _(
+                'Books with some of the specified ISBNs already exist in the calibre library.'
+                ' Click "Show details" for the full list. Do you want to add them anyway?'), det_msg=det_msg
+            ):
+                ok += [x[0] for x in duplicates]
+        return ok
+
+    def add_isbns(self, books, add_tags=[], check_for_existing=False):
+        books = list(books)
+        if check_for_existing:
+            books = self.check_for_existing_isbns(books)
+            if not books:
+                return
+        self.isbn_books = books
         self.add_by_isbn_ids = set()
         self.isbn_add_tags = add_tags
         QTimer.singleShot(10, self.do_one_isbn_add)
         self.isbn_add_dialog = ProgressDialog(_('Adding'),
                 _('Creating book records from ISBNs'), max=len(books),
                 cancelable=False, parent=self.gui)
-        self.isbn_add_dialog.exec_()
+        self.isbn_add_dialog.exec()
 
     def do_one_isbn_add(self):
         try:
@@ -361,6 +458,9 @@ class AddAction(InterfaceAction):
         formats = []
         from calibre.gui2.dnd import image_extensions
         image_exts = set(image_extensions()) - set(tweaks['cover_drop_exclude'])
+        if iswindows:
+            from calibre.gui2.add import resolve_windows_links
+            paths = list(resolve_windows_links(paths, hwnd=int(self.gui.effectiveWinId())))
         for path in paths:
             ext = os.path.splitext(path)[1].lower()
             if ext:
@@ -377,11 +477,16 @@ class AddAction(InterfaceAction):
                 accept = True
         if accept and event is not None:
             event.accept()
+        add_as_book = False
         if do_confirm and formats:
-            if not confirm(
+            ok, add_as_book = confirm(
                 _('You have dropped some files onto the book <b>%s</b>. This will'
                   ' add or replace the files for this book. Do you want to proceed?') % db.title(cid, index_is_id=True),
-                'confirm_drop_on_book', parent=self.gui):
+                'confirm_drop_on_book', parent=self.gui,
+                extra_button=ngettext('Add as new book', 'Add as new books', len(formats)))
+            if ok and add_as_book:
+                add_as_book = [path for ext, path in formats]
+            if not ok or add_as_book:
                 formats = []
         for ext, path in formats:
             db.add_format_with_hooks(cid, ext, path, index_is_id=True)
@@ -389,9 +494,11 @@ class AddAction(InterfaceAction):
             self.gui.library_view.model().current_changed(current_idx, current_idx)
         if cover_changed:
             self.gui.refresh_cover_browser()
+        if add_as_book:
+            self.files_dropped(add_as_book)
 
     def __add_filesystem_book(self, paths, allow_device=True):
-        if isinstance(paths, basestring):
+        if isinstance(paths, string_or_bytes):
             paths = [paths]
         books = [path for path in map(os.path.abspath, paths) if os.access(path,
             os.R_OK)]
@@ -409,8 +516,8 @@ class AddAction(InterfaceAction):
     def add_from_isbn(self, *args):
         from calibre.gui2.dialogs.add_from_isbn import AddFromISBN
         d = AddFromISBN(self.gui)
-        if d.exec_() == d.Accepted and d.books:
-            self.add_isbns(d.books, add_tags=d.set_tags)
+        if d.exec() == QDialog.DialogCode.Accepted and d.books:
+            self.add_isbns(d.books, add_tags=d.set_tags, check_for_existing=d.check_for_existing)
 
     def add_books(self, *args):
         '''
@@ -422,7 +529,7 @@ class AddAction(InterfaceAction):
             fmts = self.gui.device_manager.device.settings().format_map
             filters = [(_('Supported books'), fmts)]
 
-        books = choose_files(self.gui, 'add books dialog dir',
+        books = choose_files_and_remember_all_files(self.gui, 'add books dialog dir',
                 _('Select books'), filters=filters)
         if not books:
             return
@@ -438,6 +545,14 @@ class AddAction(InterfaceAction):
         Adder(paths, db=None if to_device else self.gui.current_db,
               parent=self.gui, callback=partial(self._files_added, on_card=on_card), pool=self.gui.spare_pool())
 
+    def refresh_gui(self, num, set_current_row=-1, recount=True):
+        self.gui.library_view.model().books_added(num)
+        if set_current_row > -1:
+            self.gui.library_view.set_current_row(0)
+        self.gui.refresh_cover_browser()
+        if recount:
+            self.gui.tags_view.recount()
+
     def _files_added(self, adder, on_card=None):
         if adder.items:
             paths, infos, names = [], [], []
@@ -451,10 +566,7 @@ class AddAction(InterfaceAction):
             return
 
         if adder.number_of_books_added > 0:
-            self.gui.library_view.model().books_added(adder.number_of_books_added)
-            self.gui.library_view.set_current_row(0)
-            self.gui.refresh_cover_browser()
-            self.gui.tags_view.recount()
+            self.refresh_gui(adder.number_of_books_added, set_current_row=0)
 
         if adder.merged_books:
             merged = defaultdict(list)
@@ -462,18 +574,18 @@ class AddAction(InterfaceAction):
                 merged[author].append(title)
             lines = []
             for author in sorted(merged, key=sort_key):
-                lines.append(author)
-                for title in sorted(merged[author], key=sort_key):
-                    lines.append('\t' + title)
-                lines.append('')
+                lines.append(f'<b><i>{prepare_string_for_xml(author)}</i></b><ol style="margin-top: 0">')
+                for title in sorted(merged[author]):
+                    lines.append(f'<li>{prepare_string_for_xml(title)}</li>')
+                lines.append('</ol>')
             pm = ngettext('The following duplicate book was found.',
                           'The following {} duplicate books were found.',
                           len(adder.merged_books)).format(len(adder.merged_books))
             info_dialog(self.gui, _('Merged some books'), pm + ' ' +
                 _('Incoming book formats were processed and merged into your '
                     'calibre database according to your auto-merge '
-                    'settings:'),
-                    det_msg='\n'.join(lines), show=True)
+                    'settings. Click "Show details" to see the list of merged books.'),
+                    det_msg='\n'.join(lines), show=True, only_copy_details=True)
 
         if adder.number_of_books_added > 0 or adder.merged_books:
             # The formats of the current book could have changed if
@@ -500,7 +612,7 @@ class AddAction(InterfaceAction):
             rows = view.selectionModel().selectedRows()
             if not rows or len(rows) == 0:
                 d = error_dialog(self.gui, _('Add to library'), _('No book selected'))
-                d.exec_()
+                d.exec()
                 return
             paths = [p for p in view.model().paths(rows) if p is not None]
         ve = self.gui.device_manager.device.VIRTUAL_BOOK_EXTENSIONS
@@ -509,7 +621,7 @@ class AddAction(InterfaceAction):
             ans = os.path.splitext(x)[1]
             ans = ans[1:] if len(ans) > 1 else ans
             return ans.lower()
-        remove = set([p for p in paths if ext(p) in ve])
+        remove = {p for p in paths if ext(p) in ve}
         if remove:
             paths = [p for p in paths if p not in remove]
             vmsg = getattr(self.gui.device_manager.device, 'VIRTUAL_BOOK_EXTENSION_MESSAGE', None) or _(
@@ -520,7 +632,7 @@ class AddAction(InterfaceAction):
                 return
         if not paths or len(paths) == 0:
             d = error_dialog(self.gui, _('Add to library'), _('No book files found'))
-            d.exec_()
+            d.exec()
             return
 
         self.gui.device_manager.prepare_addable_books(self.Dispatcher(partial(
@@ -541,7 +653,7 @@ class AddAction(InterfaceAction):
             self.gui.device_job_exception(job)
             return
         paths = job.result
-        ok_paths = [x for x in paths if isinstance(x, basestring)]
+        ok_paths = [x for x in paths if isinstance(x, string_or_bytes)]
         failed_paths = [x for x in paths if isinstance(x, tuple)]
         if failed_paths:
             if not ok_paths:
