@@ -1,10 +1,9 @@
 #!/usr/bin/env python
-# vim:fileencoding=utf-8
 # License: GPLv3 Copyright: 2015, Kovid Goyal <kovid at kovidgoyal.net>
 
 
 from qt.core import QDialog
-from polyglot.builtins import iteritems, map, range
+from polyglot.builtins import iteritems
 
 from calibre.gui2 import gprefs
 from calibre.gui2.actions import InterfaceAction
@@ -43,7 +42,7 @@ class TagMapAction(InterfaceAction):
             'The changes will be applied to <b>one book in the library</b>',
             'The changes will be applied to <b>{} books in the library</b>', len(book_ids))
         d.edit_widget.msg_label.setText(d.edit_widget.msg_label.text() + '<p>' + txt.format(len(book_ids)))
-        if d.exec_() != QDialog.DialogCode.Accepted:
+        if d.exec() != QDialog.DialogCode.Accepted:
             return
         with BusyCursor():
             rules = d.rules
