@@ -1,5 +1,4 @@
 #!/usr/bin/env python
-# vim:fileencoding=utf-8
 
 
 __license__ = 'GPL v3'
@@ -33,7 +32,7 @@ from calibre.utils import join_with_timeout
 from calibre.utils.config import prefs
 from calibre.utils.filenames import make_long_path_useable
 from calibre.utils.ipc.pool import Failure, Pool
-from polyglot.builtins import iteritems, map, string_or_bytes, unicode_type
+from polyglot.builtins import iteritems, string_or_bytes
 from polyglot.queue import Empty
 
 
@@ -180,7 +179,7 @@ class Adder(QObject):
                             self.file_groups[len(self.file_groups)] = files
         else:
             def find_files(root):
-                if isinstance(root, unicode_type):
+                if isinstance(root, str):
                     root = root.encode(filesystem_encoding)
                 for dirpath, dirnames, filenames in os.walk(root):
                     try:
@@ -314,7 +313,7 @@ class Adder(QObject):
             except Failure as err:
                 error_dialog(self.pd, _('Cannot add books'), _(
                 'Failed to add some books, click "Show details" for more information.'),
-                det_msg=unicode_type(err.failure_message) + '\n' + unicode_type(err.details), show=True)
+                det_msg=str(err.failure_message) + '\n' + str(err.details), show=True)
                 self.pd.canceled = True
             else:
                 # All tasks completed
@@ -524,7 +523,7 @@ class Adder(QObject):
 
     def finish(self):
         if DEBUG:
-            prints('Added %s books in %.1f seconds' % (len(self.added_book_ids or self.items), time.time() - self.start_time))
+            prints(f'Added {len(self.added_book_ids or self.items)} books in {time.time() - self.start_time:.1f} seconds')
         if self.report:
             added_some = self.items or self.added_book_ids
             d = warning_dialog if added_some else error_dialog
