@@ -1,12 +1,8 @@
-#!/usr/bin/env python2
-# vim:fileencoding=utf-8
+#!/usr/bin/env python
 # License: GPLv3 Copyright: 2015, Kovid Goyal <kovid at kovidgoyal.net>
 
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
 
 from collections import OrderedDict
-from future_builtins import map
 
 from calibre.db.adding import compile_glob, filter_filename, compile_rule
 from calibre.gui2 import elided_text, Application, error_dialog
@@ -66,15 +62,15 @@ class RuleEdit(RuleEditBase):
     def rule(self, rule):
         def sc(name):
             c = getattr(self, name)
-            idx = c.findData(unicode(rule.get(name, '')))
+            idx = c.findData(str(rule.get(name, '')))
             if idx < 0:
                 idx = 0
             c.setCurrentIndex(idx)
         sc('action'), sc('match_type')
-        self.query.setText(unicode(rule.get('query', '')).strip())
+        self.query.setText(str(rule.get('query', '')).strip())
 
     def validate(self):
-        ans = super(RuleEdit, self).validate()
+        ans = super().validate()
         if ans:
             rule = self.rule
             if 'glob' in rule['match_type']:
@@ -109,11 +105,11 @@ class Rules(RulesBase):
     RuleItemClass = RuleItem
     RuleEditDialogClass = RuleEditDialog
     MSG = _('You can specify rules to add/ignore files here. They will be used'
-            ' when recursively adding files from directories/archives and also'
+            ' when recursively adding files from folders/archives and also'
             ' when auto-adding. Click the "Add Rule" button'
             ' below to get started. The rules will be processed in order for every file until either an'
             ' "add" or an "ignore" rule matches. If no rules match, the file will be added only'
-            ' if its file extension is of a known ebook type.')
+            ' if its file extension is of a known e-book type.')
 
 
 class Tester(TesterBase):
@@ -121,7 +117,7 @@ class Tester(TesterBase):
     DIALOG_TITLE = _('Test filename filter rules')
     PREFS_NAME = 'test-file-filter-rules'
     LABEL = _('Enter a filename to test:')
-    PLACEHOLDER = _('Enter filename and click the Test button')
+    PLACEHOLDER = _('Enter filename and click the "Test" button')
     EMPTY_RESULT = '<p>&nbsp;</p>'
 
     def do_test(self):
@@ -141,6 +137,7 @@ class RulesDialog(RulesDialogBase):
     TesterClass = Tester
     PREFS_OBJECT = add_filters
 
+
 if __name__ == '__main__':
     app = Application([])
     d = RulesDialog()
@@ -148,7 +145,7 @@ if __name__ == '__main__':
         {'action':'ignore', 'query':'ignore-me', 'match_type':'startswith'},
         {'action':'add', 'query':'*.moose', 'match_type':'glob'},
     ]
-    d.exec_()
+    d.exec()
     from pprint import pprint
     pprint(d.rules)
     del d, app
