@@ -1,5 +1,4 @@
 #!/usr/bin/env python
-# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
 
 
 __license__   = 'GPL v3'
@@ -15,7 +14,6 @@ from calibre.constants import ismacos
 from calibre.utils.icu import sort_key, primary_startswith, primary_contains
 from calibre.gui2.widgets import EnComboBox, LineEditECM
 from calibre.utils.config import tweaks
-from polyglot.builtins import unicode_type
 
 
 def containsq(x, prefix):
@@ -64,6 +62,11 @@ class CompleteModel(QAbstractListModel):  # {{{
 
     def data(self, index, role):
         if role == Qt.ItemDataRole.DisplayRole:
+            try:
+                return self.current_items[index.row()].replace('\n', ' ')
+            except IndexError:
+                pass
+        if role == Qt.ItemDataRole.UserRole:
             try:
                 return self.current_items[index.row()]
             except IndexError:
@@ -79,6 +82,7 @@ class CompleteModel(QAbstractListModel):  # {{{
 class Completer(QListView):  # {{{
 
     item_selected = pyqtSignal(object)
+    apply_current_text = pyqtSignal()
     relayout_needed = pyqtSignal()
 
     def __init__(self, completer_widget, max_visible_items=7, sort_func=sort_key, strip_completion_entries=True):
@@ -98,6 +102,8 @@ class Completer(QListView):  # {{{
         self.pressed.connect(self.item_chosen)
         self.installEventFilter(self)
         self.setFocusPolicy(Qt.FocusPolicy.NoFocus)
+        self.tab_accepts_uncompleted_text = (tweaks['tab_accepts_uncompleted_text'] and
+                                             not tweaks['preselect_first_completion'])
 
     def hide(self):
         self.setCurrentIndex(QModelIndex())
@@ -107,8 +113,8 @@ class Completer(QListView):  # {{{
         if not self.isVisible():
             return
         self.hide()
-        text = self.model().data(index, Qt.ItemDataRole.DisplayRole)
-        self.item_selected.emit(unicode_type(text))
+        text = self.model().data(index, Qt.ItemDataRole.UserRole)
+        self.item_selected.emit(str(text))
 
     def set_items(self, items):
         self.model().set_items(items)
@@ -144,7 +150,7 @@ class Completer(QListView):  # {{{
         widget = self.parent()
         if widget is None:
             return
-        screen = QApplication.desktop().availableGeometry(widget)
+        screen = widget.screen().availableGeometry()
         h = (p.sizeHintForRow(0) * min(self.max_visible_items, m.rowCount()) + 3) + 3
         hsb = p.horizontalScrollBar()
         if hsb and hsb.isVisible():
@@ -229,6 +235,9 @@ class Completer(QListView):  # {{{
                 if idx.isValid():
                     self.item_chosen(idx)
                     self.hide()
+                elif self.tab_accepts_uncompleted_text:
+                    self.hide()
+                    self.apply_current_text.emit()
                 elif self.model().rowCount() > 0:
                     self.next_match()
                 e.accept()
@@ -291,6 +300,7 @@ class LineEdit(QLineEdit, LineEditECM):
 
     def __init__(self, parent=None, completer_widget=None, sort_func=sort_key, strip_completion_entries=True):
         QLineEdit.__init__(self, parent)
+        self.setClearButtonEnabled(True)
 
         self.sep = ','
         self.space_before_sep = False
@@ -302,6 +312,8 @@ class LineEdit(QLineEdit, LineEditECM):
         self.mcompleter = Completer(completer_widget, sort_func=sort_func, strip_completion_entries=strip_completion_entries)
         self.mcompleter.item_selected.connect(self.completion_selected,
                 type=Qt.ConnectionType.QueuedConnection)
+        self.mcompleter.apply_current_text.connect(self.apply_current_text,
+                type=Qt.ConnectionType.QueuedConnection)
         self.mcompleter.relayout_needed.connect(self.relayout)
         self.mcompleter.setFocusProxy(completer_widget)
         self.textEdited.connect(self.text_edited)
@@ -379,7 +391,7 @@ class LineEdit(QLineEdit, LineEditECM):
     def update_completions(self):
         ' Update the list of completions '
         self.original_cursor_pos = cpos = self.cursorPosition()
-        text = unicode_type(self.text())
+        text = str(self.text())
         prefix = text[:cpos]
         complete_prefix = prefix.lstrip()
         if self.sep:
@@ -396,7 +408,7 @@ class LineEdit(QLineEdit, LineEditECM):
                 cursor_pos = self.cursorPosition()
             self.original_cursor_pos = None
             # Split text
-            curtext = unicode_type(self.text())
+            curtext = str(self.text())
             before_text = curtext[:cursor_pos]
             after_text = curtext[cursor_pos:].rstrip()
             # Remove the completion prefix from the before text
@@ -416,11 +428,19 @@ class LineEdit(QLineEdit, LineEditECM):
             return before_text + completed_text, after_text
 
     def completion_selected(self, text):
-        before_text, after_text = self.get_completed_text(unicode_type(text))
+        before_text, after_text = self.get_completed_text(str(text))
         self.setText(before_text + after_text)
         self.setCursorPosition(len(before_text))
         self.item_selected.emit(text)
 
+    def apply_current_text(self):
+        if self.sep is not None:
+            txt = str(self.text())
+            sep_pos = txt.rfind(self.sep)
+            if sep_pos:
+                ntxt = txt[sep_pos+1:].strip()
+                self.completion_selected(ntxt)
+
 
 class EditWithComplete(EnComboBox):
 
@@ -458,7 +478,7 @@ class EditWithComplete(EnComboBox):
         self.lineEdit().set_add_separator(what)
 
     def show_initial_value(self, what):
-        what = unicode_type(what) if what else u''
+        what = str(what) if what else ''
         self.setText(what)
         self.lineEdit().selectAll()
 
@@ -480,10 +500,13 @@ class EditWithComplete(EnComboBox):
 
     def set_elide_mode(self, val):
         self.lineEdit().set_elide_mode(val)
+
+    def set_clear_button_enabled(self, val=True):
+        self.lineEdit().setClearButtonEnabled(bool(val))
     # }}}
 
     def text(self):
-        return unicode_type(self.lineEdit().text())
+        return str(self.lineEdit().text())
 
     def selectAll(self):
         self.lineEdit().selectAll()
@@ -528,8 +551,8 @@ if __name__ == '__main__':
     d.setLayout(QVBoxLayout())
     le = EditWithComplete(d)
     d.layout().addWidget(le)
-    items = ['one', 'otwo', 'othree', 'ooone', 'ootwo', 'other', 'odd', 'over', 'orc', 'oven', 'owe',
-        'oothree', 'a1', 'a2',u'Edgas', u'Èdgar', u'Édgaq', u'Edgar', u'Édgar']
+    items = ['oane\n line2\n line3', 'otwo', 'othree', 'ooone', 'ootwo', 'other', 'odd', 'over', 'orc', 'oven', 'owe',
+        'oothree', 'a1', 'a2','Edgas', 'Èdgar', 'Édgaq', 'Edgar', 'Édgar']
     le.update_items_cache(items)
     le.show_initial_value('')
-    d.exec_()
+    d.exec()
