@@ -1,6 +1,3 @@
-# -*- coding: utf-8 -*-
-
-
 __license__ = 'GPL 3'
 __copyright__ = '2011, John Schember <john@nachtimwald.com>, 2012 Eli Algranti <idea00@hotmail.com>'
 __docformat__ = 'restructuredtext en'
@@ -17,7 +14,6 @@ from calibre import as_unicode
 from calibre.utils.localization import localize_user_manual_link
 from calibre.ebooks.conversion.search_replace import compile_regular_expression
 from calibre.ebooks.conversion.config import OPTIONS
-from polyglot.builtins import unicode_type, range
 
 
 class SearchAndReplaceWidget(Widget, Ui_Form):
@@ -193,7 +189,7 @@ class SearchAndReplaceWidget(Widget, Ui_Form):
         edit_search = self.sr_search.regex
 
         if edit_search:
-            edit_replace = unicode_type(self.sr_replace.text())
+            edit_replace = str(self.sr_replace.text())
             found = False
             for search, replace in definitions:
                 if search == edit_search and replace == edit_replace:
@@ -234,7 +230,7 @@ class SearchAndReplaceWidget(Widget, Ui_Form):
         for row in range(0, self.search_replace.rowCount()):
             colItems = []
             for col in range(0, self.search_replace.columnCount()):
-                colItems.append(unicode_type(self.search_replace.item(row, col).text()))
+                colItems.append(str(self.search_replace.item(row, col).text()))
             ans.append(colItems)
         return ans
 
@@ -281,7 +277,7 @@ class SearchAndReplaceWidget(Widget, Ui_Form):
                 rest[name] = val
 
         if rest:
-            super(SearchAndReplaceWidget, self).apply_recommendations(rest)
+            super().apply_recommendations(rest)
 
         self.set_value(self.opt_search_replace, None)
         if new_val is None and legacy:
