@@ -1,35 +1,40 @@
-#!/usr/bin/env python2
-# vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
-
-__license__ = 'GPL v3'
-__copyright__ = '2014, Kovid Goyal <kovid at kovidgoyal.net>'
+#!/usr/bin/env python
+# License: GPLv3 Copyright: 2014, Kovid Goyal <kovid at kovidgoyal.net>
 
+import os
 from collections import OrderedDict
-from functools import partial
+from contextlib import suppress
+from copy import deepcopy
+from qt.core import (
+    QApplication, QCheckBox, QColor, QColorDialog, QDialog, QDialogButtonBox,
+    QFormLayout, QFrame, QGridLayout, QHBoxLayout, QIcon, QInputDialog, QLabel,
+    QLineEdit, QListWidget, QListWidgetItem, QMenu, QPixmap, QPushButton, QSize,
+    QSizePolicy, QSpinBox, Qt, QTabWidget, QTimer, QToolButton, QVBoxLayout, QWidget,
+    pyqtSignal
+)
 
-from PyQt5.Qt import (
-    QWidget, QHBoxLayout, QTabWidget, QLabel, QSizePolicy, QSize, QFormLayout,
-    QSpinBox, pyqtSignal, QPixmap, QDialog, QVBoxLayout, QDialogButtonBox,
-    QListWidget, QListWidgetItem, Qt, QGridLayout, QPushButton, QIcon,
-    QColorDialog, QToolButton, QLineEdit, QColor, QFrame, QTimer, QCheckBox)
-
-from calibre.ebooks.covers import all_styles, cprefs, generate_cover, override_prefs, default_color_themes
-from calibre.gui2 import gprefs, error_dialog
+from calibre.constants import config_dir
+from calibre.ebooks.covers import (
+    all_styles, cprefs, default_color_themes, generate_cover, override_prefs
+)
+from calibre.gui2 import error_dialog, gprefs
 from calibre.gui2.font_family_chooser import FontFamilyChooser
 from calibre.utils.date import now
-from calibre.utils.icu import sort_key
+from calibre.utils.filenames import make_long_path_useable
+from calibre.utils.icu import primary_sort_key, sort_key
+from polyglot.builtins import iteritems, itervalues
+
 
 class Preview(QLabel):
 
     def __init__(self, parent=None):
         QLabel.__init__(self, parent)
-        self.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Minimum)
+        self.setSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Minimum)
 
     def sizeHint(self):
         return QSize(300, 400)
 
+
 class ColorButton(QToolButton):
 
     def __init__(self, color, parent=None):
@@ -41,13 +46,13 @@ class ColorButton(QToolButton):
         self.setIcon(QIcon(self.pix))
         self.clicked.connect(self.choose_color)
 
-    @dynamic_property
+    @property
     def color(self):
-        def fget(self):
-            return self._color.name(QColor.HexRgb)[1:]
-        def fset(self, val):
-            self._color = QColor('#' + val)
-        return property(fget=fget, fset=fset)
+        return self._color.name(QColor.NameFormat.HexRgb)[1:]
+
+    @color.setter
+    def color(self, val):
+        self._color = QColor('#' + val)
 
     def update_display(self):
         self.pix.fill(self._color)
@@ -59,6 +64,7 @@ class ColorButton(QToolButton):
             self._color = c
             self.update_display()
 
+
 class CreateColorScheme(QDialog):
 
     def __init__(self, scheme_name, scheme, existing_names, edit_scheme=False, parent=None):
@@ -76,7 +82,7 @@ class CreateColorScheme(QDialog):
         l.addRow(_('Color &2:'), self.color2)
         l.addRow(_('Contrast color &1 (mainly for text):'), self.contrast_color1)
         l.addRow(_('Contrast color &2 (mainly for text):'), self.contrast_color2)
-        self.bb = bb = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel)
+        self.bb = bb = QDialogButtonBox(QDialogButtonBox.StandardButton.Ok | QDialogButtonBox.StandardButton.Cancel)
         bb.accepted.connect(self.accept)
         bb.rejected.connect(self.reject)
         l.addRow(bb)
@@ -96,6 +102,7 @@ class CreateColorScheme(QDialog):
                     'A color scheme with the name "%s" already exists.') % name, show=True)
         QDialog.accept(self)
 
+
 class CoverSettingsWidget(QWidget):
 
     changed = pyqtSignal()
@@ -166,20 +173,21 @@ class CoverSettingsWidget(QWidget):
         self.style_map = OrderedDict()
 
         self.font_page = fp = QWidget(st)
-        st.addTab(fp, _('&Fonts and Sizes'))
+        st.addTab(fp, _('&Fonts and sizes'))
         fp.l = l = QFormLayout()
         fp.setLayout(l)
         fp.f = []
+
         def add_hline():
             f = QFrame()
             fp.f.append(f)
-            f.setFrameShape(f.HLine)
+            f.setFrameShape(QFrame.Shape.HLine)
             l.addRow(f)
 
         for x, label, size_label in (
                 ('title', _('&Title font family:'), _('&Title font size:')),
-                ('subtitle', _('&Subtitle font family'), _('&Subtitle font size:')),
-                ('footer', _('&Footer font family'), _('&Footer font size')),
+                ('subtitle', _('&Subtitle font family:'), _('&Subtitle font size:')),
+                ('footer', _('&Footer font family:'), _('&Footer font size:')),
         ):
             attr = '%s_font_family' % x
             ff = FontFamilyChooser(fp)
@@ -196,6 +204,7 @@ class CoverSettingsWidget(QWidget):
             add_hline()
         self.changed_timer = t = QTimer(self)
         t.setSingleShot(True), t.setInterval(500), t.timeout.connect(self.emit_changed)
+
         def create_sz(label):
             ans = QSpinBox(self)
             ans.setSuffix(' px'), ans.setMinimum(100), ans.setMaximum(10000)
@@ -222,7 +231,7 @@ class CoverSettingsWidget(QWidget):
             ' in the templates for bold, italic and line breaks, respectively. The'
             ' default templates use the title, series and authors. You can change them to use'
             ' whatever metadata you like.'))
-        la.setWordWrap(True), la.setTextFormat(Qt.PlainText)
+        la.setWordWrap(True), la.setTextFormat(Qt.TextFormat.PlainText)
         l.addWidget(la)
 
         def create_template_widget(title, which, button):
@@ -232,16 +241,16 @@ class CoverSettingsWidget(QWidget):
             l.addWidget(heading)
             la = QLabel()
             setattr(self, attr, la)
-            l.addWidget(la), la.setTextFormat(Qt.PlainText), la.setStyleSheet('QLabel {font-family: monospace}')
+            l.addWidget(la), la.setTextFormat(Qt.TextFormat.PlainText), la.setStyleSheet('QLabel {font-family: monospace}')
             la.setWordWrap(True)
             b = QPushButton(button)
-            b.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
-            b.clicked.connect(partial(self.change_template, which))
+            b.setSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Fixed)
+            connect_lambda(b.clicked, self, lambda self: self.change_template(which))
             setattr(self, attr + '_button', b)
             l.addWidget(b)
             if which != 'footer':
                 f = QFrame(tp)
-                setattr(tp, attr + '_sep', f), f.setFrameShape(QFrame.HLine)
+                setattr(tp, attr + '_sep', f), f.setFrameShape(QFrame.Shape.HLine)
                 l.addWidget(f)
             l.addSpacing(10)
 
@@ -292,68 +301,68 @@ class CoverSettingsWidget(QWidget):
         self.colors_map = {}
         for name in sorted(color_themes, key=sort_key):
             self.colors_map[name] = li = QListWidgetItem(name, self.colors_list)
-            li.setFlags(li.flags() | Qt.ItemIsUserCheckable)
-            li.setCheckState(Qt.Unchecked if name in disabled else Qt.Checked)
-            li.setData(Qt.UserRole, color_themes[name])
+            li.setFlags(li.flags() | Qt.ItemFlag.ItemIsUserCheckable)
+            li.setCheckState(Qt.CheckState.Unchecked if name in disabled else Qt.CheckState.Checked)
+            li.setData(Qt.ItemDataRole.UserRole, color_themes[name])
         lu = prefs.get('last_used_colors')
-        if not self.for_global_prefs and lu in self.colors_map and self.colors_map[lu].checkState() == Qt.Checked:
+        if not self.for_global_prefs and lu in self.colors_map and self.colors_map[lu].checkState() == Qt.CheckState.Checked:
             self.colors_map[lu].setSelected(True)
         else:
-            for name, li in self.colors_map.iteritems():
-                if li.checkState() == Qt.Checked:
+            for name, li in iteritems(self.colors_map):
+                if li.checkState() == Qt.CheckState.Checked:
                     li.setSelected(True)
                     break
             else:
-                next(self.colors_map.itervalues()).setSelected(True)
+                next(itervalues(self.colors_map)).setSelected(True)
 
         disabled = set(prefs['disabled_styles'])
         self.styles_list.clear()
         self.style_map.clear()
         for name in sorted(all_styles(), key=sort_key):
             self.style_map[name] = li = QListWidgetItem(name, self.styles_list)
-            li.setFlags(li.flags() | Qt.ItemIsUserCheckable)
-            li.setCheckState(Qt.Unchecked if name in disabled else Qt.Checked)
+            li.setFlags(li.flags() | Qt.ItemFlag.ItemIsUserCheckable)
+            li.setCheckState(Qt.CheckState.Unchecked if name in disabled else Qt.CheckState.Checked)
         lu = prefs.get('last_used_style')
-        if not self.for_global_prefs and lu in self.style_map and self.style_map[lu].checkState() == Qt.Checked:
+        if not self.for_global_prefs and lu in self.style_map and self.style_map[lu].checkState() == Qt.CheckState.Checked:
             self.style_map[lu].setSelected(True)
         else:
-            for name, li in self.style_map.iteritems():
-                if li.checkState() == Qt.Checked:
+            for name, li in iteritems(self.style_map):
+                if li.checkState() == Qt.CheckState.Checked:
                     li.setSelected(True)
                     break
             else:
-                next(self.style_map.itervalues()).setSelected(True)
+                next(itervalues(self.style_map)).setSelected(True)
 
     @property
     def current_colors(self):
-        for name, li in self.colors_map.iteritems():
+        for name, li in iteritems(self.colors_map):
             if li.isSelected():
                 return name
 
     @property
     def disabled_colors(self):
-        for name, li in self.colors_map.iteritems():
-            if li.checkState() == Qt.Unchecked:
+        for name, li in iteritems(self.colors_map):
+            if li.checkState() == Qt.CheckState.Unchecked:
                 yield name
 
     @property
     def custom_colors(self):
         ans = {}
-        for name, li in self.colors_map.iteritems():
+        for name, li in iteritems(self.colors_map):
             if name.startswith('#'):
-                ans[name] = li.data(Qt.UserRole)
+                ans[name] = li.data(Qt.ItemDataRole.UserRole)
         return ans
 
     @property
     def current_style(self):
-        for name, li in self.style_map.iteritems():
+        for name, li in iteritems(self.style_map):
             if li.isSelected():
                 return name
 
     @property
     def disabled_styles(self):
-        for name, li in self.style_map.iteritems():
-            if li.checkState() == Qt.Unchecked:
+        for name, li in iteritems(self.style_map):
+            if li.checkState() == Qt.CheckState.Unchecked:
                 yield name
 
     @property
@@ -380,7 +389,7 @@ class CoverSettingsWidget(QWidget):
             self.colors_list.insertItem(0, li)
             cm = OrderedDict()
             cm[name] = li
-            for k, v in self.colors_map.iteritems():
+            for k, v in iteritems(self.colors_map):
                 cm[k] = v
             self.colors_map = cm
             li.setSelected(True)
@@ -388,12 +397,12 @@ class CoverSettingsWidget(QWidget):
                 self.colors_list.item(i).setSelected(False)
 
     def create_color_scheme(self):
-        scheme = self.colors_map[self.current_colors].data(Qt.UserRole)
+        scheme = self.colors_map[self.current_colors].data(Qt.ItemDataRole.UserRole)
         d = CreateColorScheme('#' + _('My Color Scheme'), scheme, set(self.colors_map), parent=self)
-        if d.exec_() == d.Accepted:
+        if d.exec() == QDialog.DialogCode.Accepted:
             name, scheme = d.data
             li = QListWidgetItem(name)
-            li.setData(Qt.UserRole, scheme), li.setFlags(li.flags() | Qt.ItemIsUserCheckable), li.setCheckState(Qt.Checked)
+            li.setData(Qt.ItemDataRole.UserRole, scheme), li.setFlags(li.flags() | Qt.ItemFlag.ItemIsUserCheckable), li.setCheckState(Qt.CheckState.Checked)
             self.insert_scheme(name, li)
             self.emit_changed()
             self.original_prefs['color_themes'] = self.current_prefs['color_themes']
@@ -405,11 +414,11 @@ class CoverSettingsWidget(QWidget):
                 'Cannot edit a builtin color scheme. Create a new'
                 ' color scheme instead.'), show=True)
         li = self.colors_map[cs]
-        d = CreateColorScheme(cs, li.data(Qt.UserRole), set(self.colors_map), edit_scheme=True, parent=self)
-        if d.exec_() == d.Accepted:
+        d = CreateColorScheme(cs, li.data(Qt.ItemDataRole.UserRole), set(self.colors_map), edit_scheme=True, parent=self)
+        if d.exec() == QDialog.DialogCode.Accepted:
             name, scheme = d.data
             li.setText(name)
-            li.setData(Qt.UserRole, scheme)
+            li.setData(Qt.ItemDataRole.UserRole, scheme)
             if name != cs:
                 self.colors_map.pop(cs, None)
                 self.insert_scheme(name, li)
@@ -444,7 +453,7 @@ class CoverSettingsWidget(QWidget):
         attr = which + '_template'
         templ = getattr(self, attr).text()
         d = TemplateDialog(self, templ, mi=self.mi, fm=field_metadata)
-        if d.exec_() == d.Accepted:
+        if d.exec() == QDialog.DialogCode.Accepted:
             templ = d.rule[1]
             getattr(self, attr).setText(templ)
             self.emit_changed()
@@ -458,7 +467,8 @@ class CoverSettingsWidget(QWidget):
     def update_preview(self):
         if self.ignore_changed:
             return
-        w, h = self.preview_label.sizeHint().width(), self.preview_label.sizeHint().height()
+        dpr = getattr(self, 'devicePixelRatioF', self.devicePixelRatio)()
+        w, h = int(dpr * self.preview_label.sizeHint().width()), int(dpr * self.preview_label.sizeHint().height())
         prefs = self.prefs_for_rendering
         hr = h / prefs['cover_height']
         for x in ('title', 'subtitle', 'footer'):
@@ -466,6 +476,7 @@ class CoverSettingsWidget(QWidget):
             prefs[attr] = int(prefs[attr] * hr)
         prefs['cover_width'], prefs['cover_height'] = w, h
         img = generate_cover(self.mi, prefs=prefs, as_qimage=True)
+        img.setDevicePixelRatio(dpr)
         self.preview_label.setPixmap(QPixmap.fromImage(img))
 
     def default_mi(self):
@@ -494,9 +505,24 @@ class CoverSettingsWidget(QWidget):
 
     def save_as_prefs(self):
         with self.original_prefs:
-            for k, v in self.current_prefs.iteritems():
+            for k, v in iteritems(self.current_prefs):
                 self.original_prefs[k] = v
 
+    @property
+    def serialized_prefs(self) -> bytes:
+        from calibre.utils.serialize import json_dumps
+        c = dict(deepcopy(self.original_prefs))
+        c.update(self.current_prefs)
+        return json_dumps(c, indent=2)
+
+    @serialized_prefs.setter
+    def serialized_prefs(self, val: bytes) -> None:
+        from calibre.utils.serialize import json_loads
+        prefs = json_loads(val)
+        self.apply_prefs(prefs)
+        self.update_preview()
+
+
 class CoverSettingsDialog(QDialog):
 
     def __init__(self, mi=None, prefs=None, parent=None):
@@ -509,11 +535,19 @@ class CoverSettingsDialog(QDialog):
         self.save_settings = ss = QCheckBox(_('Save these settings as the &defaults for future use'))
         ss.setChecked(gprefs.get('cover_generation_save_settings_for_future', True))
         l.addWidget(ss)
-        self.bb = bb = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel)
+        self.bb = bb = QDialogButtonBox(QDialogButtonBox.StandardButton.Ok | QDialogButtonBox.StandardButton.Cancel)
         l.addWidget(bb)
         bb.accepted.connect(self.accept), bb.rejected.connect(self.reject)
-        bb.b = b = bb.addButton(_('Restore &defaults'), bb.ActionRole)
+        bb.b = b = bb.addButton(_('Restore &defaults'), QDialogButtonBox.ButtonRole.ActionRole)
         b.clicked.connect(self.restore_defaults)
+        bb.ld = b = bb.addButton(_('&Save'), QDialogButtonBox.ButtonRole.ActionRole)
+        b.clicked.connect(self.export_settings)
+        b.setToolTip(_('Save the current cover generation settings for later re-use'))
+        bb.sd = b = bb.addButton(_('&Load'), QDialogButtonBox.ButtonRole.ActionRole)
+        self.load_menu = QMenu(b)
+        self.load_menu.aboutToShow.connect(self.populate_load_menu)
+        b.setMenu(self.load_menu)
+        b.setToolTip(_('Load previously saved cover generation settings'))
         ss.setToolTip('<p>' + _(
             'Save the current settings as the settings to use always instead of just this time. Remember that'
             ' for styles and colors the actual style or color used is chosen at random from'
@@ -522,13 +556,40 @@ class CoverSettingsDialog(QDialog):
         self.resize(self.sizeHint())
         geom = gprefs.get('cover_settings_dialog_geom', None)
         if geom is not None:
-            self.restoreGeometry(geom)
+            QApplication.instance().safe_restore_geometry(self, geom)
         self.prefs_for_rendering = None
 
     def restore_defaults(self):
         self.settings.restore_defaults()
         self.settings.save_as_prefs()
 
+    def export_settings(self):
+        name, ok = QInputDialog.getText(self, _('Name for these settings'), _('Theme name:'), text=_('My cover style'))
+        if ok:
+            base = os.path.join(config_dir, 'cover-generation-themes')
+            os.makedirs(base, exist_ok=True)
+            path = make_long_path_useable(os.path.join(base, name + '.json'))
+            raw = self.settings.serialized_prefs
+            with open(path, 'wb') as f:
+                f.write(raw)
+
+    def populate_load_menu(self):
+        m = self.load_menu
+        m.clear()
+        base = os.path.join(config_dir, 'cover-generation-themes')
+        entries = ()
+        with suppress(FileNotFoundError):
+            entries = sorted((x.rpartition('.')[0] for x in os.listdir(base) if x.endswith('.json')), key=primary_sort_key)
+        for name in entries:
+            m.addAction(name, self.import_settings)
+
+    def import_settings(self):
+        fname = self.sender().text() + '.json'
+        base = os.path.join(config_dir, 'cover-generation-themes')
+        with open(os.path.join(base, fname), 'rb') as f:
+            raw = f.read()
+        self.settings.serialized_prefs = raw
+
     def _save_settings(self):
         gprefs.set('cover_generation_save_settings_for_future', self.save_settings.isChecked())
         gprefs.set('cover_settings_dialog_geom', bytearray(self.saveGeometry()))
@@ -545,11 +606,12 @@ class CoverSettingsDialog(QDialog):
         self._save_settings()
         QDialog.reject(self)
 
+
 if __name__ == '__main__':
     from calibre.gui2 import Application
     app = Application([])
     d = CoverSettingsDialog()
     d.show()
-    app.exec_()
+    app.exec()
     del d
     del app
