@@ -1,5 +1,3 @@
-
-
 __license__   = 'GPL v3'
 __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
 
@@ -35,7 +33,7 @@ from calibre.utils.config import tweaks, device_prefs
 from calibre.utils.img import scale_image
 from calibre.library.save_to_disk import find_plugboard
 from calibre.ptempfile import PersistentTemporaryFile, force_unicode as filename_to_unicode
-from polyglot.builtins import unicode_type, string_or_unicode
+from polyglot.builtins import string_or_unicode
 from polyglot import queue
 # }}}
 
@@ -107,7 +105,7 @@ class DeviceJob(BaseJob):  # {{{
             call_job_done = True
         self._aborted = True
         self.failed = True
-        self._details = unicode_type(err)
+        self._details = str(err)
         self.exception = err
         if call_job_done:
             self.job_done()
@@ -125,7 +123,7 @@ def device_name_for_plugboards(device_class):
     return device_class.__class__.__name__
 
 
-class BusyCursor(object):
+class BusyCursor:
 
     def __enter__(self):
         QApplication.setOverrideCursor(QCursor(Qt.CursorShape.WaitCursor))
@@ -134,6 +132,37 @@ class BusyCursor(object):
         QApplication.restoreOverrideCursor()
 
 
+def convert_open_popup(opm, skip_key):
+    class OPM(OpenFeedback):
+
+        def __init__(self, opm):
+            super().__init__('placeholder')
+            self.opm = opm
+            self.skip_key = skip_key
+
+        def custom_dialog(self, parent):
+            from calibre.gui2.dialogs.message_box import MessageBox
+
+            class M(MessageBox):
+                def on_cd_finished(s):
+                    gprefs.set(self.skip_key, not s.toggle_checkbox.isChecked())
+            m = M({
+                'info': MessageBox.INFO, 'information': MessageBox.INFO,
+                'warn': MessageBox.WARNING, 'warning': MessageBox.WARNING,
+                }[self.opm.level], self.opm.title, self.opm.message,
+                parent=parent
+            )
+            tc = m.toggle_checkbox
+            tc.setVisible(True)
+            tc.setText(_('Show this message again'))
+            tc.setChecked(not self.opm.skip_dialog_skip_precheck)
+            m.resize_needed.emit()
+            m.finished.connect(m.on_cd_finished)
+            return m
+
+    return OPM(opm)
+
+
 class DeviceManager(Thread):  # {{{
 
     def __init__(self, connected_slot, job_manager, open_feedback_slot,
@@ -142,8 +171,7 @@ class DeviceManager(Thread):  # {{{
         '''
         :sleep_time: Time to sleep between device probes in secs
         '''
-        Thread.__init__(self)
-        self.setDaemon(True)
+        Thread.__init__(self, daemon=True)
         # [Device driver, Showing in GUI, Ejected]
         self.devices        = list(device_plugins())
         self.disabled_device_plugins = list(disabled_device_plugins())
@@ -152,7 +180,7 @@ class DeviceManager(Thread):  # {{{
         self.unmanaged_devices = [x for x in self.devices if
                 x.MANAGES_DEVICE_PRESENCE]
         self.sleep_time     = sleep_time
-        self.connected_slot = connected_slot
+        self.connected_slot = connected_slot  # see DeviceMixin.device_connected()
         self.allow_connect_slot = allow_connect_slot
         self.jobs           = queue.Queue(0)
         self.job_steps      = queue.Queue(0)
@@ -194,6 +222,11 @@ class DeviceManager(Thread):  # {{{
         for dev, detected_device in connected_devices:
             if dev.OPEN_FEEDBACK_MESSAGE is not None:
                 self.open_feedback_slot(dev.OPEN_FEEDBACK_MESSAGE)
+            opm = dev.get_open_popup_message()
+            if opm is not None:
+                skip_key = f'do_not_show_device_open_popup_message_{dev.__class__.__name__}'
+                if not gprefs.get(skip_key, False):
+                    self.open_feedback_msg(dev.get_gui_name(), convert_open_popup(opm, skip_key))
             try:
                 dev.reset(detected_device=detected_device,
                     report_progress=self.report_progress)
@@ -209,7 +242,7 @@ class DeviceManager(Thread):  # {{{
                 tb = traceback.format_exc()
                 if DEBUG or tb not in self.reported_errors:
                     self.reported_errors.add(tb)
-                    prints('Unable to open device', unicode_type(dev))
+                    prints('Unable to open device', str(dev))
                     prints(tb)
                 continue
             self.after_device_connect(dev, device_kind)
@@ -250,8 +283,9 @@ class DeviceManager(Thread):  # {{{
             pass
         if self.connected_device in self.ejected_devices:
             self.ejected_devices.remove(self.connected_device)
+            call_connected_slot = False
         else:
-            self.connected_slot(False, self.connected_device_kind)
+            call_connected_slot = True
         if self.call_shutdown_on_disconnect:
             # The current device is an instance of a plugin class instantiated
             # to handle this connection, probably as a mounted device. We are
@@ -263,6 +297,8 @@ class DeviceManager(Thread):  # {{{
         device_prefs.set_overrides()
         self.connected_device = currently_connected_device._device = None
         self._device_information = None
+        if call_connected_slot:
+            self.connected_slot(False, None)
 
     def detect_device(self):
         self.scanner.scan()
@@ -279,7 +315,7 @@ class DeviceManager(Thread):  # {{{
                 if not connected:
                     if DEBUG:
                         # Allow the device subsystem to output debugging info about
-                        # why it thinks the device is not connected. Used, for e.g.
+                        # why it thinks the device is not connected. Used, e.g.
                         # in the can_handle() method of the T1 driver
                         self.scanner.is_device_connected(self.connected_device,
                                 only_presence=True, debug=True)
@@ -349,7 +385,7 @@ class DeviceManager(Thread):  # {{{
                 self.connected_device.eject()
                 if self.connected_device_kind != 'unmanaged-device':
                     self.ejected_devices.add(self.connected_device)
-                self.connected_slot(False, self.connected_device_kind)
+                self.connected_slot(False, None)
             elif hasattr(self.connected_device, 'unmount_device'):
                 # As we are on the wrong thread, this call must *not* do
                 # anything besides set a flag that the right thread will see.
@@ -505,7 +541,7 @@ class DeviceManager(Thread):  # {{{
             self._device_information['info'] = tuple(info)
 
     def get_current_device_information(self):
-        return self._device_information
+        return self._device_information if self.is_device_present else None
 
     def _books(self):
         '''Get metadata from device'''
@@ -563,7 +599,7 @@ class DeviceManager(Thread):  # {{{
             self.connected_device.set_plugboards(plugboards, find_plugboard)
         if metadata and files and len(metadata) == len(files):
             for f, mi in zip(files, metadata):
-                if isinstance(f, unicode_type):
+                if isinstance(f, str):
                     ext = f.rpartition('.')[-1].lower()
                     cpb = find_plugboard(
                             device_name_for_plugboards(self.connected_device),
@@ -877,7 +913,9 @@ class DeviceSignals(QObject):  # {{{
 
     #: This signal is emitted once when the device is detected and once when
     #: it is disconnected. If the parameter is True, then it is a connection,
-    #: otherwise a disconnection.
+    #: otherwise a disconnection. Device information is not available in either
+    # case. If you need device information when connecting then use the
+    # device_metadata_available signal.
     device_connection_changed = pyqtSignal(object)
 
 
@@ -885,7 +923,7 @@ device_signals = DeviceSignals()
 # }}}
 
 
-class DeviceMixin(object):  # {{{
+class DeviceMixin:  # {{{
 
     def __init__(self, *args, **kwargs):
         pass
@@ -930,7 +968,7 @@ class DeviceMixin(object):  # {{{
         d.show()
 
     def auto_convert_question(self, msg, autos):
-        autos = '\n'.join(map(unicode_type, map(force_unicode, autos)))
+        autos = '\n'.join(map(str, map(force_unicode, autos)))
         return self.ask_a_yes_no_question(
                 _('No suitable formats'), msg,
                 ans_when_user_unavailable=True,
@@ -986,7 +1024,7 @@ class DeviceMixin(object):  # {{{
             if cw.validate():
                 QDialog.accept(config_dialog)
         config_dialog.accept = validate
-        if config_dialog.exec_() == QDialog.DialogCode.Accepted:
+        if config_dialog.exec() == QDialog.DialogCode.Accepted:
             dev.save_settings(cw)
 
             do_restart = show_restart_warning(_('Restart calibre for the changes to %s'
@@ -1028,7 +1066,7 @@ class DeviceMixin(object):  # {{{
 
         try:
             if 'Could not read 32 bytes on the control bus.' in \
-                    unicode_type(job.details):
+                    str(job.details):
                 error_dialog(self, _('Error talking to device'),
                              _('There was a temporary error talking to the '
                              'device. Please unplug and reconnect the device '
@@ -1039,7 +1077,7 @@ class DeviceMixin(object):  # {{{
         if getattr(job, 'exception', None).__class__.__name__ == 'MTPInvalidSendPathError':
             try:
                 from calibre.gui2.device_drivers.mtp_config import SendError
-                return SendError(self, job.exception).exec_()
+                return SendError(self, job.exception).exec()
             except:
                 traceback.print_exc()
         try:
@@ -1069,6 +1107,8 @@ class DeviceMixin(object):  # {{{
     def device_detected(self, connected, device_kind):
         '''
         Called when a device is connected to the computer.
+
+        If connected is False then device_kind is None.
         '''
         # This can happen as this function is called in a queued connection and
         # the user could have yanked the device in the meantime
@@ -1229,7 +1269,7 @@ class DeviceMixin(object):  # {{{
         rows = self.library_view.selectionModel().selectedRows()
         if not rows or len(rows) == 0:
             error_dialog(self, _('No books'), _('No books')+' '+
-                    _('selected to send')).exec_()
+                    _('selected to send')).exec()
             return
 
         fmt = None
@@ -1259,7 +1299,7 @@ class DeviceMixin(object):  # {{{
                 elif f in aval_out_formats:
                     formats.append((f, _('0 of %i books') % len(rows), True))
             d = ChooseFormatDeviceDialog(self, _('Choose format to send to device'), formats)
-            if d.exec_() != QDialog.DialogCode.Accepted:
+            if d.exec() != QDialog.DialogCode.Accepted:
                 return
             if d.format():
                 fmt = d.format().lower()
@@ -1267,15 +1307,15 @@ class DeviceMixin(object):  # {{{
         if dest in ('main', 'carda', 'cardb'):
             if not self.device_connected or not self.device_manager:
                 error_dialog(self, _('No device'),
-                        _('Cannot send: No device is connected')).exec_()
+                        _('Cannot send: No device is connected')).exec()
                 return
             if dest == 'carda' and not self.device_manager.has_card():
                 error_dialog(self, _('No card'),
-                        _('Cannot send: Device has no storage card')).exec_()
+                        _('Cannot send: Device has no storage card')).exec()
                 return
             if dest == 'cardb' and not self.device_manager.has_card():
                 error_dialog(self, _('No card'),
-                        _('Cannot send: Device has no storage card')).exec_()
+                        _('Cannot send: Device has no storage card')).exec()
                 return
             if dest == 'main':
                 on_card = None
@@ -1354,7 +1394,7 @@ class DeviceMixin(object):  # {{{
             names = []
             for book_id, mi in zip(ids, metadata):
                 prefix = ascii_filename(mi.title)
-                if not isinstance(prefix, unicode_type):
+                if not isinstance(prefix, str):
                     prefix = prefix.decode(preferred_encoding, 'replace')
                 prefix = ascii_filename(prefix)
                 names.append('%s_%d%s'%(prefix, book_id,
@@ -1432,7 +1472,7 @@ class DeviceMixin(object):  # {{{
             names = []
             for book_id, mi in zip(ids, metadata):
                 prefix = ascii_filename(mi.title)
-                if not isinstance(prefix, unicode_type):
+                if not isinstance(prefix, str):
                     prefix = prefix.decode(preferred_encoding, 'replace')
                 prefix = ascii_filename(prefix)
                 names.append('%s_%d%s'%(prefix, book_id,
@@ -1446,7 +1486,7 @@ class DeviceMixin(object):  # {{{
                     self.location_manager.free[2] : 'cardb'}
                 on_card = space.get(sorted(space.keys(), reverse=True)[0], None)
                 try:
-                    total_size = sum([os.stat(f).st_size for f in files])
+                    total_size = sum(os.stat(f).st_size for f in files)
                 except:
                     try:
                         import traceback
@@ -1511,7 +1551,7 @@ class DeviceMixin(object):  # {{{
                 if not a:
                     a = _('Unknown')
                 prefix = ascii_filename(t+' - '+a)
-                if not isinstance(prefix, unicode_type):
+                if not isinstance(prefix, str):
                     prefix = prefix.decode(preferred_encoding, 'replace')
                 prefix = ascii_filename(prefix)
                 names.append('%s_%d%s'%(prefix, id, os.path.splitext(f)[1]))
@@ -1562,7 +1602,7 @@ class DeviceMixin(object):  # {{{
                 'as no suitable formats were found. Convert the book(s) to a '
                 'format supported by your device first.'
                 ), bad)
-            d.exec_()
+            d.exec()
 
     def upload_dirtied_booklists(self):
         '''
@@ -1632,7 +1672,7 @@ class DeviceMixin(object):  # {{{
 
         if job.exception is not None:
             if isinstance(job.exception, FreeSpaceError):
-                where = 'in main memory.' if 'memory' in unicode_type(job.exception) \
+                where = 'in main memory.' if 'memory' in str(job.exception) \
                         else 'on the storage card.'
                 titles = '\n'.join(['<li>'+mi.title+'</li>'
                                     for mi in metadata])
@@ -1640,10 +1680,10 @@ class DeviceMixin(object):  # {{{
                                  _('<p>Cannot upload books to device there '
                                  'is no more free space available ')+where+
                                  '</p>\n<ul>%s</ul>'%(titles,))
-                d.exec_()
+                d.exec()
             elif isinstance(job.exception, WrongDestinationError):
                 error_dialog(self, _('Incorrect destination'),
-                        unicode_type(job.exception), show=True)
+                        str(job.exception), show=True)
             else:
                 self.device_job_exception(job)
             return
