@@ -1,5 +1,4 @@
 #!/usr/bin/env python
-# vim:fileencoding=utf-8
 # License: GPLv3 Copyright: 2008, Kovid Goyal <kovid at kovidgoyal.net>
 
 
@@ -8,7 +7,7 @@ import regex
 from collections import defaultdict, namedtuple
 from io import BytesIO
 from qt.core import (
-    QApplication, QComboBox, QCompleter, QCoreApplication, QDateTime, QDialog,
+    QApplication, QComboBox, QCompleter, QDateTime, QDialog,
     QDialogButtonBox, QFont, QGridLayout, QInputDialog, QLabel, QLineEdit,
     QProgressBar, QSize, Qt, QVBoxLayout, pyqtSignal
 )
@@ -33,9 +32,7 @@ from calibre.utils.config import JSONConfig, dynamic, prefs, tweaks
 from calibre.utils.date import internal_iso_format_string, qt_to_dt
 from calibre.utils.icu import capitalize, sort_key
 from calibre.utils.titlecase import titlecase
-from polyglot.builtins import (
-    error_message, filter, iteritems, itervalues, native_string_type, unicode_type
-)
+from polyglot.builtins import error_message, iteritems, itervalues, native_string_type
 
 Settings = namedtuple('Settings',
     'remove_all remove add au aus do_aus rating pub do_series do_autonumber '
@@ -57,6 +54,9 @@ class Caser(LineEditECM):
     def setText(self, text):
         self.title = text
 
+    def hasSelectedText(self):
+        return False
+
 
 class MyBlockingBusy(QDialog):  # {{{
 
@@ -164,10 +164,11 @@ class MyBlockingBusy(QDialog):  # {{{
                 self.error = (err, traceback.format_exc())
         QDialog.accept(self)
 
-    def exec_(self):
+    def exec(self):
         self.thread = Thread(target=self.do_it)
         self.thread.start()
-        return QDialog.exec_(self)
+        return QDialog.exec(self)
+    exec_ = exec
 
     def do_it(self):
         try:
@@ -175,7 +176,7 @@ class MyBlockingBusy(QDialog):  # {{{
         except Exception as err:
             import traceback
             try:
-                err = unicode_type(err)
+                err = str(err)
             except:
                 err = repr(err)
             self.error = (err, traceback.format_exc())
@@ -500,7 +501,7 @@ class MetadataBulkDialog(QDialog, Ui_MetadataBulkDialog):
         self.refresh_book_list.toggled.connect(self.save_refresh_booklist)
         self.ids = [self.db.id(r) for r in rows]
         self.first_title = self.db.title(self.ids[0], index_is_id=True)
-        self.cover_clone.setToolTip(unicode_type(self.cover_clone.toolTip()) + ' (%s)' % self.first_title)
+        self.cover_clone.setToolTip(str(self.cover_clone.toolTip()) + ' (%s)' % self.first_title)
         self.setWindowTitle(ngettext(
             'Editing metadata for one book',
             'Editing metadata for {} books', len(rows)).format(len(rows)))
@@ -577,29 +578,28 @@ class MetadataBulkDialog(QDialog, Ui_MetadataBulkDialog):
         self.authors.setFocus(Qt.FocusReason.OtherFocusReason)
         self.generate_cover_settings = None
         self.button_config_cover_gen.clicked.connect(self.customize_cover_generation)
-        self.exec_()
+        self.exec()
 
     def sizeHint(self):
-        desktop = QCoreApplication.instance().desktop()
-        geom = desktop.availableGeometry(self)
+        geom = self.screen().availableSize()
         nh, nw = max(300, geom.height()-50), max(400, geom.width()-70)
         return QSize(nw, nh)
 
     def customize_cover_generation(self):
         from calibre.gui2.covers import CoverSettingsDialog
         d = CoverSettingsDialog(parent=self)
-        if d.exec_() == QDialog.DialogCode.Accepted:
+        if d.exec() == QDialog.DialogCode.Accepted:
             self.generate_cover_settings = d.prefs_for_rendering
 
     def set_comments(self):
         from calibre.gui2.dialogs.comments_dialog import CommentsDialog
         d = CommentsDialog(self, '' if self.comments is null else (self.comments or ''), _('Comments'))
-        if d.exec_() == QDialog.DialogCode.Accepted:
+        if d.exec() == QDialog.DialogCode.Accepted:
             self.comments = d.textbox.html
             b = self.comments_button
             b.setStyleSheet('QPushButton { font-weight: bold }')
-            if unicode_type(b.text())[-1] != '*':
-                b.setText(unicode_type(b.text()) + ' *')
+            if str(b.text())[-1] != '*':
+                b.setText(str(b.text()) + ' *')
 
     def save_refresh_booklist(self, *args):
         gprefs['refresh_book_list_on_bulk_edit'] = bool(self.refresh_book_list.isChecked())
@@ -769,18 +769,18 @@ class MetadataBulkDialog(QDialog, Ui_MetadataBulkDialog):
     def s_r_sf_itemdata(self, idx):
         if idx is None:
             idx = self.search_field.currentIndex()
-        return unicode_type(self.search_field.itemData(idx) or '')
+        return str(self.search_field.itemData(idx) or '')
 
     def s_r_df_itemdata(self, idx):
         if idx is None:
             idx = self.destination_field.currentIndex()
-        return unicode_type(self.destination_field.itemData(idx) or '')
+        return str(self.destination_field.itemData(idx) or '')
 
     def s_r_get_field(self, mi, field):
         if field:
             if field == '{template}':
                 v = SafeFormat().safe_format(
-                    unicode_type(self.s_r_template.text()), mi, _('S/R TEMPLATE ERROR'), mi)
+                    str(self.s_r_template.text()), mi, _('S/R TEMPLATE ERROR'), mi)
                 return [v]
             fm = self.db.metadata_for_field(field)
             if field == 'sort':
@@ -790,10 +790,10 @@ class MetadataBulkDialog(QDialog, Ui_MetadataBulkDialog):
             else:
                 val = mi.get(field, None)
             if isinstance(val, (numbers.Number, bool)):
-                val = unicode_type(val)
+                val = str(val)
             elif fm['is_csp']:
                 # convert the csp dict into a list
-                id_type = unicode_type(self.s_r_src_ident.currentText())
+                id_type = str(self.s_r_src_ident.currentText())
                 if id_type:
                     val = [val.get(id_type, '')]
                 else:
@@ -842,7 +842,7 @@ class MetadataBulkDialog(QDialog, Ui_MetadataBulkDialog):
             if len(t) > 1:
                 t = t[self.starting_from.value()-1:
                       self.starting_from.value()-1 + self.results_count.value()]
-            w.setText(unicode_type(self.multiple_separator.text()).join(t))
+            w.setText(str(self.multiple_separator.text()).join(t))
 
         if self.search_mode.currentIndex() == 0:
             self.destination_field.setCurrentIndex(idx)
@@ -907,8 +907,8 @@ class MetadataBulkDialog(QDialog, Ui_MetadataBulkDialog):
             getattr(self, 'book_%d_result'%(i+1)).setText('')
 
     def s_r_func(self, match):
-        rfunc = self.s_r_functions[unicode_type(self.replace_func.currentText())]
-        rtext = unicode_type(self.replace_with.text())
+        rfunc = self.s_r_functions[str(self.replace_func.currentText())]
+        rtext = str(self.replace_with.text())
         rtext = match.expand(rtext)
         return rfunc(rtext)
 
@@ -916,7 +916,7 @@ class MetadataBulkDialog(QDialog, Ui_MetadataBulkDialog):
         src_field = self.s_r_sf_itemdata(None)
         src = self.s_r_get_field(mi, src_field)
         result = []
-        rfunc = self.s_r_functions[unicode_type(self.replace_func.currentText())]
+        rfunc = self.s_r_functions[str(self.replace_func.currentText())]
         for s in src:
             t = self.s_r_obj.sub(self.s_r_func, s)
             if self.search_mode.currentIndex() == 0:
@@ -950,7 +950,7 @@ class MetadataBulkDialog(QDialog, Ui_MetadataBulkDialog):
         dest_mode = self.replace_mode.currentIndex()
 
         if self.destination_field_fm['is_csp']:
-            dest_ident = unicode_type(self.s_r_dst_ident.text())
+            dest_ident = str(self.s_r_dst_ident.text())
             if not dest_ident or (src == 'identifiers' and dest_ident == '*'):
                 raise Exception(_('You must specify a destination identifier type'))
 
@@ -967,7 +967,7 @@ class MetadataBulkDialog(QDialog, Ui_MetadataBulkDialog):
         if dest_mode != 0:
             dest_val = mi.get(dest, '')
             if self.db.metadata_for_field(dest)['is_csp']:
-                dst_id_type = unicode_type(self.s_r_dst_ident.text())
+                dst_id_type = str(self.s_r_dst_ident.text())
                 if dst_id_type:
                     dest_val = [dest_val.get(dst_id_type, '')]
                 else:
@@ -1000,7 +1000,7 @@ class MetadataBulkDialog(QDialog, Ui_MetadataBulkDialog):
             flags |= regex.IGNORECASE
 
         try:
-            stext = unicode_type(self.search_for.text())
+            stext = str(self.search_for.text())
             if not stext:
                 raise Exception(_('You must specify a search expression in the "Search for" field'))
             if self.search_mode.currentIndex() == 0:
@@ -1036,7 +1036,7 @@ class MetadataBulkDialog(QDialog, Ui_MetadataBulkDialog):
                 if len(t) > 1 and self.destination_field_fm['is_multiple']:
                     t = t[self.starting_from.value()-1:
                           self.starting_from.value()-1 + self.results_count.value()]
-                    t = unicode_type(self.multiple_separator.text()).join(t)
+                    t = str(self.multiple_separator.text()).join(t)
                 else:
                     t = self.s_r_replace_mode_separator().join(t)
                 wr.setText(t)
@@ -1060,7 +1060,7 @@ class MetadataBulkDialog(QDialog, Ui_MetadataBulkDialog):
             if dfm['is_csp']:
                 # convert the colon-separated pair strings back into a dict,
                 # which is what set_identifiers wants
-                dst_id_type = unicode_type(self.s_r_dst_ident.text())
+                dst_id_type = str(self.s_r_dst_ident.text())
                 if dst_id_type and dst_id_type != '*':
                     v = ''.join(val)
                     ids = mi.get(dest)
@@ -1141,7 +1141,7 @@ class MetadataBulkDialog(QDialog, Ui_MetadataBulkDialog):
 
     def tag_editor(self, *args):
         d = TagEditor(self, self.db, None)
-        d.exec_()
+        d.exec()
         if d.result() == QDialog.DialogCode.Accepted:
             tag_string = ', '.join(d.tags)
             self.tags.setText(tag_string)
@@ -1188,19 +1188,19 @@ class MetadataBulkDialog(QDialog, Ui_MetadataBulkDialog):
         remove_all = self.remove_all_tags.isChecked()
         remove = []
         if not remove_all:
-            remove = unicode_type(self.remove_tags.text()).strip().split(',')
-        add = unicode_type(self.tags.text()).strip().split(',')
-        au = unicode_type(self.authors.text())
-        aus = unicode_type(self.author_sort.text())
+            remove = str(self.remove_tags.text()).strip().split(',')
+        add = str(self.tags.text()).strip().split(',')
+        au = str(self.authors.text())
+        aus = str(self.author_sort.text())
         do_aus = self.author_sort.isEnabled()
         rating = self.rating.rating_value
         if not self.apply_rating.isChecked():
             rating = -1
-        pub = unicode_type(self.publisher.text())
+        pub = str(self.publisher.text())
         do_series = self.write_series
         clear_series = self.clear_series.isChecked()
         clear_pub = self.clear_pub.isChecked()
-        series = unicode_type(self.series.currentText()).strip()
+        series = str(self.series.currentText()).strip()
         do_autonumber = self.autonumber_series.isChecked()
         do_series_restart = self.series_numbering_restarts.isChecked()
         series_start_value = self.series_start_number.value()
@@ -1255,7 +1255,7 @@ class MetadataBulkDialog(QDialog, Ui_MetadataBulkDialog):
         # which can slow down bulk editing of large numbers of books
         self.model.stop_metadata_backup()
         try:
-            bb.exec_()
+            bb.exec()
         finally:
             self.model.start_metadata_backup()
 
@@ -1276,11 +1276,11 @@ class MetadataBulkDialog(QDialog, Ui_MetadataBulkDialog):
                 'Covers were compressed by {percent:.1%} from a total size of'
                 ' {old} to {new}.').format(
                     percent=percent, old=human_readable(total_old), new=human_readable(total_new))
-                ).exec_()
+                ).exec()
         return QDialog.accept(self)
 
     def series_changed(self, *args):
-        self.write_series = bool(unicode_type(self.series.currentText()).strip())
+        self.write_series = bool(str(self.series.currentText()).strip())
         self.autonumber_series.setEnabled(True)
 
     def s_r_remove_query(self, *args):
@@ -1293,7 +1293,7 @@ class MetadataBulkDialog(QDialog, Ui_MetadataBulkDialog):
             return
 
         item_id = self.query_field.currentIndex()
-        item_name = unicode_type(self.query_field.currentText())
+        item_name = str(self.query_field.currentText())
 
         self.query_field.blockSignals(True)
         self.query_field.removeItem(item_id)
@@ -1321,7 +1321,7 @@ class MetadataBulkDialog(QDialog, Ui_MetadataBulkDialog):
                 error_dialog(self, _("Save search/replace"),
                         _("You must provide a name."), show=True)
         new = True
-        name = unicode_type(name)
+        name = str(name)
         if name in list(self.queries.keys()):
             if not question_dialog(self, _("Save search/replace"),
                     _("That saved search/replace already exists and will be overwritten. "
@@ -1331,21 +1331,21 @@ class MetadataBulkDialog(QDialog, Ui_MetadataBulkDialog):
 
         query = {}
         query['name'] = name
-        query['search_field'] = unicode_type(self.search_field.currentText())
-        query['search_mode'] = unicode_type(self.search_mode.currentText())
-        query['s_r_template'] = unicode_type(self.s_r_template.text())
-        query['s_r_src_ident'] = unicode_type(self.s_r_src_ident.currentText())
-        query['search_for'] = unicode_type(self.search_for.text())
+        query['search_field'] = str(self.search_field.currentText())
+        query['search_mode'] = str(self.search_mode.currentText())
+        query['s_r_template'] = str(self.s_r_template.text())
+        query['s_r_src_ident'] = str(self.s_r_src_ident.currentText())
+        query['search_for'] = str(self.search_for.text())
         query['case_sensitive'] = self.case_sensitive.isChecked()
-        query['replace_with'] = unicode_type(self.replace_with.text())
-        query['replace_func'] = unicode_type(self.replace_func.currentText())
-        query['destination_field'] = unicode_type(self.destination_field.currentText())
-        query['s_r_dst_ident'] = unicode_type(self.s_r_dst_ident.text())
-        query['replace_mode'] = unicode_type(self.replace_mode.currentText())
+        query['replace_with'] = str(self.replace_with.text())
+        query['replace_func'] = str(self.replace_func.currentText())
+        query['destination_field'] = str(self.destination_field.currentText())
+        query['s_r_dst_ident'] = str(self.s_r_dst_ident.text())
+        query['replace_mode'] = str(self.replace_mode.currentText())
         query['comma_separated'] = self.comma_separated.isChecked()
         query['results_count'] = self.results_count.value()
         query['starting_from'] = self.starting_from.value()
-        query['multiple_separator'] = unicode_type(self.multiple_separator.text())
+        query['multiple_separator'] = str(self.multiple_separator.text())
 
         self.queries[name] = query
         self.queries.commit()
@@ -1364,7 +1364,7 @@ class MetadataBulkDialog(QDialog, Ui_MetadataBulkDialog):
             self.s_r_reset_query_fields()
             self.saved_search_name = ''
             return
-        item = self.queries.get(unicode_type(item_name), None)
+        item = self.queries.get(str(item_name), None)
         if item is None:
             self.s_r_reset_query_fields()
             return
