@@ -1,41 +1,48 @@
 #!/usr/bin/env python
-# vim:fileencoding=utf-8
 
 
 __license__ = 'GPL v3'
 __copyright__ = '2015, Kovid Goyal <kovid at kovidgoyal.net>'
 
-import os, errno, json, importlib, math, bz2, shutil, sys
-from itertools import count
+import bz2
+import errno
+import importlib
+import json
+import math
+import os
+import shutil
+import sys
 from io import BytesIO
-from threading import Thread, Event
+from itertools import count
 from multiprocessing.pool import ThreadPool
-
 from qt.core import (
-    QImageReader, QFormLayout, QVBoxLayout, QSplitter, QGroupBox, QListWidget,
-    QLineEdit, QSpinBox, QTextEdit, QSize, QListWidgetItem, QIcon, QImage,
-    pyqtSignal, QStackedLayout, QWidget, QLabel, Qt, QComboBox, QPixmap, QDialog,
-    QGridLayout, QStyledItemDelegate, QApplication, QStaticText, sip,
-    QStyle, QPen, QProgressDialog, QAbstractItemView, QDialogButtonBox
+    QAbstractItemView, QApplication, QComboBox, QDialog, QDialogButtonBox,
+    QFormLayout, QGridLayout, QGroupBox, QIcon, QImage, QImageReader, QLabel,
+    QLineEdit, QListWidget, QListWidgetItem, QPen, QPixmap, QProgressDialog, QSize,
+    QSpinBox, QSplitter, QStackedLayout, QStaticText, QStyle, QStyledItemDelegate,
+    Qt, QTextEdit, QVBoxLayout, QWidget, pyqtSignal, sip
 )
+from threading import Event, Thread
 
-from calibre import walk, fit_image, human_readable, detect_ncpus as cpu_count
+from calibre import detect_ncpus as cpu_count, fit_image, human_readable, walk
 from calibre.constants import cache_dir, config_dir
 from calibre.customize.ui import interface_actions
-from calibre.gui2 import must_use_qt, gprefs, choose_dir, error_dialog, choose_save_file, question_dialog, empty_index
+from calibre.gui2 import (
+    choose_dir, choose_save_file, empty_index, error_dialog, gprefs, must_use_qt,
+    question_dialog, safe_open_url
+)
 from calibre.gui2.dialogs.progress import ProgressDialog
 from calibre.gui2.progress_indicator import ProgressIndicator
 from calibre.gui2.widgets2 import Dialog
 from calibre.utils.date import utcnow
-from calibre.utils.filenames import ascii_filename
-from calibre.utils.https import get_https_resource_securely, HTTPError
+from calibre.utils.filenames import ascii_filename, atomic_rename
+from calibre.utils.https import HTTPError, get_https_resource_securely
 from calibre.utils.icu import numeric_sort_key as sort_key
-from calibre.utils.img import image_from_data, Canvas, optimize_png, optimize_jpeg
-from calibre.utils.zipfile import ZipFile, ZIP_STORED
-from calibre.utils.filenames import atomic_rename
-from polyglot.builtins import iteritems, reraise, as_bytes
+from calibre.utils.img import Canvas, image_from_data, optimize_jpeg, optimize_png
+from calibre.utils.zipfile import ZIP_STORED, ZipFile
 from polyglot import http_client
-from polyglot.queue import Queue, Empty
+from polyglot.builtins import as_bytes, iteritems, reraise
+from polyglot.queue import Empty, Queue
 
 IMAGE_EXTENSIONS = {'png', 'jpg', 'jpeg'}
 THEME_COVER = 'icon-theme-cover.jpg'
@@ -179,7 +186,7 @@ def create_cover(report, icons=(), cols=5, size=120, padding=16):
             with lopen(ipath, 'rb') as f:
                 img = image_from_data(f.read())
             scaled, nwidth, nheight = fit_image(img.width(), img.height(), size, size)
-            img = img.scaled(nwidth, nheight, Qt.AspectRatioMode.IgnoreAspectRatio, Qt.TransformationMode.SmoothTransformation)
+            img = img.scaled(int(nwidth), int(nheight), Qt.AspectRatioMode.IgnoreAspectRatio, Qt.TransformationMode.SmoothTransformation)
             dx = (size - nwidth) // 2
             canvas.compose(img, x + dx, y)
     return canvas.export()
@@ -408,11 +415,11 @@ def create_theme(folder=None, parent=None):
             return
     report = read_theme_from_folder(folder)
     d = ThemeCreateDialog(parent, report)
-    if d.exec_() != QDialog.DialogCode.Accepted:
+    if d.exec() != QDialog.DialogCode.Accepted:
         return
     d.save_metadata()
     d = Compress(d.report, parent=parent)
-    d.exec_()
+    d.exec()
     if d.wasCanceled() or d.raw is None:
         return
     raw, prefix = d.raw, d.prefix
@@ -482,7 +489,8 @@ def get_cover(metadata):
 
 def get_covers(themes, dialog, num_of_workers=8):
     items = Queue()
-    tuple(map(items.put, themes))
+    for i in themes:
+        items.put(i)
 
     def callback(metadata, x):
         if not sip.isdeleted(dialog) and not dialog.dialog_closed:
@@ -537,11 +545,12 @@ class Delegate(QStyledItemDelegate):
             <p>by <i>{author}</i> with <b>{number}</b> icons [{size}]</p>
             <p>{description}</p>
             <p>Version: {version} Number of users: {usage}</p>
-            '''.format(title=theme.get('title', _('Unknown')), author=theme.get('author', _('Unknown')),
+            <p><i>Right click to visit theme homepage</i></p>
+            ''').format(title=theme.get('title', _('Unknown')), author=theme.get('author', _('Unknown')),
                        number=theme.get('number', 0), description=theme.get('description', ''),
                        size=human_readable(theme.get('compressed-size', 0)), version=theme.get('version', 1),
                        usage=theme.get('usage', 0),
-        )))
+        ))
         painter.drawStaticText(COVER_SIZE[0] + self.SPACING, option.rect.top() + self.SPACING, theme['static-text'])
         painter.restore()
 
@@ -592,8 +601,7 @@ class ChooseTheme(Dialog):
         self.dialog_closed = True
 
     def sizeHint(self):
-        desktop  = QApplication.instance().desktop()
-        h = desktop.availableGeometry(self).height()
+        h = self.screen().availableSize().height()
         return QSize(900, h - 75)
 
     def setup_ui(self):
@@ -645,12 +653,22 @@ class ChooseTheme(Dialog):
         self.delegate = Delegate(tl)
         tl.setItemDelegate(self.delegate)
         tl.itemDoubleClicked.connect(self.accept)
+        tl.itemPressed.connect(self.item_clicked)
         add_row(tl)
 
         t = Thread(name='GetIconThemes', target=self.get_themes)
         t.daemon = True
         t.start()
 
+    def item_clicked(self, item):
+        if QApplication.mouseButtons() & Qt.MouseButton.RightButton:
+            theme = item.data(Qt.ItemDataRole.UserRole) or {}
+            url = theme.get('url')
+            if url:
+                safe_open_url(url)
+            else:
+                error_dialog(self, _('No homepage'), _('The {} theme has no homepage').format(theme.get('name', _('Unknown'))), show=True)
+
     def start_spinner(self, msg=None):
         self.pi.startAnimation()
         self.stack.setCurrentIndex(0)
@@ -673,7 +691,7 @@ class ChooseTheme(Dialog):
             self.themes.sort(key=lambda x:self.usage.get(x.get('name'), 0), reverse=True)
         self.theme_list.clear()
         for theme in self.themes:
-            i = QListWidgetItem(theme.get('title', '') + ' %s %s' % (theme.get('number'), self.usage.get(theme.get('name'))), self.theme_list)
+            i = QListWidgetItem(theme.get('title', '') + ' {} {}'.format(theme.get('number'), self.usage.get(theme.get('name'))), self.theme_list)
             i.setData(Qt.ItemDataRole.UserRole, theme)
             if 'cover-pixmap' in theme:
                 i.setData(Qt.ItemDataRole.DecorationRole, theme['cover-pixmap'])
@@ -779,7 +797,7 @@ class ChooseTheme(Dialog):
         t = Thread(name='DownloadIconTheme', target=download)
         t.daemon = True
         t.start()
-        ret = d.exec_()
+        ret = d.exec()
 
         if self.downloaded_theme and not isinstance(self.downloaded_theme, BytesIO):
             return error_dialog(self, _('Download failed'), _(
@@ -861,6 +879,6 @@ if __name__ == '__main__':
     app = Application([])
     # create_theme('.')
     d = ChooseTheme()
-    if d.exec_() == QDialog.DialogCode.Accepted and d.commit_changes is not None:
+    if d.exec() == QDialog.DialogCode.Accepted and d.commit_changes is not None:
         d.commit_changes()
     del app
