@@ -1,4 +1,6 @@
-#!/usr/bin/env  python2
+#!/usr/bin/env python
+
+
 __license__   = 'GPL v3'
 __copyright__ = '2008, Kovid Goyal kovid@kovidgoyal.net'
 __docformat__ = 'restructuredtext en'
@@ -7,14 +9,13 @@ __docformat__ = 'restructuredtext en'
 Job management.
 '''
 
-import re, time
-from Queue import Empty, Queue
+import time
 
-from PyQt5.Qt import (QAbstractTableModel, QModelIndex, Qt, QPainter,
-    QTimer, pyqtSignal, QIcon, QDialog, QAbstractItemDelegate, QApplication,
+from qt.core import (QAbstractTableModel, QModelIndex, Qt, QStylePainter,
+    QTimer, pyqtSignal, QIcon, QDialog, QAbstractItemDelegate, QApplication, QEvent,
     QSize, QStyleOptionProgressBar, QStyle, QToolTip, QWidget, QStyleOption,
-    QHBoxLayout, QVBoxLayout, QSizePolicy, QLabel, QCoreApplication, QAction,
-    QByteArray, QSortFilterProxyModel, QTextBrowser, QPlainTextEdit)
+    QHBoxLayout, QVBoxLayout, QSizePolicy, QLabel, QCoreApplication, QAction, QItemSelectionModel,
+    QByteArray, QSortFilterProxyModel, QTextBrowser, QPlainTextEdit, QDialogButtonBox)
 
 from calibre import strftime
 from calibre.constants import islinux, isbsd
@@ -30,6 +31,7 @@ from calibre.gui2.threaded_jobs import ThreadedJobServer, ThreadedJob
 from calibre.gui2.widgets2 import Dialog
 from calibre.utils.search_query_parser import SearchQueryParser, ParseException
 from calibre.utils.icu import lower
+from polyglot.queue import Empty, Queue
 
 
 class AdaptSQP(SearchQueryParser):
@@ -72,13 +74,13 @@ class JobManager(QAbstractTableModel, AdaptSQP):  # {{{
 
         self.jobs          = []
         self.add_job       = Dispatcher(self._add_job)
-        self.server        = Server(limit=int(config['worker_limit']/2.0),
+        self.server        = Server(limit=config['worker_limit']//2,
                                 enforce_cpu_limit=config['enforce_cpu_limit'])
         self.threaded_server = ThreadedJobServer()
         self.changed_queue = Queue()
 
         self.timer         = QTimer(self)
-        self.timer.timeout.connect(self.update, type=Qt.QueuedConnection)
+        self.timer.timeout.connect(self.update, type=Qt.ConnectionType.QueuedConnection)
         self.timer.start(1000)
 
     def columnCount(self, parent=QModelIndex()):
@@ -88,9 +90,9 @@ class JobManager(QAbstractTableModel, AdaptSQP):  # {{{
         return len(self.jobs)
 
     def headerData(self, section, orientation, role):
-        if role != Qt.DisplayRole:
+        if role != Qt.ItemDataRole.DisplayRole:
             return None
-        if orientation == Qt.Horizontal:
+        if orientation == Qt.Orientation.Horizontal:
             return ({
               0: _('Job'),
               1: _('Status'),
@@ -126,12 +128,12 @@ class JobManager(QAbstractTableModel, AdaptSQP):  # {{{
 
     def data(self, index, role):
         try:
-            if role not in (Qt.DisplayRole, Qt.DecorationRole):
+            if role not in (Qt.ItemDataRole.DisplayRole, Qt.ItemDataRole.DecorationRole):
                 return None
             row, col = index.row(), index.column()
             job = self.jobs[row]
 
-            if role == Qt.DisplayRole:
+            if role == Qt.ItemDataRole.DisplayRole:
                 if col == 0:
                     desc = job.description
                     if not desc:
@@ -148,8 +150,8 @@ class JobManager(QAbstractTableModel, AdaptSQP):  # {{{
                         return None
                     return human_readable_interval(rtime)
                 if col == 4 and job.start_time is not None:
-                    return (strftime(u'%H:%M -- %d %b', time.localtime(job.start_time)))
-            if role == Qt.DecorationRole and col == 0:
+                    return (strftime('%H:%M -- %d %b', time.localtime(job.start_time)))
+            if role == Qt.ItemDataRole.DecorationRole and col == 0:
                 state = job.run_state
                 if state == job.WAITING:
                     return self.wait_icon
@@ -178,7 +180,7 @@ class JobManager(QAbstractTableModel, AdaptSQP):  # {{{
                 self.dataChanged.emit(idx, idx)
 
         # Update parallel jobs
-        jobs = set([])
+        jobs = set()
         while True:
             try:
                 jobs.add(self.server.changed_jobs_queue.get_nowait())
@@ -278,9 +280,9 @@ class JobManager(QAbstractTableModel, AdaptSQP):  # {{{
         self.add_job(job)
         self.threaded_server.add_job(job)
 
-    def launch_gui_app(self, name, args=[], kwargs={}, description=''):
+    def launch_gui_app(self, name, args=(), kwargs=None, description=''):
         job = ParallelJob(name, description, lambda x: x,
-                args=args, kwargs=kwargs)
+                args=list(args), kwargs=kwargs or {})
         self.server.run_job(job, gui=True, redirect_output=False)
 
     def _kill_job(self, job):
@@ -300,16 +302,16 @@ class JobManager(QAbstractTableModel, AdaptSQP):  # {{{
     def show_hidden_jobs(self):
         for j in self.jobs:
             j.hidden_in_gui = False
-        for r in xrange(len(self.jobs)):
+        for r in range(len(self.jobs)):
             self.dataChanged.emit(self.index(r, 0), self.index(r, 0))
 
     def kill_job(self, job, view):
         if isinstance(job, DeviceJob):
             return error_dialog(view, _('Cannot kill job'),
-                         _('Cannot kill jobs that communicate with the device')).exec_()
+                         _('Cannot kill jobs that communicate with the device')).exec()
         if job.duration is not None:
             return error_dialog(view, _('Cannot kill job'),
-                         _('Job has already run')).exec_()
+                         _('Job has already run')).exec()
         if not getattr(job, 'killable', True):
             return error_dialog(view, _('Cannot kill job'),
                     _('This job cannot be stopped'), show=True)
@@ -319,14 +321,14 @@ class JobManager(QAbstractTableModel, AdaptSQP):  # {{{
         devjobs = [j for j in jobs if isinstance(j, DeviceJob)]
         if devjobs:
             error_dialog(view, _('Cannot kill job'),
-                         _('Cannot kill jobs that communicate with the device')).exec_()
+                         _('Cannot kill jobs that communicate with the device')).exec()
             jobs = [j for j in jobs if not isinstance(j, DeviceJob)]
         jobs = [j for j in jobs if j.duration is None]
         unkillable = [j for j in jobs if not getattr(j, 'killable', True)]
         if unkillable:
-            names = u'\n'.join(as_unicode(j.description) for j in unkillable)
+            names = '\n'.join(as_unicode(j.description) for j in unkillable)
             error_dialog(view, _('Cannot kill job'),
-                    _('Some of the jobs cannot be stopped. Click Show details'
+                    _('Some of the jobs cannot be stopped. Click "Show details"'
                         ' to see the list of unstoppable jobs.'), det_msg=names,
                     show=True)
             jobs = [j for j in jobs if getattr(j, 'killable', True)]
@@ -424,12 +426,12 @@ class ProgressBarDelegate(QAbstractItemDelegate):  # {{{
         opts.maximum = 100
         opts.textVisible = True
         try:
-            percent = int(index.model().data(index, Qt.DisplayRole))
+            percent = int(index.model().data(index, Qt.ItemDataRole.DisplayRole))
         except (TypeError, ValueError):
             percent = 0
         opts.progress = percent
         opts.text = (_('Unavailable') if percent == 0 else '%d%%'%percent)
-        QApplication.style().drawControl(QStyle.CE_ProgressBar, opts, painter)
+        QApplication.style().drawControl(QStyle.ControlElement.CE_ProgressBar, opts, painter)
 # }}}
 
 
@@ -458,11 +460,11 @@ class DetailView(Dialog):  # {{{
             self.tb = w = QTextBrowser(self)
         else:
             self.log = w = QPlainTextEdit(self)
-            w.setReadOnly(True), w.setLineWrapMode(w.NoWrap)
+            w.setReadOnly(True), w.setLineWrapMode(QPlainTextEdit.LineWrapMode.NoWrap)
         l.addWidget(w)
         l.addWidget(self.bb)
-        self.bb.clear(), self.bb.setStandardButtons(self.bb.Close)
-        self.copy_button = b = self.bb.addButton(_('&Copy to clipboard'), self.bb.ActionRole)
+        self.bb.clear(), self.bb.setStandardButtons(QDialogButtonBox.StandardButton.Close)
+        self.copy_button = b = self.bb.addButton(_('&Copy to clipboard'), QDialogButtonBox.ButtonRole.ActionRole)
         b.setIcon(QIcon(I('edit-copy.png')))
         b.clicked.connect(self.copy_to_clipboard)
         self.next_pos = 0
@@ -497,34 +499,42 @@ class JobsButton(QWidget):  # {{{
 
     def __init__(self, parent=None):
         QWidget.__init__(self, parent)
+        self.num_jobs = 0
         self.mouse_over = False
-        self.pi = ProgressIndicator(self, self.style().pixelMetric(QStyle.PM_ToolBarIconSize))
-        self._jobs = QLabel('<b>'+_('Jobs:')+' 0 ')
+        self.pi = ProgressIndicator(self, self.style().pixelMetric(QStyle.PixelMetric.PM_ToolBarIconSize))
+        self.pi.setVisible(False)
+        self._jobs = QLabel('')
         self._jobs.mouseReleaseEvent = self.mouseReleaseEvent
+        self.update_label()
         self.shortcut = 'Alt+Shift+J'
 
         self.l = l = QHBoxLayout(self)
         l.setSpacing(3)
         l.addWidget(self.pi)
         l.addWidget(self._jobs)
-        m = self.style().pixelMetric(QStyle.PM_DefaultFrameWidth)
+        m = self.style().pixelMetric(QStyle.PixelMetric.PM_DefaultFrameWidth)
         self.layout().setContentsMargins(m, m, m, m)
-        self._jobs.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Minimum)
-        self.setCursor(Qt.PointingHandCursor)
+        self._jobs.setSizePolicy(QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Minimum)
+        self.setCursor(Qt.CursorShape.PointingHandCursor)
         b = _('Click to see list of jobs')
-        self.setToolTip(b + u' [%s]'%self.shortcut)
+        self.setToolTip(b + _(' [Alt+Shift+J]'))
         self.action_toggle = QAction(b, parent)
         parent.addAction(self.action_toggle)
         self.action_toggle.triggered.connect(self.toggle)
         if hasattr(parent, 'keyboard'):
             parent.keyboard.register_shortcut('toggle jobs list', _('Show/hide the Jobs List'), default_keys=(self.shortcut,), action=self.action_toggle)
 
+    def update_label(self):
+        n = self.jobs()
+        prefix = '<b>' if n > 0 else ''
+        self._jobs.setText(prefix + ' ' + _('Jobs:') + f' {n} ')
+
     def event(self, ev):
         m = None
         et = ev.type()
-        if et == ev.Enter:
+        if et == QEvent.Type.Enter:
             m = True
-        elif et == ev.Leave:
+        elif et == QEvent.Type.Leave:
             m = False
         if m is not None and m != self.mouse_over:
             self.mouse_over = m
@@ -552,13 +562,14 @@ class JobsButton(QWidget):  # {{{
 
     def start(self):
         self.pi.startAnimation()
+        self.pi.setVisible(True)
 
     def stop(self):
         self.pi.stopAnimation()
+        self.pi.setVisible(False)
 
     def jobs(self):
-        src = unicode(self._jobs.text())
-        return int(re.search(r'\d+', src).group())
+        return self.num_jobs
 
     def tray_tooltip(self, num=0):
         if num == 0:
@@ -572,20 +583,14 @@ class JobsButton(QWidget):  # {{{
         return text
 
     def job_added(self, nnum):
-        jobs = self._jobs
-        src = unicode(jobs.text())
-        num = self.jobs()
-        text = src.replace(str(num), str(nnum))
-        jobs.setText(text)
+        self.num_jobs = nnum
+        self.update_label()
         self.start()
         self.tray_tooltip_updated.emit(self.tray_tooltip(nnum))
 
     def job_done(self, nnum):
-        jobs = self._jobs
-        src = unicode(jobs.text())
-        num = self.jobs()
-        text = src.replace(str(num), str(nnum))
-        jobs.setText(text)
+        self.num_jobs = nnum
+        self.update_label()
         if nnum == 0:
             self.no_more_jobs()
         self.tray_tooltip_updated.emit(self.tray_tooltip(nnum))
@@ -597,12 +602,12 @@ class JobsButton(QWidget):  # {{{
 
     def paintEvent(self, ev):
         if self.mouse_over:
-            p = QPainter(self)
+            p = QStylePainter(self)
             tool = QStyleOption()
+            tool.initFrom(self)
             tool.rect = self.rect()
-            tool.state = QStyle.State_Raised | QStyle.State_Active | QStyle.State_MouseOver
-            s = self.style()
-            s.drawPrimitive(QStyle.PE_PanelButtonTool, tool, p, self)
+            tool.state = QStyle.StateFlag.State_Raised | QStyle.StateFlag.State_Active | QStyle.StateFlag.State_MouseOver
+            p.drawPrimitive(QStyle.PrimitiveElement.PE_PanelButtonTool, tool)
             p.end()
         QWidget.paintEvent(self, ev)
 
@@ -620,7 +625,7 @@ class JobsDialog(QDialog, Ui_JobsDialog):
         self.proxy_model.setSourceModel(self.model)
         self.proxy_model.search_done.connect(self.search.search_done)
         self.jobs_view.setModel(self.proxy_model)
-        self.setWindowModality(Qt.NonModal)
+        self.setWindowModality(Qt.WindowModality.NonModal)
         self.setWindowTitle(__appname__ + _(' - Jobs'))
         self.details_button.clicked.connect(self.show_details)
         self.kill_button.clicked.connect(self.kill_job)
@@ -640,8 +645,9 @@ class JobsDialog(QDialog, Ui_JobsDialog):
 
     def restore_state(self):
         try:
-            geom = gprefs.get('jobs_dialog_geometry', bytearray(''))
-            self.restoreGeometry(QByteArray(geom))
+            geom = gprefs.get('jobs_dialog_geometry', None)
+            if geom:
+                QApplication.instance().safe_restore_geometry(self, QByteArray(geom))
             state = gprefs.get('jobs view column layout3', None)
             if state is not None:
                 self.jobs_view.horizontalHeader().restoreState(QByteArray(state))
@@ -650,7 +656,7 @@ class JobsDialog(QDialog, Ui_JobsDialog):
         idx = self.jobs_view.model().index(0, 0)
         if idx.isValid():
             sm = self.jobs_view.selectionModel()
-            sm.select(idx, sm.ClearAndSelect|sm.Rows)
+            sm.select(idx, QItemSelectionModel.SelectionFlag.ClearAndSelect|QItemSelectionModel.SelectionFlag.Rows)
 
     def save_state(self):
         try:
@@ -667,7 +673,7 @@ class JobsDialog(QDialog, Ui_JobsDialog):
             row = index.row()
             job = self.model.row_to_job(row)
             d = DetailView(self, job)
-            d.exec_()
+            d.exec()
             d.timer.stop()
 
     def show_details(self, *args):
@@ -709,7 +715,7 @@ class JobsDialog(QDialog, Ui_JobsDialog):
         self.proxy_model.beginResetModel(), self.proxy_model.endResetModel()
 
     def hide_all(self, *args):
-        self.model.hide_jobs(list(xrange(0,
+        self.model.hide_jobs(list(range(0,
             self.model.rowCount(QModelIndex()))))
         self.proxy_model.beginResetModel(), self.proxy_model.endResetModel()
 
