@@ -1,5 +1,3 @@
-
-
 __license__   = 'GPL v3'
 __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
 
@@ -12,7 +10,6 @@ from qt.core import (
 from calibre.gui2.lrf_renderer.text import TextBlock, FontLoader, COLOR, PixmapItem
 from calibre.ebooks.lrf.objects import RuledLine as _RuledLine
 from calibre.ebooks.lrf.objects import Canvas as __Canvas
-from polyglot.builtins import unicode_type
 
 
 class Color(QColor):
@@ -28,7 +25,7 @@ class Pen(QPen):
                       (Qt.PenStyle.SolidLine if width > 0 else Qt.PenStyle.NoPen))
 
 
-class ContentObject(object):
+class ContentObject:
 
     has_content = True
 
@@ -156,7 +153,7 @@ class _Canvas(QGraphicsRectItem):
             max_width  = min(br.width(), self.max_x-x)
             if br.height() > max_height or br.width() > max_width:
                 p = ib.pixmap()
-                ib.setPixmap(p.scaled(max_width, max_height, Qt.AspectRatioMode.IgnoreAspectRatio,
+                ib.setPixmap(p.scaled(int(max_width), int(max_height), Qt.AspectRatioMode.IgnoreAspectRatio,
                                       Qt.TransformationMode.SmoothTransformation))
                 br = ib.boundingRect()
             ib.setParentItem(self)
@@ -300,7 +297,7 @@ class Page(_Canvas):
         self.layout_block(block, 0, self.current_y)
 
 
-class Chapter(object):
+class Chapter:
 
     num_of_pages = property(fget=lambda self: len(self.pages))
 
@@ -418,7 +415,7 @@ class Document(QGraphicsScene):
             fdata = QByteArray(lrf.font_map[font].data)
             id = QFontDatabase.addApplicationFontFromData(fdata)
             if id != -1:
-                font_map[font] = [unicode_type(i) for i in QFontDatabase.applicationFontFamilies(id)][0]
+                font_map[font] = [str(i) for i in QFontDatabase.applicationFontFamilies(id)][0]
 
         if load_substitutions:
             base = P('fonts/liberation/*.ttf')
