@@ -1,23 +1,22 @@
 #!/usr/bin/env python
-# vim:fileencoding=utf-8
 # License: GPL v3 Copyright: 2013, Kovid Goyal <kovid at kovidgoyal.net>
 
 
 import json
 from operator import itemgetter
 from qt.core import (
-    QAction, QComboBox, QGridLayout, QHBoxLayout, QIcon, QInputDialog,
-    QItemSelectionModel, QLabel, QListWidget, QListWidgetItem, QPushButton, Qt,
-    QWidget, pyqtSignal
+    QAbstractItemView, QAction, QComboBox, QGridLayout, QHBoxLayout, QIcon,
+    QInputDialog, QItemSelectionModel, QLabel, QListWidget, QListWidgetItem,
+    QPushButton, Qt, QWidget, pyqtSignal
 )
 
 from calibre.gui2 import choose_files, choose_save_file
 from calibre.gui2.dialogs.confirm_delete import confirm
+from calibre.gui2.gestures import GestureManager
 from calibre.gui2.viewer.shortcuts import get_shortcut_for
 from calibre.gui2.viewer.web_view import vprefs
 from calibre.utils.date import EPOCH, utcnow
 from calibre.utils.icu import primary_sort_key
-from polyglot.builtins import range, unicode_type
 
 
 class BookmarksList(QListWidget):
@@ -34,6 +33,17 @@ class BookmarksList(QListWidget):
         self.addAction(ac)
         self.ac_delete = ac = QAction(QIcon(I('trash.png')), _('Remove this bookmark'), self)
         self.addAction(ac)
+        self.gesture_manager = GestureManager(self)
+        self.setVerticalScrollMode(QAbstractItemView.ScrollMode.ScrollPerPixel)
+
+    def viewportEvent(self, ev):
+        try:
+            ret = self.gesture_manager.handle_event(ev)
+        except AttributeError:
+            ret = None
+        if ret is not None:
+            return ret
+        return super().viewportEvent(ev)
 
     @property
     def current_non_removed_item(self):
@@ -236,12 +246,12 @@ class BookmarkManager(QWidget):
         q = base
         while q in all_titles:
             c += 1
-            q = '{} #{}'.format(base, c)
+            q = f'{base} #{c}'
         return q
 
     def item_changed(self, item):
         self.bookmarks_list.blockSignals(True)
-        title = unicode_type(item.data(Qt.ItemDataRole.DisplayRole)) or _('Unknown')
+        title = str(item.data(Qt.ItemDataRole.DisplayRole)) or _('Unknown')
         title = self.uniqify_bookmark_title(title)
         item.setData(Qt.ItemDataRole.DisplayRole, title)
         item.setData(Qt.ItemDataRole.ToolTipRole, title)
@@ -346,13 +356,13 @@ class BookmarkManager(QWidget):
         c = 0
         while True:
             c += 1
-            default_title = '{} #{}'.format(base_default_title, c)
+            default_title = f'{base_default_title} #{c}'
             if default_title not in all_titles:
                 break
 
         title, ok = QInputDialog.getText(self, _('Add bookmark'),
                 _('Enter title for bookmark:'), text=pos_data.get('selected_text') or default_title)
-        title = unicode_type(title).strip()
+        title = str(title).strip()
         if not ok or not title:
             return
         title = self.uniqify_bookmark_title(title)
