@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright (C) 2006-2008 Søren Roug, European Environment Agency
 #
 # This library is free software; you can redistribute it and/or
@@ -98,7 +97,7 @@ class TextProps:
 
     def __unicode__(self):
 
-        return "[italic=%s, bold=i%s, fixed=%s]" % (unicode_type(self.italic),
+        return "[italic={}, bold=i{}, fixed={}]".format(unicode_type(self.italic),
                                           unicode_type(self.bold),
                                           unicode_type(self.fixed))
     __str__ = __unicode__
@@ -145,7 +144,7 @@ class ListProperties:
         self.ordered = value
 
 
-class ODF2MoinMoin(object):
+class ODF2MoinMoin:
 
     def __init__(self, filepath):
         self.footnotes = []
@@ -335,7 +334,7 @@ class ODF2MoinMoin(object):
         if link.strip() == text.strip():
             return "[%s] " % link.strip()
         else:
-            return "[%s %s] " % (link.strip(), text.strip())
+            return f"[{link.strip()} {text.strip()}] "
 
     def text_line_break(self, node):
         return "[[BR]]"
@@ -386,7 +385,7 @@ class ODF2MoinMoin(object):
                 mark.append(",,")
         revmark = mark[:]
         revmark.reverse()
-        return "%s%s%s" % (''.join(mark), text, ''.join(revmark))
+        return "{}{}{}".format(''.join(mark), text, ''.join(revmark))
 
 # -----------------------------------
     def listToString(self, listElement, indent=0):
@@ -469,7 +468,7 @@ class ODF2MoinMoin(object):
 
             buffer.append("----")
             for cite, body in self.footnotes:
-                buffer.append("%s: %s" % (cite, body))
+                buffer.append(f"{cite}: {body}")
 
         buffer.append("")
         return self.compressCodeBlocks('\n'.join(buffer))
